"""
Contains embedded image and icon resources for Skyperious. Auto-generated.

------------------------------------------------------------------------------
This file is part of Skyperious - a Skype database viewer and merger.
Released under the MIT License.

@author      Erki Suurjaak
@created     07.02.2012
@modified    28.04.2015
------------------------------------------------------------------------------
"""
try:
    import wx
    from wx.lib.embeddedimage import PyEmbeddedImage
except ImportError:
    class PyEmbeddedImage(object):
        """Data stand-in for wx.lib.embeddedimage.PyEmbeddedImage."""
        def __init__(self, data):
            self.data = data


"""Returns the application icon bundle, for several sizes and colour depths."""
def get_appicons():
    icons = wx.IconBundle()
    [icons.AddIcon(i.Icon) for i in [
        Icon16x16_32bit, Icon16x16_8bit, Icon24x24_32bit, Icon24x24_8bit,
        Icon32x32_32bit, Icon32x32_8bit, Icon40x40_32bit, Icon40x40_8bit,
        Icon48x48_32bit, Icon48x48_8bit, Icon64x64_32bit, Icon64x64_8bit
    ]]
    return icons


"""Skyperious application 16x16 icon, 32-bit colour."""
Icon16x16_32bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAIAAACQkWg2AAAAB3RJTUUH3QMaFiYCLwtVsAAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAAEhSURBVHjaY/z//z8DKYAF"
    "mXP07Z/WWz9vf/mHLJgkz1qpxgHnMsJtePztn9nBLy9+oltoIsB8yp6HkRHKZYJLrHj6C1M1"
    "EJz98PftL4Q4wkkvfqCofuzOK8PJhKkfIWQuxIwsUXntx5c///Fp8BZnNeRHcFc++T37wa/f"
    "//7j1MDNwjjTgFOYDeq73/8ZWm7+3PHyD04NQGAqyDLfiFOIFarn3e//See/n/3wB6cGIPAQ"
    "Y6nTYGeHCb/59T/r4o8ff//j1MDKxJgizxYjywoXOfX+73Ykh2EJOKBn6tQ5kEWOvEVogMYD"
    "MLr77v4sufKDARuQ4GBEtwEY834SLFhVy3AwRsqwodsABMrczA4izE++owS8Pj9TmSo7cpQz"
    "kpq8AZv/aJiCXJ+bAAAAAElFTkSuQmCC"
)


"""Skyperious application 16x16 icon, 8-bit colour."""
Icon16x16_8bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAAoLQ9TAAAAB3RJTUUH3QMaFiYSMrxF1AAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAACEUExURf////3+/0DD8yS6"
    "8lPJ9Mnu+9rz/ACv8By38eP2/f7+/4zb+Ira9+H1/XLS9vT7/mPO9a7l+vr9/pDc+Aex8DbA"
    "8/f8/rLn+hK08RS18czv+8ru+xO18bLm+jG+8gWw8I7b+Knk+VzM9fP7/gGv8Ov4/uL1/Ri2"
    "8TS/88Ts+0nG9IbZ91Ok98QAAABxSURBVHjabc9HEoAwCAVQe+y9917vfz+D4iQLWeW/DAwI"
    "wl8dpwx1fXndyFO7iWC/mcwLggvJ4EYMAOPEwGspdH3IpKqp5A3XlOVUipJBGMVUkpQb48Pg"
    "gGtyANz3bVq4GLHxV8Osf8uJ0nObov5efgN2BQW1nHoFYgAAAABJRU5ErkJggg=="
)


"""Skyperious application 24x24 icon, 32-bit colour."""
Icon24x24_32bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAABtklEQVRIx9WVT08TURTFf+/N"
    "9N9QiHFDamoEAyysKxbExC/AsomJm66JYaMS+QpESfgEJLBw5wZISNi5cEfKwhg1YnVhjJRI"
    "SIgdoe10Zq5rOn2lw6QL73LOyz33vXPPGUTkmQyvnisREYZYmiGXbQK8EKpnPi9rHnu/O8YG"
    "TybSrN3PkrdVPIL9M5+F901qf8O+E35qBJy0xUhgfKL1H96VzQE+NkKO2xL/iXbqfs/vD25a"
    "ZPTlaU9aIWDFIwgMy/V61mE6r5NvUWms90QrtTYXgSQneHo3jWNFhXtz1GHlq8f5gCRGgvnx"
    "FOWCTfdytAJh86fHtkGj7jI6WYAvbsDjgyafG0EEL+Y0bx+OMHOFHkZUAfdGLbbmHIq56LFf"
    "zZBH1QsO3RBJEhUzec1qKUshGz166Aa8+tbitI8PBtq3csFmcTLNSJcgvsBW3efdqZ+MwLEU"
    "y1MZKsVUBHN9Ye17O3ma5ix4MZXpiX34E8SLilCgXD3H7QpRk8EspeIRaAVjlmL3uDPQ7cq3"
    "7OsZTQ3QvDSqWZxMXyeLNHec/hJVbqfYmHWYu2HHd/J/80/WwNIQ+y/9A4Hd7fikpHoZAAAA"
    "AElFTkSuQmCC"
)


"""Skyperious application 24x24 icon, 8-bit colour."""
Icon24x24_8bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAMAAADXqc3KAAAAB3RJTUUH3gQUEBkMQIv3tAAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAAMAUExURQAAAACt9wC19wi1"
    "9xi19xi99yG99ym99zG99zHG9znG90LG90rG90rO91rO92PO93vW/4Te/6Xn/63n/8bv/87v"
    "/873/9b3/973/+////f/////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "/zk5v24AAAABdFJOUwBA5thmAAAAjklEQVR42o2SWQ6AIAxEIRAxalwQUy33v6dLXVKo0X7O"
    "K2lnilJRLPWib+QDYGg0VbcwENyp62pmoL10bQIDhrrrrUYBQD68pMGYAX88MT2mYKHp1mcG"
    "gfa1kIII9iAOUhC9lZzvqQy0wJiCiB25zNMFyoSBZg+jFMATom6581t3PN2puPoD/jutAN6+"
    "zwrAjS3DagoRyQAAAABJRU5ErkJggg=="
)


"""Skyperious application 32x32 icon, 32-bit colour."""
Icon32x32_32bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAALGPC/xhBQAAAAlw"
    "SFlzAAALEQAACxEBf2RfkQAAAAd0SU1FB90DGhY0Alf/JWMAAAAadEVYdFNvZnR3YXJlAFBh"
    "aW50Lk5FVCB2My41LjEwMPRyoQAAAYFJREFUWEftlr1KxFAUhG9h4QNYWFpY+gg+hoWFhYWP"
    "ZOFDCIog/qAgYiOuYmFhoWih4B8Ys8m6utk97gQWUuxmJoeVNHsgVW7O+e5kcifBzEL/qqtC"
    "ncPzTU8AZAVu4swWTxMLW5F0PaZdyVcSwMbDjzS0CIdnlKIA11FWeThAls5TZT434UojdQFM"
    "70SW9TgDVWBm98sFABXOPjJKQAGY6W6bmtlGkVAApoCyyzIZKADMxFR4+varQAFO3jsUYOG4"
    "aW0nAwWAfIoKq5ctarhhCySApP89zR3EVIn1e+3wKYJIAHjgLulSAHjl6K1TSQkZAF3RnBkS"
    "99UccMWxkgvzh7HhtSlVSYFBw7WrFlUCa5RyAWBzSjQjyFi5AND0pd0zBE6ZJxBkrNwAaNz4"
    "LI9qHOOs/hUA6rAqBcC7nt3zx/FYFFAcP8oHY/HA/qt2+AyDwI8sK+oBpNzUtvYnXITYfP5l"
    "s/P7FACLlDQcDF++SPPcUEsCUJt51k0AoECdKoQ/7vOGvN+DWQIAAAAASUVORK5CYII="
)


"""Skyperious application 32x32 icon, 8-bit colour."""
Icon32x32_8bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAMAAABEpIrGAAAABGdBTUEAALGPC/xhBQAAAwBQ"
    "TFRFAAAAAK/wBLDwCLLwFLXxGLfxILnyJLryLL3zPML0VMn1WMv1XMz1YM32ZM72lN35mN/5"
    "rOX6uOn7vOr7wOv7xOz8zO/80PD81PL82PP93PT94PX95Pf96Pj+9Pz++P3//v7/AAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
    "AAAAAAAAAAAAAAAAAAAA6f5MRgAAAQB0Uk5T////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////////////////////////////////////////////////////////////"
    "////////////////AFP3ByUAAAAJcEhZcwAACxEAAAsRAX9kX5EAAAAHdElNRQfdAxoWMzY5"
    "CkcRAAAAGnRFWHRTb2Z0d2FyZQBQYWludC5ORVQgdjMuNS4xMDD0cqEAAADDSURBVDhPrZNb"
    "FoIwDAWJ4AsRUVQQFbr/TeLxQG4TwuOH/tEMN20GgtbNrjaYrzu3KlAeCeuNzr5F5stEmQUK"
    "WafYAokCQgtECqAHEzgD11+DwQDgBLzagwBijvjoCAB3Bg6NIvwcEHGeAOodZ1wkIWRVuGgu"
    "CGkzH1GhdUPHvh6R9d9KOSOdAByMF2bU3cY37DOSCcA9eyBaAsgAW63bJuACHWjPcNMJpWnR"
    "bCRxHZkDbBKdKi9Duhh8bN3jCsDS7/8DqHNjgvHE59IAAAAASUVORK5CYII="
)


"""Skyperious application 40x40 icon, 32-bit colour."""
Icon40x40_32bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAADX0lEQVRYw82YT2gUVxzHP29m"
    "drO747JGGm2jRJsEQWMjaaWKnoqXaqmCpId6qAjWSxG9eG6hhULx0n8UKoXS2l4ULNRLtbai"
    "lyiKEomI0kSSVEuySU2is3/mz6+HZG92583uMO4Phrn83pvP+/1+7/t+b5SILAMGgYPAaqCH"
    "52ujwATwLXDWAvYDnwAdtIZ1Lz3rAaVE5DrwGq1pQxbQGXWUJ3Bn3md43me8JMxWhbIvBCHj"
    "Dq1N8+pyM8qn1ljAS7rebgBfjVX4/K8q406ARFzYMks1BKhl866w75rDxWmv4Xz9PuVBX7Qx"
    "xtK7bjAEeP9WqSk4gJEFn38qgY6r1JhqgKqe9x/THqf/dpuu+GoAl4u+jquqMRk63t+MVSPX"
    "2/9Z1CwYOhvj8owXm278WfQIIqw2dJPMeYsyopuXdMiSJ0vCAyeg2zbiAQwErRXnTMWFHTk2"
    "5U2UCveNLYJ5a1G/Frz6lFkTBgom2Qgfj6UGs6ZicyE8HTNV4cO7ldjPOq1CeKczpTXZifsV"
    "vh6tJA94oCvNyrbw1Alw7HaZMw/dZAELKcVnfRl0qssTOHCjxJWYpMnQdXyvK83hdWktX8cX"
    "9l11GJn3kwNUwJf9Wfa+qNdfFKvC20MOE6UgGUCAlAGntuTYvkKvZRpzAvYMOTx2JRnAWk/3"
    "y1abjXm9obfmfAavOZR8SQYQoKNNcW6bzeqMnihfnPY4dLNEI4xGo6F/2Tb4dZtNe0oP8udJ"
    "l4/ulpMDBBhYbnL69RxZzS7+03sVzk95yQEC7Oyw+G4gh84R7AscGS5FqsemAQHeXZPixCY9"
    "Ib/3JODUhJssIMDRnjaO9OgJ+ffj1eQBFfDxhgwZjRmvP/Z5qpnm8IYVqGhMJsDVWR8dTa4G"
    "8Kgs9NqqecA5V+g9v0A5CAcs+6J9uRJNx1DA9pSiv2ByqRjfxSltQKemyGvV4K5VFnHa1nYT"
    "22phwA+62xrexc+sjL68ydpcPBv+rVUWg/WvEFIP8JlxNxS8ubL5KL7xgsWPW0JPHdWQDu5u"
    "MM0K6LUNvujP8Nt2/eaiFk0lIq6u3BwfKde9xKulptY2FSvSiq6sweaCwYa8SYPX5QklIsPA"
    "K7Sm3TBY/Jv+bwvCFYGTFvDD0s45CPQDqecMNglMASeBn/4Da4ke3UlVsiQAAAAASUVORK5C"
    "YII="
)


"""Skyperious application 40x40 icon, 8-bit colour."""
Icon40x40_8bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAMAAAC7IEhfAAAAB3RJTUUH3wQcFAAJogD5tQAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAAAwUExURf////7//+39/2PO"
    "93TW94je/6fn/8zv/wCt9ya995fe/0rI9znG99z3/7nr/2vW94Q4YkwAAAABdFJOUwBA5thm"
    "AAAA1ElEQVR42o3V2xKFIAgFUMUbVtb//23nWpg7Yr81s8aGAHPOeUOcs7m3tLmX7B9LTnEL"
    "v5RbmCrLxBtIM/epN/DqmAnCdXCcIRwP5ABhHWFFkBik6bDQP/DVJ3zozII/9Ain88hJhSTK"
    "zhqUR3LRoA+a7KZHNKc2DdIiJCmwK2jWoG9CBg36Ypjwb+RYrhr0+WHMziTYS7TX2wEXHYrR"
    "1KGop40QrgKCaAvxqxNwsJgCYEYQLLacCwHD4PC6yt59+tevmLz2wpGY8nUX7Fez+bK3/j52"
    "YpAULg8lhSQAAAAASUVORK5CYII="
)


"""Skyperious application 48x48 icon, 32-bit colour."""
Icon48x48_32bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAYAAAEgBckRAAAAB3RJTUUH3QYHESErTSRNrAAA"
    "AAlwSFlzAAAK8AAACvABQqw0mAAAAARnQU1BAACxjwv8YQUAAAUgSURBVHjazVldbFRFFD73"
    "7m677e62oBDBGEQgioY2xAS1kSrrP4oJD2rCi+FBfTAaNfHBBxPlyUQf1PBioom+mGhiAkYN"
    "4A8iUEoEUwQTaqLBCrE0qP3b7W7pdq/zXTrXu3dn7p2zvUv5ksndvTPnb+bMOWfmWo7jtBNR"
    "kQKwRIfj/tg97r10tnaS7R/14upW9+Wl3jmUKoJy15j863gU6QTRqze1eiyTnjCfDCl8VDwX"
    "BZQ6go5q76GidfifiqcRHpARfOlysv0v/HI8rWTn5v6iZ0f1tdMl1wafHVUInxD9OUn5W7FK"
    "9/cV6Y+pKgW5Sfu8wUEbVfAmXUV06p4sretIhBMAx8dmacOBglKlGgKdSmtzNp2+N+cRXBDP"
    "JfhTFbS2Zf0/EqysALUg+NQxR1edDduOT9En52aU+kuVqlKwyoZNS5L0/caMVNBdB4tCcODv"
    "isdcEigRVEXC1hHopriGoM5ABVGdhCDRDV9PRqvkJ/J7bZ3RJt4KCU7YgGVpb9YvhRKVt0rU"
    "+dYcwT7xfCBSl8ZwUWqk8mPKfTlBhYoTyaX/rizdcVVC1eVoTTaZ4BpOGs9LsriEMNJB69qP"
    "XZtSvodl/u0bhVCvkAx1mNzSQdlk6OaOFgCcLzu0fO9EQ9NmJEDis79m6PEfp5R9o4900KJU"
    "vTV2FFM/sC7Isios/kptodaLuG56qDfDnyITIS+vaaW31qX1AwT/J5EYGKmHgz4IyIvWHzPj"
    "P0V7U7QEpghpKBE5F43h3Ug3fef3afr47AydGJ8lVdzbujxFu25v15GP1CRmP1CYiarNSE3N"
    "RnMjtO37UwNT5jqZ8oc9p32NBU8PlFjc9o5Ugq88nso1SH0+TgZ5xsP6zgQN5LPKPuVORoQc"
    "m6mXUHq00y3KOVDGohdWtygHt33BCx/aKXI7QsIEJ6tpo+mHt7ZpiVAMzFvA9hUt9Nwq9VSh"
    "0rj5u0kyQWg+2NndRhuvVkf0wcmqW17PSwCAOL8mox6G5P/emYuh9MYpM6wIC0v+xikTTHRY"
    "uke/6KycrHPP8qyehiVAtbujoHQRlCYoUeKA0gK4Z1yQJ5Aa232nDiOcfTAXfBWeD0yFwDVR"
    "0V3XVjcRkriq3AfylWXphZiMETgJAWfEj5W8mTUC0lwvNHlCtMEmFl/NwLRo+0TrxgwNCUtW"
    "NGGGLgd+Zh0//MANwEunyrR7eP77HccWOHsjkAbI2jFyayGNbfihYHQ65+CNW9L0yo2tJkP9"
    "gi3WCuCyKX94XvWqFijmRh+OqajT4fmT5aYoDyAh7viVz591DfLLxCxnuHvufOr6FuXRXIVG"
    "vJK1ArryTochsdFNlQeSvAzqgrUHEHmCF5YmwErgJqAZYIdRGNG1nx+FMLvf3Jlxr6kX1ACJ"
    "1wfLtGNwmk2HImhgU45dccVuAIBF2HasseIV+wkfDBrx+9gMkIA79RwssqMUEHlLdjkMkECW"
    "zvcV3KtVDrAKWA1ulIvdAIlvL1Ro85EiO66jlEBJseAGSOCCDwUfB293pd3PvFeEAQD2B+c2"
    "BtFp+CHzytTY6eAOuFCPuwoNgrt/jEsJbLSd3Y1HC1Pg2osDvwvV1Nk6gp6DBTr6Lz9cmuC+"
    "pUk3W2ug1M//WTFUcYngJ/s4sLLdpj09GfdrPhesTSyHPnOiRB8MmWdfuB/uRnBYgbJrszbd"
    "tjhBW65JUi7FV9qHYRiwX/y4m5il9RUAXKt8BANww58X7VnR1ou2bKE1C8F50UZE+0m09/H8"
    "D5GjATh9KtVTAAAAAElFTkSuQmCC"
)


"""Skyperious application 48x48 icon, 8-bit colour."""
Icon48x48_8bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAMAAAEX2zkjAAAAB3RJTUUH3QYHESIrZgkebwAA"
    "AAlwSFlzAAAK8AAACvABQqw0mAAAAARnQU1BAACxjwv8YQUAAAC0UExURQAAAP////7+/v3+"
    "/v/+/2TO9QCv8ASw8CC58ii88kjG9HDS9qbj+fL6/Ri38Qyz8XTT9+/6/svv/Nfy/X/X9+P2"
    "/TjB80TE9Of3/sPs+5/h+WDN9lTJ9fv+/3zW9/P7/o/c+Kvl+giy8Lvq+3jV9yS68pfe+Sy9"
    "8/f8/zzC9FjL9WzR9tPx/N/1/b/r+6fj+ovb+HDS9xC08TC+82TO9rfo+0DD9IfZ+Mft/FDI"
    "9dv0/TS/88xqrdsAAAABdFJOUwBA5thmAAABrUlEQVR42pWUDVuCQAzHOayJViBqpSZh9iaa"
    "ZvZqff/vFdxxsHE3wT2Pwu63P9ttB44jMkv/AUBe3qS3AHkRt4CYikxAsuxfMRUJ3fQy1J74"
    "9UBHZPLSybwXoR3Rr4Z1lNNKfydlWJ5cOkslKTVftjx5UEkKp/DUo1PHK/MwVct7H5XTqtZG"
    "TIMAtP0QAMgswPKoqSKvZg79sNgAYkRSkHLvFAkNkI4caLtouWOiKJZ7ZES2ljjMcr1g9teW"
    "OfZ0H+NqDy3lEvCBwCcGbQTeMdjJpYU5jyE7KDBIDiYKBOYEV8XJrw4qUWRpjvZRkSdztAE5"
    "Dri7CsxNQI4dBqEJptzZHXEAz+OSAF8vByHZh+vmbnEj+FPCvoSc4Lh4I/V6D1XbHRAMArDY"
    "NyvYgNXmC07wbBfAAyfAQduwXI84QYIEqyZdWpNCtg3auiZdam8azGFIkvijWoGIyNmFJKoT"
    "CBHT7vZqBenAfbKVca1AiHv8AeIHh22GFbMGAhHjbtkFkfWsohfayDDh4m+4kq7s8dfspvu2"
    "cG/AttV1T3H/vU737PyiftKHzTnyO+M4/8CIIdP02kirAAAAAElFTkSuQmCC"
)


"""Skyperious application 64x64 icon, 32-bit colour."""
Icon64x64_32bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAYAAAHdbkFIAAAAB3RJTUUH3QYHETMOftTpOAAA"
    "AAlwSFlzAAAK8AAACvABQqw0mAAAAARnQU1BAACxjwv8YQUAAAaKSURBVHja1ZtdbFRFFIDP"
    "3W3LdrdbiZYoBrBUUSuCEuoP2FaiqPhDRKMJD5r4IqCBqA9Gjb4YHjTRqAkmYPBBowkmmoiB"
    "1J+iglsCCkYLRhpBC7YREqqy3f7R3e51zoW7nbt75//e3foly7LduTNnzpw5c+bMrAUEmwAM"
    "LPpLa3u68IW96gLnPeL3JU2hQCwKJU97CoyunPxj9Wfp0gL0kzlK5CppGf69t77wx6uTVMWk"
    "lx02m5cLeqCb+PSmOKyaWV0qpMsD34+UCumnA08N7hfFvfHVA11IXg9MOUgvU7YBHnvhicLS"
    "QhUIKH6gmBIJWFKwKvKtgFXR+qYa2LSwVr4Cv0pYOsBKLBkllvbNtt83GUasoEnz2Q4pHYgo"
    "sQOeDvyG0mPrx0fyyhJ4Kpj7VYZb+K3fz/K78NicGnjvz/HC5xUXV8HnSxLcSoWTKdWWgNaL"
    "2FOmZL4fuT3p+dyWGvZ4UmEF6DQ3Nsc8f6N9tbALLs1fZ6An4x0V4TDyuoL4jRKzAr8W/ezE"
    "o963/xiHDYdGQQUln9h7ZxIa45NrARYX+kQXXEHpZRZc/2E6G7GCheS926QSA45I9+DZX8bg"
    "9WNnZYo6iJYTWgMoQMEnF1O7Iw1jE+pdwxnlZ8u0jNimaxK+ja/cP6zVOFI8C/06j/9wrXD3"
    "AL91H8tUhjuTT67gqtAZHj8nrYLQCNGV8rzhpLD1cEnMEpZTFsBl56mcYxMiZK1fWQCXh38Y"
    "gU/+ynLLzKqNQN9dSan6tD1ZcucgDOX4j8oYqbkrVQyIi+FOQ10npAJ3GtJ7OB2qJCaFKwDT"
    "bT0yu1qrcQzks/eLOyC0AfzWkp3epKxtMfw6SwOW5VQ/wJRQpTbFxgmrz/fS3kheO0w2Gopg"
    "W3PcIegl741qggdGk7EfMGSPlACnxmxY8E0GBsblZL35wijsa6+TKhsRFXjh1zGY+cWgdOPI"
    "/n/kvZcrgG/tx4bz8OpvZus9AzcMBG5I1rwrI1thCfRG0QfLbZM7BDkD88Shk0FoA7qg4cog"
    "vTXyQzX6CVwDJkMkJUCdYD2VCVaNBDi6XBzXYWgWmgAYZi9r4JsJxoXzOvWnq9AGvm1NCON9"
    "dFh0vjhQARDcdIjCq+0ns1peU2k1lMlmijJbRgKEsU1T8gM4DKfvqReWw9VTFmVH1FBjweHb"
    "xGu97MzQ8oTX1kfh4xvj3DI4M2Q8pbYrfujSalg9i79nwLySCKOYEJ0QzxPicIlshjlfRJXL"
    "IBPGMYdAtBDJsHyG2B9wbcBz4qfB1kW1wjJ0CyX62nq9uAIWqEEqMUzjaYcuUaJzFZdKg6OX"
    "uY9pfJ52hDpWNYU3F8SktuXnyXOnYdhbc8IergZC3pojPSjAGvXnAqHPsqx1jtBkFB4kb2+Q"
    "12UVEqbc9JDXGqKAFNrgFvJhbaUlqhCvoZ9pCbpWjAK6/s7BwTMT0J2egP5R2zmyG8vb0jtG"
    "P56/chq8ck1M+3kflgaWIPqwL+scOZ7Jhptvwrsx06vNl2kX2guqn74T3j0x7uwVHv1xJPTO"
    "I7JJDwGFvvpZgPtZqGbcCuJurNwUnT/LwOyTXy0WSHQet6CV6DyiYQXMPmmHm+8cH9d91JiP"
    "+rPwczqYQyTt9JzOhR2adXNr4EXi1fFsUYcgMnOItgXgxtiEp5r0O48EsF8zU8Bz86YZNYwX"
    "jJ45HIhHN8IoDsAOmN5awJHcdkPcyXL87xSA4HUemfSLCMygpNrqjNMAZVcAguFtW2rISYaZ"
    "gqd8nUsTgSRlyqYAF4z/7943IrzMIQOuEpuv08/JVEQBLhgeP/6T2lVEFtta4sIM6JRTgMsT"
    "3aOwpdc8YMLpcODWcPxD6NcFcDrgtMDpYQr6B3SUQbqHst2XwMhxyXdDRvkAFzy3Ex0eTjkF"
    "uOweyMEde4eNQ1lMvz59uVkwVhEFuGAAZRoJdt6SkDp/mJIKQPpH8zD7S/0z7pbpUTiwTO5m"
    "EIvyhl0U6BM2HDKzAMw5mqJtPziPd5029+wmmOwmXbRr2GxwchYUaxtrKqeAKxIRJ01dKXD3"
    "+NJV4awC0klRXMpmdAyWJRtM88HiuOql4uCToghGZMU/NwwLnO/4CzW8Jadxo5rZJ6NlME8e"
    "xVrbu8xCXbzWE4tYTk4A/48p7/nJKLQ3RKG5LgKW0jGtEs5vZA+S/ywOq4Upzl6cAk+SV1+l"
    "JakAJ8hrfcG2iCUsgnOnxPPJq7XS0oVAF5w7EjtKXpvItHJ+QvgfUYX3p2LQMIkAAAAASUVO"
    "RK5CYII="
)


"""Skyperious application 64x64 icon, 8-bit colour."""
Icon64x64_8bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAMAAAHqsLF6AAAAB3RJTUUH3QYHES0v5vzGuQAA"
    "AAlwSFlzAAAK8AAACvABQqw0mAAAAARnQU1BAACxjwv8YQUAAADDUExURQAAAP////7+///+"
    "//7///3+/v7+/jzC9ACv8Bi38SC58jS/80zH9GzR9pfe+dfy/e/6/gSw8Ci88o/c+Pf8/6Pi"
    "+hy48rfo+1zM9ZPd+fv+/xC08cPs+wyz8d/1/XjV91TJ9dv0/YPY+ETE9PP7/giy8FDI9RS1"
    "8eP2/VjL9XDS97vq+6vl+r/r+9Px/CS68uv5/jjB8+f3/mjQ9pvg+afj+rPn+4fZ+EDD9Ivb"
    "+GDN9kjG9Cy988/w/J/h+a/m+nTT9/c0HisAAAABdFJOUwBA5thmAAACTklEQVR42rWWaVvi"
    "MBCAO147RV0sLB6VFg8QZcUDvN1d/f+/SkqbJpnJNOTx2flCmbx5m5mmgSiCMopPRFxeoLoY"
    "FZ/VUFJeAGX2EDvVUMGWQx82s9RUHhA8qIfMqNgyrIRFAOqUchSZXTMBzGElrNtGzd9tR53I"
    "aULdpKsSb4sv78zRNxPtIpOatZyx4jrWXZSmJVZ75GzhInKS+GVJG+p3taOMT9ThAEZoRocB"
    "O2gHAzI0l+pYw9DqumuRaW2fSVU8OWogjTlRxL4A6LXqxbJG0tsoYIRCIyI6s4qEAYfW+BFf"
    "5KaxDvPa97gjD+AbX2FDGY2ajXUR9w5g4N4vNXCKHiCxgX8MIJ2OfQCKgFimAlIJyCSFAmYK"
    "yKROvijiVmp1rIgLAdC9ehYA3Yu+ADjfPms/3DUdIMu44aWQHXXNOkq33FUFTCRAHSRjAtQH"
    "jPhu0vFzCnQIkFOgb4/rZ+p+/RPggLmtzaOwBraM5JYLkGKF88Hr+J/zxRWm09fDl1ZMmj9w"
    "Gniqd4xC7K0ieEA55isILrApcq/guXE+XnkFrWYB/vUJWNWX5PH4BH+IoA2+IIIeXcIkUAAT"
    "akiuwwTwyho3bgcJYHbLFPfDEMHifMmYYh4kcO7nxyABwJwZsnaQAIY/eCvSEMHiX3vMFNMg"
    "AcA0oYZumACgSw2/AwVwSQTbgYKcHTAewbnnPDD/GDsFpz7Bma+EQfP8E+9TSI+b5vfAK+BH"
    "ii7/wNVm/hToFo5b2zu7P8UzgQnW1lzY+vrGhiD49o/rd36fi8lfI7xBk5tF2NYAAAAASUVO"
    "RK5CYII="
)


"""Skyperious application 256x256 icon, 32-bit colour."""
Icon256x256_32bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAQAAAAEACAYAAABccqhmAAAXqklEQVR42u2deZQdV33nP7eq"
    "Xr33eu9Wd0sty5JauyXbeBtjGwI4MeAce7J4yIEkZzhkIGwDZIAQjMGY2Int4xAghG0mBBiC"
    "57CECR4nk2HYMcwknmBLttaWLKnV2tWt3rtfvVru/PFkx35udXvR0l33+znnHbmebZ3zbtX9"
    "1O9361e/a5gDa+2quq9+r+74yrrjtO74ZoQQL5R/nGN+PVJ3/NdPPzDGHJjtL/c0vkK4iwQg"
    "hAQghHARM0POf0vdV5+oO16hYRNi3rKz7vitdWsCDykCEEJIAEJIAEIId9cArLWX1n33QN3x"
    "Sg2TEAuWHXXHNykCEEJIAEJIAEIIZwmAdyrnFyK3XFR3/A5FAEIICUAICUAI4fQawCYNgxDO"
    "cLUiACGEBCCEBCCEcHoNoKxhEMKpOT/zgZh/pNaSZBBbSC0k1pJZiDPIqH0Hte/tqX9++vd5"
    "paNg6C56GKNr5IzZQJy/ST6ZwEhsGU8sE4llMoXJ1DJctZyMLUNR7d+djC3TKQxVLZW09p0F"
    "hmNLnNX+viizTCQ1EeQ1b31bb8ifX1yi7MsAEsACYzyx9E9lHJzOOFKxHIkyjlUshyuWwWrG"
    "iag20Yerlkqm8aonA/ZO1sZvbZOvAXmRArhSw3B2iTLLE5MZm0dTtoymPDGZcbRiOVG1DEaW"
    "0cTmPmQ/0+ydzNg/ZVnbpLF4nrxMEcC5uEtZ2DGe8uDRmB+cSOmfyhhLLKOx7upngoHpjH1T"
    "Kan18bUQoBTgfGMtTKSWI5WM7x1P+NpAzONjKVEGie7uZ5xKBptHM4ZjS2coAUgA53Hin6hm"
    "9E1kfPNQzN8diTk4rRl/Ltg8kjIYWTpDjYUEcB4m/mhi+flQwt8fjXnwaMKhiib+uWT7eMrR"
    "yLK+yWKUBkgA527yW/55OOWrAzHfO56wbyrTIt55YDSBx8dSruvwURYgAZyb3DO1fKm/yqf3"
    "Vtk/lRFpQe+88vOhlDcth1BF7RLA2Qz3E2vZOpbxp30R3z0WM5FqXOYDDw0lTKeW5kAhgARw"
    "lsL9sQQeOBJzd1/Ergnd8ucThyuW7eMZ3UWFAC8EjdocnIzhk09EfHBbRZN/nvKDE4kGQQI4"
    "8xypZNy5s8LHd0ccjbTKN1/5oQQgAZyNyf/HOyP+qr/KpPL9ec2uiYxD04rOzpYAsrpP7nP+"
    "w9MZH9sZ8dWBKtOa/POe6dTy0JCigBcyXxUB1E3+QxXLHTsjvtSvyb9QqGbwk6H0qX4IQinA"
    "C2Iihc/sjbj/YFX1+wuIxMJjoykjsU6aBPAi+NpAlc/v051/IXIssmwd14k7GwIwdR9b98kF"
    "PxlMuG93xJhSyQXJyWqt34ILmeocn/r5qghg9ry/tuL/J7sq9E8phFyojMawbSwlznQOlQI8"
    "r7zfck9fxENDKbp0Fi4ZtceBeyb1OFACeI4kmeW7x2qv8+qlnoVP/1TGExLA8+K5vAswVx5h"
    "X+T/f94YmLZ8baDKPoX+ueBIxbJnMiOxlmDh9gc4p/PJ2QggSi3/63isOvIcUclqfRhHqhK6"
    "BDAHg1XL5/ZW9Vpvztg2lnFM721IALPGWBa+cqDK1nHli3njsbGUg5UMq7LAcyYAM8dn3nEs"
    "yvjCvqrOfg4ZT+Dx0QXdqemczicnI4CvDlQ5ojAxt/zTcMqUUjsJYCaOVjL+7nCiJp55FsDJ"
    "hAm9zCEBzMT3TyR6VpxzjlSsujdJAM9mMrH847GEIT0myjUZ8P3jerwrAdSxeTRl21iK7g1u"
    "RHqpngRIAE+SWssjIyn7pjT9XWDXRMph7dQkATzJyaply1iq130dIcrg/5zUyZYATnFwurax"
    "h3BkHcDCj0/oWaAEQK3X38B0xrYxXRDOCAB4eDhhUo8DJYCpFLaMpqr7d4wT1domrsJxAUym"
    "tfxfuMVYbHlkVOddEUCKCkMcZDypdQuuqOzTXQFYC+OJ5QkJwMl1gP6pjH7tGuRyBGDZP5mh"
    "a8BNDk5n9En+bqcA2/Tev7MMTFu2jqWqCnQ2BQCFgA4TW9g5njGo17/djQAGVP7rNDsnMpUF"
    "uyyAE3r7z2n6JjIOVTJtHupqCnCsogjAZUZjy/axlIp2DXIzAtD8dxsL/NNwxph2D3ZTAGoT"
    "LR4eThhJrLoF1xFoCNyjMzT0NngsKRk6CoamwFDM+a3AM9DgG518CcBdlpcNv9ZT4FWdAReW"
    "DV2hR1vB0BBAaMAYTRAJQOQOA7xikc+t64pc3e7TXjCa7MIdARRMrSDEVa5s87l7Y4mXdvj4"
    "mvji6amRCz+yu+juRd8UwC1LA65bFGjyCzcF0BS4e+F3FAy/3KlMTzgqAAMsdjgCaAwMG5p9"
    "XenC3Qigp+TsLugEBpoDhf7CYQGsKLs7ATILVZXACpdTgNVN7kYA0yns0V6IwuUIYH2Tj6sx"
    "wHBs+fmQGmMKVyMAY+gpGTpDNxUwllh+PJhwsqooQDgaATT4hvXNbqYBqYX/ezLhR4Op3ocX"
    "rgoANja7uw4wMG35+sGY/VOZ3oYT7gmgKTBc1hrgqgIy4DtHYr5yoMp4giQg3BJA6BnWNBq6"
    "HC4ISix8dl+VB4/GaPoLpwQAcEHJczoNABiqWu7cFWm/POGgAMoeG1USS99Exm3bK2zVXonC"
    "JQE0B3BJS+1deNf52VDKHTsq7Fe7dAnAmR9qDFe1e6xplAASC//jaMLtOyr0T2VkWhSUAFzg"
    "omafjS0+ag1Xk8C3DsV8Yk/EsUjNMiUAB2jwDdd3BkoDThFl8OUDVT67r8qUlgQkABe4oStg"
    "WVkCeJLxBD67N+Kv+6vopUEJIPf0lDxe0+1uUdBMjMTwZ7sjvnGwqsGQAHL+gw28dWWRBnXJ"
    "egYHK5aP7oz474djYoUCEkCeWd3ocUtPQWe/jj2TGXftqvDDwURNRCSAfPPmFSEtigKexebR"
    "jHv7In4xkpLqyYAEkFcubvG4cbEMMBM/HUy5a1fE/ikJQALIKa0Fw+8sC1kU6olAPRnwveMJ"
    "H9xWYSyRBCSAHOIbwzXtPr/REyAFPJvEwoNHYz6wtcJ4YtVMRALIH4tLHr+1tMDaRj0UnIlq"
    "BvcPVLlzZ4WRWAaQAHLItR0BNy0JKMsBMzKZwlcPxPyX/ZEkIAHkj5aC4Y3LQ2d7Bj4Xjlct"
    "n91X5ZuHqkynkoAEkDMuafF464qQkkbjtAxMW+7pi/je8USDIQHkC9/UooDru9QwZDb2T1ne"
    "8/g0W0dTvT0oAeSLxsDwyUvKTm8j9lzon7L89r9M8ehopkIhCSBf9DZ4fHBdkTZVCc/K9vGM"
    "D26bZtu4molIADki9Ay/2VPgt5YWCDUypyUDfjqUcveuiL1TqhGQAHJEd9Hwjt4i17ZrPWA2"
    "qlmtUOi+vohBbTsmAeRmQIzhkhaP968tsqpB6wGzMZXC/Qer3Le7qlRAAsgPgWe4aXHAretK"
    "zm4q+nwk8Ik9EX++J1KNgASQr0jgzStC3r4ypKxsYM41gXv6qtw/EDOll4ckgPxIAN6/tsjv"
    "LtOi4FwMx5Z7+yo8eDSmokhAAsgLrQHcuq7Iv1taUDvxOdg7Zblnd8SPBxO1FZMA8oExhpUN"
    "Hh9aW+TVXWogMhsWeHw04/YdFbaOaSvyBXF9W52l55bnWstjYxl/8Ng0PxtK0YOvWS4q4Io2"
    "n29dXWZl2ccoclIEsPDXAwyXtfrcu6nES1o1bHNFAr8YSXnXlgoD01KlBJAjru0I+NONJZbr"
    "nYE5+f6JhLt2VTigTUglgDxxY3fAHRtKLC1JArNRzeCbh2I+szfiWEUSkABylOW+/oICf7S2"
    "qH0G52AsgS/1x/zXgZgxdRSSAHIx/Q00+PCm5SHvXR3SqocDszIUW+7ti/jmIdUISAC5kYCh"
    "tWB4z+oib+sNaZYEZmU4trxv6zTfORKTqEZAAsgLrQXD+9YU+e1lIaoYnp3xBG7dXuEHg4ma"
    "iUgA+WFx0eMj64rcvERhwFwcmLJ8dEfEz4ZS9RGQAPLDhQ0ed15U4lfUV3BWLPDoSMrdfRGP"
    "jqYaEAkgP2xq8fjjDSX+TZuGdTZiCz86kXB3X4UnJtVgVALICb4xXNcRcO+mMle0+hrcOSTw"
    "7cMJH9kecXBaApAAcoIx8MrOgNvWF1nTpOGdi28cirlvd8RwVRKQAHITCcCv9wR8eF2RjlDj"
    "MdeawBf7q3xuX8S4molIAHkhMIbXLytw+/oSRY3yrFQy+Mu9Vb7cX1WhkASQH4qe4feWh3xg"
    "TRFVDM/O8cjyiScivn04Vo2ABJAfWgL4g9Uh71kd0qAnhLOmAv1Tlg9vr/DAEXUUkgBygjGG"
    "zqLHu1cVecMFBTUYnYP+actHdlT4yaAeD0oAOWJFg8cfri3yq90BgdKBWdkxnnHr9gqPj+kV"
    "YgkgR1zU7POxDSVeql2H5uQXIym376iwZ0LVghJAjri4xedTl5ZY06ihn4vvHk+4a1fEETUT"
    "kQDysyYAl7f6fP6yMmsaDcoGTk+UwdcPxdy5K+JYpC7DEkBO8I3hlYtqDUZXNUoBs1HN4Cv9"
    "VT71RJXh2EoCEkA+KHiGG7trbcWWqbfgrFQy+NzeiL/qr6KuYhJAbmgMDK+/IOQ/rS7SolYC"
    "szKWwJ/trvLfBmINhgSQH1oLhresDHn3Kr00MBdDVcuf9EX8z2OSwJlCOwPNEyYSy9s3T/P1"
    "QzEqh5/lggUub/X49KVlru3w8bTtkCKAPNAUGD5+cYnXLS3o5aFZsMAjoxl37KwVCmW6f0kA"
    "eWFx0eOj64u8pjsg1I1tVn46mPLJPRGHKnoyIAHkJbw1sKHZ47Z1Ra7t8LUd+SzEFv72cMwX"
    "9lVRnZAEkJ8TYgxXtfl87KISG9RRaFYmU/jCvirfOaJFwRd809Ei4PwkySwPDaXc8vAUI3r4"
    "PSu9DYYHr2lkY7OH0aKgIoA8EHiG67sCvnxFme6iLurZ2D9lefdjFQbUXFQCyBuv7Q64Y32R"
    "HlULnhYLPDyc8Om9ai4qAeSMsm94w7IC/7E3pEuPBmZdD/jWoZh/OBYTqZuQBJAnOkKPt6wI"
    "+d0LCzSplcBpGZi2fH5flV3jmbYdkwDyRXfR49Z1RW5aUtAmpLOmAilf7K+qxbgEkC+Mge7Q"
    "40tXlLmhO9CJOw2Jhc/tq/L3R2NFARJA/iTQ4Bv+82VlXtEpCZyO1MKduyIOTqtCSALIIcvK"
    "hns2FrmuQ/sPno7dExl/sTci0ptVEkDe8I3hijaf29YXubRVp3AmMuDbh2N+NJhoMCSA/BF6"
    "hus7Az60rsiKsh4PzsTAtOUrB2KOR0oFJIAcUvINt/QUuPOiEt2qEZhxLeAHJxIeOKLtxiSA"
    "nBJ4hjcuD7ltfZE2bUD4LAarlm8dStg9odoACSDHvG1lyDt7Q1olgWfx85MJPzyRqEJQAsgv"
    "RQ/etSrk319Y0CakdUylcP/BmOORVRQgAeQTYwyLi4b3ri7ymz0FbUdex78Mp3z3eEytXlBI"
    "AHk8mcawssFw18ZaWzF1FPpXqhY+s7eKKoQlgNxLoLfB5y8uLfFLi5QLPJ3HxjL+4ZjqAiQA"
    "B1jd6POpS8pc0qJT/HS+3F+loupACcAFLm3xuHtjiYvUW/ApHhlJeXRUW45LAA5gjOGGroAP"
    "rVe14JMMVi3fOaI0QAJwhJJveN3SAu9fU1S1ILWNRv/fcMLAlMqDJQBHKPuGt/eG/NG6Ip2S"
    "ALsmMh4aUhQgAThEwTO8Y2XIf1gROn/Sj0eWLWOpFgMlALdoCAx/uCbk5Y4/HkwsbBvL1DBE"
    "AnCPrqLH7etLzu810DeRaR8BCcBNrunwefOK0Okx2DeVsW8qc/41YQnAQRp9eMMFBa7pcDcV"
    "SCw8OpIy6vi2ghKAgxhjWNvkcfPigJLDV8C28ZSxRBGAcJCyb7iuI2B1o7uXQN9ExoQEIFxl"
    "Q7NHb4O7l8DRyDJUtbi8QbYE4DBdRUO7w4VBqYU9kxkuPwyUABw/+a7XBe6eyHC5U5gE4DAn"
    "Istw7HYOfGBaAhCOsncq45Dj1XCHp5UCCAeJMss/D6fsmXRbAEcj63SXQAnAQTJr2Tya8jcH"
    "qow5/lLceOJ2p+BA08EdrIXYWraMZrxrS4XNo3oZZiR2OwKQAHI94S1RVmuCMRZbjkeW/30i"
    "5uO7I4ZjjQ/U9gxAAsgvj42mPDzs7lkeijOOVyx9kxmbR1IOVyy67wtnBLB7MuX3N0/rTAsx"
    "A7lfBHzlooCiljqFcFMA7aHh2nZtkCFmpjs0TldDei78wH/bU9CVLmak5CMB5J2XLfJpUhAg"
    "ZmBpycMYCSC3GGO4oOTxklYZQDybTqUA+aclME63vxKnZ1nZ4CkCyDeNAVzVFhDqaYCoY1WD"
    "73Q9vOfGjzSsbTJcqp1yxdPTQ2BTi6cIIP/rALCs5LGpWWmA+FfaCoalZQ/P4VVAZ26Ji0LD"
    "phafsoIAcYp1TR7Ngds9kZyZDoFn2NjssVQGEKfY1OzR4vjrcE7Nhg3NPheWtTuuqOX/l7X5"
    "tBYUAThDb4Ph4hafghzgPMvLHuubdC04JQDPGK5p92mTAZxnY7NHb4PBGEUATnF1u09HKAG4"
    "TOjVwv8LtR7kngB6GzzWNXlIAe7SUzRc1eZT8nUVOCeAwDO8qjNwuvjDdVY3elyr0nA3BQBw"
    "85JAC4GO0uTDq7sLLC7qAnBWAOuafNY2Kf9zkWVlj9ctDZyu/nNeAACv7VZDZBf59Z4Ca9Qc"
    "QgJ4TXeA1oDcoq0Ab1kRaiDOsADsHJ95yYZmn94GpQEu8cYLQ1Y1znvrn9P55OwMaA4ML9NK"
    "sDOsajC8dWWox78SQI1GH67vUhrgAmUf3rumyOpGz/nKPwngFAXPsK7JY1lJF0TeL/AbugJu"
    "7A5U+CMBPJOeksdlbUoD8szKBsObloesbNR6jwRQx+Ki4bJWX3uk5zjNe90FBW7oCggU+ksA"
    "9ZQ8w4Ymjy5VheWSl7b7vHd1kRaVfUoAM2EMXNzis15VgbljbaPHFy8vq+T3DAggq/s8ax7N"
    "8ZnX9DZ4rG70lAbkiGUlw32bSvQ2+gtx1f+czifnr/vGwPCSVl9hYk64oGS4dV2R1y5WqbcE"
    "8By5ut2nS01CFjydoeGdvSG/syykrEd+EsBz5ZIWnyWqB1jQtAbwjt6Qt/cW1fLtDAvAq/vk"
    "Lw3wDb+0SA0iFypdoeGujSU+vL5Ie8GgJ36KAJ4XxsCNiwsUNRoL67wBqxs87ttU4t2rihQ9"
    "Tf7ni1ZKTnF5q09n0WNiKtNgLAAKBq7r8HnfmiK/qgU/CeDF0hQYXr7IZ78EMO9pDuCWngLv"
    "WhXyklafgho8KgU4E9yoLkHzniVFwz0bS9y9qcQVbZr8L4Bn1PXoin8ar+gMaApgItFYzLdc"
    "v8GHV3cH3LuxRG+jR6iJrxTgTNMaGH6lM+CBozLAfJn4i0LD5a0+b1kRcvOSgIZAE18COEsU"
    "/VoUIAGcfxaFtY5NNy0pcPPiQLs6SwBnn9DAFa0+7QUYjjUe5xoPWFY2vLIz4Je7Aq7vDFih"
    "vo1nI7CSAGYcGWNYWq7tIPzQUKoBOUf4Bq5r97m5J+DajoA1jR7dRYOvh/qKAM41XaHHla0S"
    "wNm8y/umlm5d1ORz4+KAmxYHrGr0aA4MRQ/17ZMAzh/toeGqdp+WAMa0FPDiUiqv1pWnwTc0"
    "BobO0HB5m8+rOgOuafdZWvK0R6MEMP9Y1eixptHjkVEVBc11Ny96tSKqpqD2Z/OpT2vBsKxk"
    "6G30WFn2WN3ksarBo6i39M43P5UA5hJAg8eaJj/XAih6POOVWY/aXgmeAc/U7tyBMZR9KPmG"
    "Bh+KnqE5qN3Rn5zojQG0FcxTn/aCoSOs3e1Vmz//MdZaq2F4Jqm1bBnNcl0WHJhaiP7UhUBt"
    "ohtqAgg98I0hNFDwTn2MoeRD6BlKXu1PhfALjp9IAEJIAE+lAL+o+w+u1BgJkd/g7+kHqrIQ"
    "wmEkACEkACGEq/nANq0BCOEMWxUBCCEkACEkACGEsxhr7dV13/1t3fGFGiYhFiz9dce/oQhA"
    "CCEBCCEBCCHcXQOo/8Ja+/t1X91dd9ypYRNi3jJQd/yBZ0x4Y76hCEAIIQEIIQEIIbQGcDqs"
    "tZfXffW2uuNNdccv17AKccb4Wd1xfZuq3XXHf1mX829RBCCEkACEEBKAEOIU/x/DqmKfaTJf"
    "kgAAAABJRU5ErkJggg=="
)


"""Skyperious application 256x256 icon, 8-bit colour."""
Icon256x256_8bit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAQAAAAEACAMAAABrrFhUAAAAB3RJTUUH3wQcFAIfZOIuZgAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAABsUExURf////7+/s/w/Kzk"
    "+bLm+rbo+sPs+8vu+9Xx/Nvz/OP1/ej3/ez4/fT7/lTJ9ACv8ASw8BO08B248SO58Sy88jK+"
    "8jvB80fE82TO9WvQ9nTS9ojZ95je+J3g+Quy8KPi+Rq28TbA87vp+lzL9TIr5gcAAAABdFJO"
    "UwBA5thmAAAIrUlEQVR42u2di3KiShCGgxoUYeRmQEDx9v7vuGazu9mzVTmV0N90Q4X/CWY+"
    "Zaanr09Pfyn4Jnr6SNYLmwHMAAwBWC/KHIL1gmYAMwBDANaLMYdgvZAZwAzAEID1Iqw1A7Be"
    "gLVmANYLsNYMwHoB1poBWC/AWnoAnHNJsn0oemgzUPEUAbjtZn1bnbu2ue/qar+/5HmfDlTj"
    "pgXAbZ5PXbOr9pfBW/6v6mg6ANzm3O6qMs+Yrb+pvE0FwKarS+pn/0v9cQoAXHS89ugP/64m"
    "GT2AOGwKP5t/VbUZOYBk1Vz8bf/xDdCHAAwgbEpP//3fog8BFIA7lvy59492sCVAAljsvG8/"
    "TS+wMcgBcOfS//Yfgg8BDEDSeT383tWOE0DcKPz9f6oaJYD4oLX/NN+OEMBWb/9pfx4fgO3B"
    "8+X/t7JmdABcq/f7P1ShzwECwFF1/2kZjgxAqHP//1GOWsNyAHGtu/80PYwKgGsUD8A3oU9i"
    "MYCl8gfwULkaEYBop77/tO/GA8Ap3wBvOoAXoRDAdm+wf/QQEALoLPaf9s9jARB7dH/+nzrO"
    "LSQD0KlfgW/acYeACEB8tdl/euGexCIA59wIAOgXkwBwd6v9g34xCYDQ5A78qZdRADAxgt7U"
    "Y4eAAEByMNt/mi1HAGBhdQe8CvOLCQCs7L6AxyFAmULDATgbM/iXCuoiHA4gNngIvwt7Eg8H"
    "ENldgq+6Q9/AcAAbyyPg8SSG8sWGA1iZ7h/ziw0HYHoGPiwB6BAYDqCxBZDemUyJ4QDUwwH/"
    "6LowBmBpB74qZw6B4QCMvGHvYvxiwwEoZcR8rB1yCAwHYL3/9IJYArYA8mt9b9qhQnwClgCK"
    "ZrmOErwEYjIAqmc681sXgDQkcEUTPQwACG8BNMRrAkCYF1CsrbcuBVDJAOyNDz85AKFDaG+9"
    "czGA9rv/A44yACVd/KMOIJQBYLP9LABEstBwBsb4bQBshddAgUW3jACIQ4M7vA5YF4D0FEyz"
    "dgwXgQDATeoSydnSB3UAG7FblE18Vwfg5I5xvBJYFUBwFOdIZfYHoQTAWu4Zz+5sDZguAHeX"
    "50lmDd8XRQ1AcALyBHvjy1AEYPsiB+ChK4YeAOmT+E2FqTkgAxAhSRKl5atAWC/AJMu+gPn/"
    "ygBCJk+msjMJhQASKFWM7xClBCBYMRnzGd0aRA1ADCUMZwcjAuLCyWeocLJvbVxk8tJZqnY+"
    "70wIyIunF4Q5+KriaPEVAOXzWN0EWhOsCMBhCXOFgX+E6CARYQljeyb3TxsAWDxUqxNAAMRY"
    "Fxl9HxnTR2ghjBK9qydL4/UABCusj0QP9wpTAoB4x34JLA1XBBBwzZTykyYBrp0e106rPCsS"
    "4BoqRoCT/Jc0PURgS80FV0IBFUMoAwAvw/RFzRxAu8qG1MPwYRJqhczYvsLP2KsgOygRgBsr"
    "n7A6kr7VCRrCAJw8ZP5buY5BRPcWd93EDCK8uzzYXxNun6oEAHwbp6WCQeRhwMKWayygUFXi"
    "Y8IEZxJm/j1EXmaMhFhZrX8PkZ8hKzfOJPSdReVpygzYaRQfq6ICAOyz1vs1iHwBcEfsMsy9"
    "hsy8TZpyLeghmiKAIOFMQp+1FR6HrYFDB/b4iDENAMFmh30F/nxkXsftgT4yb1lUfucNrjFz"
    "IIO65igDCG4cAU9p5b5HbnImoacsKt8A3BHzk/rpOOB96CrqI5sigIdJSAHw4iHSGLsLBg15"
    "D5HG4OWYM4h4D5HK5OlNjeUQ4f4RndHb64oigNdYKc0ev2EdWOmCY63h6zfsMoR7b6hNn19h"
    "YVM2cq4GwB0pAqxNrAYgSDrKT1qQSVR6AIIY85FVoDWgCIAziLJmLGN2viissiDj7kJVAOLW"
    "O39UjmHExhCFFAHMIFQGEDxDYVPsZawNwC0ZD1HGdBXWB4CFTalmbPoAgOYzP1UzbwIDAK5F"
    "CPRM0NgAADWpvjLuLC0QU1qQIaeACYAgQnxkSF9WGwBBhBhERK2xEYBgQfjIauAYtAIQrAAC"
    "BRAmMAPgznKTkCiyNAOAlBYA7kE7AEEg95Fd5H4BSwDysCnQl9USQJCIL0N56pApAHlj2lKc"
    "P2cLQPwRZOKcCVsA8sGt4lxyYwDiBBrxRWgMQBw2vkjdItYAYmGoIJNaw9YAxD0ppafg5AFI"
    "nwPWAKSfgHj8rjUAccuBatoA5Hmk5bQBrMWRssukAQD96fupAnDJdo0kzVgBWJxk6tqaCZNa"
    "AVgiq58wgARLgJ4oAOm8QUq5GQDj6fO/VZgBWGPZvyJdzQBgGV8yScNjwwGAtUASNbL9Swyh"
    "8yjuAWktnQAAYMcCkobIBQBiZsCITLk0b1ryFuBKIofrKg0NSQBwzYSHS9yHWQIg4pqIDpY4"
    "UUr0HG7M74HCNjbINQsaKuPocGR9CJjnB1gfAkB/HRmAzvgQqOU582OYNjdYRE8JoVMUKwke"
    "pBLorSMEYPsiJGYxCAHcLA+BnmiuJASwtbwIpa4AAkBi+CJk6uilkaGT2TcwkpKZEOuT9FVB"
    "7dWkACKui/LXRI2qlQJwRuEBrJmKODp8xlpjfEmAEQwB2JiEBwgbEALguGbyn1fBjWaUJ0hw"
    "MyU+rQvYaVgOIFQ3BtHxrHIATvsQGF0bHa6F9Od+f7bTNgAAGsD+SZVwW0li6qziIZBV9Ghe"
    "Ik1O70XY39fw/hEAZ639X458b2ECwFbnEOh3PuYyEwCcxoswr5de5iwgAPy/CPP66GnYCpIr"
    "fPNsDRf3k7dZMwgAph/EB8qqLvQ4eQ8BkPgKD2T9tV0nE5gyc8bvgSy/lFWz9D92kgEADhvt"
    "83Jf1bv2uNpMaOBiPPQizPKHirLcv1R1vTs0bXc8rcLFVm/yMlQxslgO0+r5oTAM1+vFJtrG"
    "fj93nwCmqxmA9QKsNQOwXoC1ZgDWC7DWDMB6AdZ6epX1Ikw3PwOYAcwAviWEp39lvaAZwAzA"
    "GMB3gvD0kawXNgOYAegD+AGOU1h0v+LQcQAAAABJRU5ErkJggg=="
)


"""Default avatar image for contacts without one."""
AvatarDefault = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAALEgAACxIB0t1+/AAAABV0RVh0Q3JlYXRpb24gVGltZQA2LzIwLzA4DqTMIgAAABx0"
    "RVh0U29mdHdhcmUAQWRvYmUgRmlyZXdvcmtzIENTNAay06AAAAF/SURBVFiFzZfbioNADIZ/"
    "t0IKgoKlhfoMQt//MYT2oiBYKFgYITCCFwMD7ZVlD27nZNf9wStj8mmSSYz6vr9jQX0sGfxf"
    "AMQ+D0kp0ff9V0dxjO12izh2cxm51MDtdkPbtlBKTd4nIhRF4QRiBaC1xuVygRDCyikRoSxL"
    "EJHR1qoGqqqyDg4ASikcj8dfv5QTgJTSytEUxPV6DQPQWuN8PjsHHyWEgJTSH6DrOmitvQEA"
    "gJn9AUKDA/jRrk4Ac2gYhmUBkiTxB3A91Xx8vAQw0dtotVr5A5jyZ6OgFDRNEwxgOozengLT"
    "PHgJkKZpMEBQCjabTTBAlmX+AGmaBqWBiJDn+Usbq33gdDoZh8p3JUmCw+FgtLM6CU1vMSXb"
    "+nkbwG63mw+AiJxqwcXeehi5zIX1em1tayzCYRgghHBeTrIsw36/9+sCpRTatgUze+2Dn2Va"
    "1Z8ASikwM4QQswyhKeV5/rxGmKiu63vXdc59PhdMXNf1nwYexcxg5uV/Th+5hK9m92rw1QAA"
    "AABJRU5ErkJggg=="
)


"""Default large avatar image for contacts."""
AvatarDefaultLarge = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAGAAAABgCAYAAADimHc4AAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAALEgAACxIB0t1+/AAAABV0RVh0Q3JlYXRpb24gVGltZQA2LzIwLzA4DqTMIgAAABx0"
    "RVh0U29mdHdhcmUAQWRvYmUgRmlyZXdvcmtzIENTNAay06AAAAQ2SURBVHic7Z3NauM8FIbf"
    "TgQqCGpQiMEFQ70L9P6voFeQRaALQwMODSggsMFQgUO/xYeHgWlnPI2PjuTqge4aydZjSUc/"
    "lm+6rntHgo0f3Bfw3UkCmEkCmEkCmEkCmEkCmEkCmEkCmEkCmBHcFzAF5xze3t4AAJfLBX3f"
    "//X/siyDlBJaawgR7m0Ge2Vt2+J4PKJt2y/9vus6AIAQAmVZoiiKOS9vNm5CmwtyzqGu6y8X"
    "/GdorVFVFaSUs6Z7LUEIGIYB1lp0XQdjDGleSikURRFM08QuoO977Pd7DMPgPe88z/Hw8MAq"
    "gjUK4ix8ADDGYL/fwznHkj/AKGAYBjw/P7MV/kjf99jtdp9GVtSwNEHGGBwOB/bC/xUhBB4f"
    "H6GU8pqvVwFUEc5cKKWw3W69RkreBIxVPQaqqvI2bvDWB1wuF19ZXU3TNN7ySnNBHzAMA/l4"
    "ZMSbgNfXV19ZzYKvWuBFwMvLC6y1PrKaDeecFwnkAqy1OJ1O1NmQ0DQNecRGKmAYBtR1TZkF"
    "OcfjkTR9UgFN0wQ12PoKbduSjpLJBAzDgPP5TJW8VygjIjIB1tron/4RygCCTADnDOPcUN5L"
    "EjARqn4gCZjIuMY8N2kqYiJU44EkYCKpBjBDFdElAROhWrgnE3B3d0eVNAtU95NqwES01iTp"
    "JgETybKMJF0yASHsOpsTqoV6MgG+t3fEShLADJmA1Wq1KAlRDsSWJCC6PgCgu2jf5Hkep4Cl"
    "LMhQ7pJLS5IToKzJaUlyArvdLr7JuFj3An2Ec47sfsgEcL3wQEV0nfBSIqCR29tbknTTSHgi"
    "0U1HU03fckD5MJEJoJq+5SBKAVLKxTRDUQoA/h/CL4FoBSylH6Bc3yafjIs9HKVuRsnXhKni"
    "Z19Q7+4gFxD79pT1ek2aPrmAmPsBKWX8NUApFe2YoCxL8jy87AvabrdRhaRCCFRV5eWavR7W"
    "wX0+0BR8H9jhdWecUir4PoFy/fcjvG9NDH1c4Hv6JO0N/QUhhPew2buAkCfoOJpH7wJCDkk5"
    "Dnf1LmC1WgXZEXNNn7P0ASEK4LomFgHr9Tq49weo53w+g0VAaM0QR/QzwhaGhjQe4HwY2ASE"
    "FA1xPgxpIAbeh4FNQOwLNXPBenh3KIyfPeGATcDhcODK+jfqukZd1yxH7HhfDzDGwFob7HlC"
    "WmsUReGtXyAX0LYtrLVBF/pHSClRliX5p05IBIwFvoS3ZIQQP2sFxVzRbAL6vsfpdFpEoX9G"
    "lmXYbDazrhVfJSCGNp0CIQQ2mw3u7++vHsT9swDnHKy1MMYs7jWkr3DtZ7EmCRi/83U+n4P9"
    "/EgIaK1//k2V8UcB45Me29HzITBVxm8CvkNn6ps/ybjpuu7dOQdjDIwx36oz5UAphTzPobWG"
    "lBI3T09P76kz5UEphR+p8Pno+z6tB3CTBDCTBDCTBDCTBDDzH7726OxO0Z6SAAAAAElFTkSu"
    "QmCC"
)


"""Small icon for clear list button on start page."""
ButtonClear = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAgdJREFUeNqkk8tu00AUho9nfKnDJU5j5NoJkTB20+JU3ZR1"
    "F+xaKhZIkRCSAYkdL9CqEgtW9B14BVa8AiyQQCELJCRUFUgXMZAKqOPY8WWYsZISaFohdaTP"
    "njnz/2c84zMcIQTO0rg3L7iT5oqUZ6P+Q8rPaSJ0SvKmbt2zdcu1Wf8kEcoygH+hu7qMhdKa"
    "Vru5qNXWF7GgrLHYNO3UBBTXsN0GCT+JJPwsGtbdBov9VwK60rJ0rrZaKjfM7Z2vsL3zDZTy"
    "NVMqVFfZ3LEEKX1MwMYPKtYdJ/I/oiAIgDHs7yH9yi2HzY00Rx6UpTTTCJLBjQuKsyLLRSON"
    "f4Hv+zlp7EOhoBjnL1orTDPp4dP06EBlDnGucXXdiYMvMExDaLVa4Hke7D9ahozsAj9bd9Le"
    "rksy8pLqB/lfiBMABv2c2+W560sIRUpv0AGvvwftdhu63S6kQghEjADESJGK1SWmHfsQK0SK"
    "CpzYVKuN+d7hW4izHgg4gu/ekxwRD3Mw50NR1+YJ4CbzMC9K6BboNu6rlfpCnH2QeeSDhMMc"
    "VXucMx4zZCmRS3OXFpiHeVGSgIUFaUOrzJgC/IAZutKYcZuMMfSqYiKe32BePgjIlllXbREf"
    "YkLYif65XMHBZv7uHzz/u34xYKM2a79/523xwQCi16/2OzTcOV7pT0+/iRxE3Fmv828BBgCY"
    "syym9ECFNgAAAABJRU5ErkJggg=="
)


"""Small icon for compare databases button on start and merger page."""
ButtonCompare = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAu9JREFUeNqcU99LFFEUPndmdmZd3fXHqmlrta6SYZnmSNhv"
    "I4l8UEjIeuihJx+iQAj6E6K3giDaoAjaHjPYCipLkyIoEiuTLNx2Ze2Huq7uOrPj3pm50xld"
    "o8fqwjf3cufc75z7ne+StpszAITA2rAYA9M0QVMUEBwO4Hh+ZV+UJAwjQVy+MAwjZOg6GJSC"
    "AH85dDzAcZyMSzm3FbI/NsEZsKxqnNOIKOIVYjIXJFuW1Y3zdss0yxlWdrStSr73fLoPz6yQ"
    "CFhOzbH2ykBaNfUfieXA5ymlBeOeYYCfJ+xUfcBTvs7rKnSKvJMA4RMJCh2tFXJ4eKoPCRKC"
    "fRdUgLN0kLwuUdrXUOx5+nr6nNvF+/e3+Cp0Cg6TWqCDhRpYQKkB2awAeja7UqKAQkT6B2KC"
    "31dQUFmS741OLZUay1rD7p3+AlUxeV4A9jWe+B6NLybVDNW7DtXL/Y8/jqCgl3mH4xFpvDhm"
    "67AJ73q4tNjZrKSWOrfWVRZLoiSazLDejce/zCczg4TjgtihG3ZW+7BDkkKCKAIJnB9elYsQ"
    "nhlG2Ocr2lvsr/QsaRQy8Zn03JzykhOELowwEccRNYgLa93h0smkiGhWFhauiHl8jVpd7R6l"
    "EqgapTMLasbj9XzDuF7GTLtTrYgyrPYg1TRQUyngHdu6r3rceWfL6jY2ppua/FHm5Eo43Uy8"
    "GVO1zVsmeH+V4CRmCVUysqEtswM9e2ojoxGbhDLGYiT/xO23Had75AdpEXRsbZmp6XRg8CeH"
    "P6Uj7fOzxClw2Ca3A8CDasVUgB1FAPE79ydXfKCjHSeymEJNZlk0tph6/2mWUXrL5XbHMg+f"
    "HPLW1Rbq630uw10gLnMCx5BMYwA010bCdwWDWE4TrucQH1Dhu7wgjCABoEdqUflduG/fv9DW"
    "bEPvydr49dAkxoVR3CHCdV5D01m/PY9+BySAHAGYhrH6yDAGrXwp18YwPrQh9ME/PKbVkiOI"
    "cUwwtLZP/sz+P+OXAAMAj/dc2TUqu0UAAAAASUVORK5CYII="
)


"""Large icon for detect databases button on start page."""
ButtonDetect = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABHNCSVQICAgIfAhkiAAACIpJ"
    "REFUWIXFl1uMXVUZx/9rrX09++xz2WfmzJnTubXTQlung7SFUrBALQba1JAQ28gDD5qIShQj"
    "gYRoTKiXB0M06oMooQ8KMVpsDJeAJERagXZgqGmBTmnPFDr3mXPm3M8++76WD05rQWmnSOJK"
    "/tkrO2uv77f+37fXyiJCCPw/G/0kH934wFH90wIgl3Ng89ePyQ4p3Uw5v16T2faIi2tDzi1G"
    "SEOi0nE3ig4Jwt9IxOzDR36+x/lUAdZ+7dlhDTiYiem5NZmknjdN1mkaiKsKOInQ9DwU7TY/"
    "U6w6pxZqdZ+TL53av/vo/wywffshqbSy/AMG+tD2gV5ttWWRZquI6ZkCyovTWChOwTSSuOaa"
    "LRgcXIvOzg6UWzYeO/SOa/v+bxxDf7jwq53eJwIge59mQzGMZGP6Z27t7dcJt3Fk5BmcPnsC"
    "WiIdhIgIYYxRQqKo3UJgt6Xurj58494H0NPTjT+9dtJ7bXxmQlONa9/67e725QCkj75Yp3rf"
    "SyvG+h29ffrU9HGMjL4ET1PCZE/PGRLWT7KgPsl4s8iZaQVJs19YPesqjj306C/3sbt234M7"
    "r9+iUiH6Dxcmfwbgm1fkwOov7x9SKHvzi4NX64FTxN9Gno1gxjzmTP+xK+YdTyQMp79/lbR1"
    "69a+RDxuekHARkZG5l4+MtbZ8LWv8rar777tbnrd5hvw0+cOObW2t7Nw4CuHlwWwffs+aT6b"
    "P7mxK79mRUwlR956Hg3iOWk+8ZPebHIuk820EolE+/bbb1+1bt3QDstK3yEEwkajMX7o0KG/"
    "Pv7E7+am7MQPY1w279l7PxxO8MTrIwuaog2e+P099scBXNgHJjqsrRpDd18iRSamxlAn3EtK"
    "tSdX92anBgb7SytXDhTXDl61oOvaO5qqfgAQj1KqJpPJgeHhIXP3HTv87iR+3Qjq0dGRV5BQ"
    "dVxtWWazVd91KQcu1ADxg+uycUvlPMBUaQIKsd9f2UmP5nvzlb6+nkou11W3rFxTVY3J0uLi"
    "BytU6SSl8g1hEAhG2JgWU6cHu2OVui+Pnp74x5YNG24iPcm0fmpu6kYAT18WQCbi1qSiKK1m"
    "BT5Cz9TpsWw228hnu+sr8/3VfDZb9SQpopRGlDb9QqF9UHKcvzBZpnOlUlylciaX6+jsqNWO"
    "TjfbGxuNhpLUdEIF33YpBy6kIAiDzaakIgxDRAKwDPqeZaXtjlymmcnn61KShpIkhYrSDB1H"
    "CbOMBVI6HQSKG6W64nY+l2t0dHS0cmn5ParIrF4vQ2MqgtBbTwjIJQHI3r0siNwulckgRAcj"
    "XMml2IRpmm6HaTkJxkJNU6JuRQmNuhSqqhq0VDVwXTdyXSVSFClSTNMxzbjblZLnqaSg3iyB"
    "hwAVggzs2td1SQBx4EDEBC+23DYI1RCTk2Ez1LOyrIdyXA4aUSQqlUhEqRTv3qSHo6OjQblc"
    "jhKJRBSPe5zzSKTTim8oRtAM1AQlTMQNC23XQRA44twLjyxcNgUkio5V2w0QwWBoXaTmymuB"
    "ELhoQ11Y+Nc8e/bsEZs2bRLA1IcmC+CjVOerFEZDSUpisVkFFfyUuMSBcwHA951XKnYtiKIQ"
    "cS0v8dC8KQgCqRXYMmOMWBYjqqrS6ekEGx8fl8+dOyfJskGllkopZcRedOQo4lKl4d6gKXFZ"
    "cIrF+qIIQ+/Vjwv+IQAOPlpqFN0gcpGKr4ZBV6x9c6x5s+M4WhAEzC35TFUdptm25Pu+7Hme"
    "bNuB1FYUylyZNoKW/urxueGmE+zIJVeyiBPMVOfaIQ+OLAtgznaOOpFbnK7NCCGqMPVh6rq9"
    "33/nzHSuUSolfFVlvh9JxSiSoyiSdd2XfN+QwjCUSpVKbHx8PjN2rvrjrNEta8oKFOtFlFpF"
    "J4y0Fy8F8KGzoGfb/dcQyo5u7d2sp4yNgKihFp52jVTp4bs/v+6lrt5MkzEacq4LA4DneXS+"
    "uWgcePHk1rGJ8qMamJUz1iscMRyZeBXy4siJ02+//tllAwBA7y3ffcRU4w9t7N4U09XVELDh"
    "8smoGUy/ocf85wa6Eye2bcidGzld7i7MlDdUavYtjuPvyqiWakp5COg4vfgeujM1LLz7Andd"
    "f/+JscK9ywYge/eynoWe0aQWv+pqa41hqHlIchx+NCtC0vSbfpXaQZMJEUQKFb4CTTNYmjGS"
    "QhgKFKpnYPuL2DVYRl9vHvufOhjJkvzgibHCL5YFAABDQ/uUqlX7ESX41prUoN4Z7yKMqiCg"
    "ICREyJsghENEFIIwgBNU3CoKtbNIGxy9Zh0xZwrZTArplIU/HHw+1FXtzrdPn31hWQDnW/5z"
    "395MqfTnmKxbaTUViysGM+QYFKYi4D5s30YraPO617CbXlUiC0ea64c3WpIiS7w1j7ioI5tJ"
    "Q1IUPP/S330K6drCxMTYsgEAYM2u+9VWC7dRsC2MkFs5xDCEiAkCjxL6Lhf8sOB4IxDNl5XC"
    "M/eB0gcHhq5X4olMwq9OIq1xZDvSaDs+Ro693bS9xqqZmcbisgH+W+vfdl968rXHagDIkuiS"
    "SDaT2SlJeKJzxYBv5fpyXmUSHQkd2Y405hcqODsxM1Vv+2tnZ2cdIYS4YgBCyPmgbEnykpTz"
    "/YRhrIkZ6uNxM0m7egZ6/GYJnekEujrSOPP+JKq11uhY4YObAfhXdDNaCk6XAqkAYgDiABIA"
    "UgAsAB0N27Yr5fp3GvVaeaIwtsiJjLliGbPFMlb25+F63saUad4BQF02wEUrpxetXFkC0S6S"
    "DsDwowjFxerjbds5Ozs92Q5DzhdKFcwXK5AkKgjFFgDqslPwEYCPC65e9FSX3MmY8dgX4rHY"
    "TlmibQGQIIjc+VL5JgCzV1QDH4GQ8O/8q0tA5yUtuUSXxhNVVZPJeOwuLsRc2HafrLnuLIDg"
    "kxYh8J9/ALuof36MuEgcQLQkDkAIIcQ/AcljIy+SCczdAAAAAElFTkSuQmCC"
)


"""Small icon for export button on start page."""
ButtonExport = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAiJJREFUeNqMk8trE1EUxr87GfOYR9KKLUQwCUi6jdK9CkUI"
    "WoluIm5dBQTXroKIZOPWTSGb/A26cCW4EFQU0UUXaRERNTXpJM1k8pyZO56bR5shEnvgm3uZ"
    "Offc+/3uGeZ5HkTk8/lbNMRxuqiRXoiJPHszHA7j5XJ5p9vtot/vw3FcNBoWzTnC4Qh0XUc0"
    "GkUsFkWhcPdBIBCAr4AIx3HQbrdhGMa4QL3ewWDIoao6HNeBFJCgqBHfUaSlB2WAMOgxPhEm"
    "On0BSp8s9KaL3bHmY97Cp1Kp9DKTyWxzztHr9aZWHGIQhqZp5D+GarX6inI//qvAZrFY3J6H"
    "WD800R2MEAwFoekqYgTx+s072YeF++IGPvwX4p/DNrqjESIEzuYr+L63i57VQbPZ9ETOQoFF"
    "iCfeOXNhHrWQvZelaywIdpdJn5dC5NwBtwfoHTXxs1qFbdswDgzsG/vP6fM1YZvNOjGXy20m"
    "EonH8xBt18bq2iqSF5OIX4hD0RXfBulz6UfyyW58AeKPXwdotBrY2/2GL++/YtgfYOv2FtbP"
    "r2NjbaPIGHtzbMGyLG8GsVarkX6j2TZgOX2cOasieSmNEQGdLn5KS16T3h2fgKjKS5uJpK5o"
    "KD8rI5VKtTqdzlvfLVABRZblcbMEg8GxhUAoAmVgQgoxqFoEV25cHfdCpVJhItdXwDRNgyA+"
    "cV1q1wlYJkQhHpAkSYyemNOfaIhRxF8BBgAPDR3S0EHTJwAAAABJRU5ErkJggg=="
)


"""Large icon for import folder button on start page."""
ButtonFolder = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAp5JREFUeNrsl01oE0EYht9NNj8btMWIhHopEhAKRgURBC+2"
    "0uLFU/HkwYMIUhEUBIMpPWlBFHtS9CB4F48iNl5r8dTSnmKjCFYpHpL2EJP9m/Wb2Z+YH2M2"
    "0c1BlwzfZjOz3zPv+81kV7IsC4M8QhjwIb+bl8SJJPkfzMdIdsxQmP5N9xck9jrXm4vuCi//"
    "gUkcpptNp6fuzqGDnR/yWcFKba1Bgb6TA+cOnrk3m8hch2oYqOm6iCpFzTRx5XMKlYqGV5OY"
    "K+azruVr7QAuUjvgJzuf8NjZhdyuQ9dgULJOx8kvM1iaxOzGolDiOLWnwsa3tz3/7xy78OwW"
    "v6vlmCQiYyI2XGuKJs3WoFmL7y4Z6ma7xkRkWfQvvM7O09ecUMDpe4nDFFdXfekfVxQMJ5OI"
    "UZSj0Y6QDUAWpAYL6EL6xNXnNy3T8AVQrVTxaWUF+0dHoSQSnlLSL6KrDAVv+cvMvhZihg6m"
    "qr4AomEJY+OnsJ5/g72pFPaNjLTY1g6GcoZsq+oAksUHMdP3MjB2ysicnsD75WVsbW52NYbZ"
    "Flwmlscy5RUKWCYBmKyntahvl5E+eqT7DWxi/MbSw/MPfi7CEJ8960EBb1bV7133DcfiXh14"
    "NcABrD4AfO0ffLItAH1Y4BuA8tQBmFuE/Vng6y9Y5LL3gsFb4Kge7nUZ9gbAQHnDTTVAVIHV"
    "gNlSA6GgFaBP8zJk4ofAANopwIIEcBXQnSLcLpWga2ogAJFoFbpbhDVNXFOGhoegqdVAAKIx"
    "BZRXEQBVGyC2UypD04NRIBqpgfLGBMDCS2BmCvFkcg891xmBAMhhmR5aEX+0SOfFLQwXvmLj"
    "yf1cJMgXko/fUKDcu/l+HOeqcBucRzTpL+fmZa86zZT++XfD/wA/BBgA+029gcH4w1sAAAAA"
    "SUVORK5CYII="
)


"""Large icon for home on start page."""
ButtonHome = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAyBpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADw/eHBhY2tl"
    "dCBiZWdpbj0i77u/IiBpZD0iVzVNME1wQ2VoaUh6cmVTek5UY3prYzlkIj8+IDx4OnhtcG1l"
    "dGEgeG1sbnM6eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IkFkb2JlIFhNUCBDb3JlIDUu"
    "MC1jMDYwIDYxLjEzNDc3NywgMjAxMC8wMi8xMi0xNzozMjowMCAgICAgICAgIj4gPHJkZjpS"
    "REYgeG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgt"
    "bnMjIj4gPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIgeG1sbnM6eG1wPSJodHRwOi8v"
    "bnMuYWRvYmUuY29tL3hhcC8xLjAvIiB4bWxuczp4bXBNTT0iaHR0cDovL25zLmFkb2JlLmNv"
    "bS94YXAvMS4wL21tLyIgeG1sbnM6c3RSZWY9Imh0dHA6Ly9ucy5hZG9iZS5jb20veGFwLzEu"
    "MC9zVHlwZS9SZXNvdXJjZVJlZiMiIHhtcDpDcmVhdG9yVG9vbD0iQWRvYmUgUGhvdG9zaG9w"
    "IENTNSBXaW5kb3dzIiB4bXBNTTpJbnN0YW5jZUlEPSJ4bXAuaWlkOkNDMzdGODExN0VENTEx"
    "RTBBRjEwRENFMzEyMUUzMjQ1IiB4bXBNTTpEb2N1bWVudElEPSJ4bXAuZGlkOkNDMzdGODEy"
    "N0VENTExRTBBRjEwRENFMzEyMUUzMjQ1Ij4gPHhtcE1NOkRlcml2ZWRGcm9tIHN0UmVmOmlu"
    "c3RhbmNlSUQ9InhtcC5paWQ6Q0MzN0Y4MEY3RUQ1MTFFMEFGMTBEQ0UzMTIxRTMyNDUiIHN0"
    "UmVmOmRvY3VtZW50SUQ9InhtcC5kaWQ6Q0MzN0Y4MTA3RUQ1MTFFMEFGMTBEQ0UzMTIxRTMy"
    "NDUiLz4gPC9yZGY6RGVzY3JpcHRpb24+IDwvcmRmOlJERj4gPC94OnhtcG1ldGE+IDw/eHBh"
    "Y2tldCBlbmQ9InIiPz41BSmtAAAC8UlEQVR42sRWy27TQBSdaztpmlSUBRSxAdEFQt0VuuiK"
    "BV/Rv2CFBHs2iA3/gMRXVKyQQDzElsemYkNRRUlsJ37NzOXO2E5sj50QhIaotxOPPT5n7j3n"
    "ZgAR2f/8eAcPj9V4jWKfYmQJd0rxkeKbV0zsv7h/+Gh3Z7RnA/3kbPr56Nmbx5pAUYKtGzvD"
    "W9/H8VsbBK5fHh4Q7pYuQTlJPCSzpAeNVWqgyADQf0FhhYHGQoRqBkCxkmiJQJ4BqGaAJlGx"
    "skRAY7FaBiQiZ5ZKUGDVMgBSMk41kFYIENZcA2UqiJWwpQHZUQJKgB0bKiyjBLkL/rwEXGKW"
    "ZDKZJSLiQqYZXQsh6Y9JoFc7wBzXcVzPBa/nOv3hhrs56LkD12E9hdUsgSIgCH5pCihvIoh5"
    "EEY8jDIRrzANbUYKGlOK2XnIJkDMNonExVH/qkFA7V5UOlQNWCIfz9KJP8sC0S5ULFPauIaG"
    "/dg04XGUCN9oRLkGpNEw/Cjzx9PMJ2BRPttCAFZcNzWApg3zTjhnkHGZnfnJecJlUuwJ9BJs"
    "gShf1zWyeodRbjNtSNsvbTBLRfQzSMfFrqEG1Pa9et011voAyrkNF41IE2BhzMMxpZ2mEUCx"
    "bG5hWRpYxzNo2LBNhMKPs+lkxkOdbr0SOsq6rgygQUALsk4g5Zj6JTgUK7pcBh33oEUTzTWQ"
    "u8FwQUy+Zqp/LB4sZafPKWBuUL1nDkn3Ya45c/OYP5S/BpGZLtD+LncO9bxBu6mgMg+them4"
    "L6sZWJyIEGGFf//diQjRJKA7ADhWCMiKCMl7anSQ5T8gls6E1AukxvKEEDmRXLFOV2NrqzGu"
    "aYJKi2eEqw3gca5OR8zVvRbqYqmMuFxj9R8k6O4WWGiAEa5by4D2yF9oABduXWMRLDLw6/gJ"
    "2773IHj35dS5c/PKJRsa+PD1h0saCCYvn2rq26PbR3uD3buHzO1dsHImE5mfnLx+Fb5//kkR"
    "GFD0KTYoejaOhBRJcVLivwUYACZZJwaDak8bAAAAAElFTkSuQmCC"
)


"""Button for databases in database list."""
ButtonListDatabase = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAB3RJTUUH3QgQDRcfgk6jPwAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAAM/SURBVHjazZc9bxMxGMcf"
    "596TFCI69ysUUWBASLAwVZ27d0JCSCULCztzlnyJrCA2vkGL1H6BDt060ObIvV9sHjv2ne96"
    "1zRqInD0yHdNes/ffz/+2UcYY/Avm6nfHB4eqssXGHtrzvUL44RfTCaTZgGqzefzvdFo9LXf"
    "72/Xv+OORVEOQZBCGGb4W4BOh2AYYBgGmKYFlmViWHhtiojj8PdwePSFEHJypwOVL0xz++rq"
    "CpOEmCyAJElEcj5jcZyC7ydCCGNEJLVtGxzHAc/zoNv1RO+6LoYDW1tbT9rydB7uLNM+oEX1"
    "fv0CCA9MIvsiKimXS3iAA+zWvfgQBk2fDQhQEvQUcrTCCbppB6QIomacNoybFv1mBBBWiiDK"
    "DboIwqQ45cS6BZDCA9Dnv17/y2qgjQOn4/H4++7u7kEURZCmKShk53kuuKDYwBsHEAePYsFi"
    "/Zdxfn7+AyQF7yvg+XA4PEAicioCpVRCaBFRnKGABMIohXxOgXASCgoqIRjYW7YlIPXqzbv9"
    "j++PvjWJaCUhH7Xv++0knEUQopCcUTDMjkimRi9IiL3nIg0dQcLN1ACDOnzqc0+LWL8AKQLq"
    "y46ohBSqgF67gGr1i8SkiQObcIBoIyI1MXpSuT9sBsVqE2rYCxWESjGrc2A1N3QKkttYhjum"
    "4P8F0fHx8QFPkGWZCJ5YjFOAKIU/QQRBHENGM+FAB1lgWsYCQg4XgoGC+P3T1y/3P3/4tBqI"
    "1EgbQYTXPv59FieQAwowCDpgiMSua4vjmJe5Akbu3AUTHWprD9yOy3NAWXjNW/KqNbCSiFKK"
    "3BFru+K9BfBNR/btnumtqH7VV1kA2irgz8Rj+d0C+HxLAf3licvkhECVA1CektQ9rqbeUgHT"
    "6VQJsFedgjqQFhtVKQyfbS8VcHZ2Bjs7O7za3fsLgCKZvjMyUhVzfX3tcAGXl5ftAvhyu7i4"
    "gMFg4KFiH1/NHnG49Hq9KgeSFPoBLs84hISmIhnBJy04YEogcQ7wQ4kNwTTyb25uuhgFvIrZ"
    "1N+OUeEAu8e4jt9i/0ydhFRxNlQBkf8H0l6mrpXdmu2nWGM/sZ/hM/02Adx6FWt4bas0bmGE"
    "kWLOwoa/OzOLOIBt5xoAAAAASUVORK5CYII="
)


"""Small icon for left merge button on merger page."""
ButtonMergeLeft = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAe1JREFUeNrEUz1MFEEUfm92uOO4cJgYz3AEjQnRhgC6ibEx"
    "VthRWRAIdnYmxl4LayoaCwqojlgRQgeFsbDlLnEbAW1MOLyD3N3esjc3+zO7vlkXc4UkJBS8"
    "5Mubef/z3huM4xiuQgyuSPzZeiM5IOIqMTOV76owfHduJBwHwiAAZhjJ3ev1wBMCfM8DTobn"
    "dpPzz0sPEZBtfflthHG8TLKOVgRkiOz/xTKtTFFrd3zPcRSM5HEokHLGp0ytej3JdGEPXNsG"
    "X0qNb0f1M8d1A7h7O1ckp9ey273Zb0wNXyUs9cuM4cdLSQVErY4jZ0eL+VvHDXsol2U3mm0x"
    "TQ4n5GmHvu/R+QM9+QlxQdxSSgHXUaMoMiU1hRBV91nNKxTuBAN8rHS/ONuuNaddW4gwUEo3"
    "svRowjyu/nybjr+MY2929169fGpaXQY1hSDCGI7OFESkZwiQH0AY5JiUq2XNXgTDGbp//loh"
    "0Xuux/ErMMCKMqDnoWhSuRHi8V8HQXD1OQ3QzdJUOAUmv2QPqHmVzY+bl966wcUFk298qviI"
    "K9D6sYPZqRdw2XVmD+b2kqecHqyo/e2yanwH7llJdioKMoSsTkLIpVzLjH8jK9w7BGlXVWXN"
    "ous4oYl91RlpoH4YF/wXRZB6y/Haf+MfAQYAtE0LahFUAvsAAAAASUVORK5CYII="
)


"""Small icon for left merge multi button on merger page."""
ButtonMergeLeftMulti = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAB2ElEQVQ4y8WTMWgTYRTHf9/d"
    "eUlsS1PFFKpFBJEOlaq3KRK6utqloou4CeLkokMXwTplcQiiLhGngqMdHIJjm6IxCK0FUWIa"
    "CdV4vV6+u+/uPockpShYnfrg8f7vPfjx4L0H+20CIP/0WxFwejUDSHp6sXx99O7fAAZApNTk"
    "5ekjZ2emc04cqVN9bRJfulCsD+8JUEHw9cfPMHDdGENHyfd2qFw3ZnhAHFRSntkTEEr5rt7c"
    "cj1PcWjQtL40XO15iuOjmVwYBDdP3397GGBqvlacmq9d/RPQ6bx8/6HZkGEQ6SjMrK63jE4Y"
    "JI1mO3sslzofRdHjibmlfCilE0p5e2JuaQciTtwpF4UQDsDY+MhkNJK1FUIcQGuFEHqjtea1"
    "fT9ScQwwdu6k01hZr2itC58e5kvi6K3F5RvXLjrVbYOPAdS3YhINhoBEw5AtSFsC6OabnYQh"
    "W8DrNxXgnhX4Pp+VSTWxiSwYzEKsu55o8DV4urvXRMN2CpQFA74PgBVKWVl4tPBPR2MA6Suz"
    "jvX8RSUUouCWZl+J/7m69MyzZYCktVoIyw9KAFavZwE2kALSQKYXbcDsA9RGbQ3ZXokrT6rA"
    "OLApdk1n9kC73eyv+jeLAQn4+/6M/AJo6ded0dcRQQAAAABJRU5ErkJggg=="
)


"""Button for open file on main page."""
ButtonOpen = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAZFJREFUeNqkUz0vBFEUPe9jyQqKJWyi4gdoKEgU/oNKKxKl"
    "UrI0CqKXqERHIxLxA0QnolkEWWJCLLtLJozdnZk1M++5M1Z8hGRkb3Jzz8u957x778xjWms0"
    "YvJwif2a4Az9FMZ+SW0pjeOPeyX5BHnvzyoqwsCMmrMcB0+2jUmjA6ZpY/WolVN6kHwtrGP7"
    "C1gYmt7OIAigtYJWXz14j2GujhljyK5PLRJ3NuogUBAhWb3WvhHVF3KE61GIBCJOSOZ1ge83"
    "/02OOmE8Ekg2UfuhiE8H5Xswc3uolR+BUIw29O6KzhTVJ+acIyEwQuR54l9LEpWuVYJPRenh"
    "8Vifrmd0Yqj6YHSfbWaWpU9jVcw7iJYU3Ici4vwXjMZ4zB3kXR+noYB8ujfQ1TcAnxYZx0RT"
    "MwpXJ0Xbxbn0FGTFekGaJWILBJ5bLtwYJcGRl75PO0mm4Ll2tPk4Zt7nCpaDS4JaEmhvbeuE"
    "oLkgeIz5GW6Ni+eqi2OC1L6Dq52Nld3/PCB6J1kvwF4k2Ohr5GjQ3gQYAE2uCzTGoVQ4AAAA"
    "AElFTkSuQmCC"
)


"""Large icon for open database button on start page."""
ButtonOpenA = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABHNCSVQICAgIfAhkiAAABYtJ"
    "REFUWIW9l79vHMcVxz8zuzxapC0QIhgVdqEICgJRBCI1AgkQTApGQf4BFWkSpHD+gFTsCClR"
    "FTXpwi6KCgEs9AdEQIQAOUixCkeFITswFYuM7ACEdBF5vNudN++5mNnj8RdMWkgesRiSM3zf"
    "z/u+t7tHeIu4devWr27fvv3Lt8lRfNs/vHHjxtylS5f+cObMmZ9cvny5/eDBgxdvA3KiWF5e"
    "vnDnzp31EIKFEOzu3bsvb968+b3/i/jS0tLUysrKP7vdromIiYh1u11bWVl5vrS0NHXSfCdq"
    "wfLy8tjZs2cfXr9+/eLIyIiLMRJjBODChQsT6+vrP7169eofHz58GI6b0wE8evRoXlV/670f"
    "GWw4t2dVVff06dPpa9euvTcxMeHMbG8i5+h0Onr//v2/ttvtxdXV1XgcgBLAzH5z5cqVBVdt"
    "ETfXDz3454+eMTc7y/j4OFVVHXpmbGzMLy4uzsYY/7S6uvqzYwMAI9576s8/ov7kLwcOPdl6"
    "l6npBc5MTtLr9Q7sNy4BTE5OvjM7O3v93r17C9PT0y+G97335IJ3gF/PzMz8oxzeaM0s0ppZ"
    "3JP88ePHfPrxx8yfnmBjY4OJiQlardag900URUFd13Q6HcbGxorNzc33X7169f7c3NwBYFVl"
    "bW3td8CP/TdZdP78eRYWFvDeMzo6Srvdpq5rqqrac4UQaLfbjI6O4r1nfn6ec+fOHZozz49A"
    "bsGwhftjamqKqandu+vZs2fUdY2IHKjq1KlTXLx48Ztqwszw3m8NAE4SZnYkwP4746hQVWKM"
    "nT0AR032/hCRIwFE5Fh56roGeA1QPl/+0Ttf7XRGvnzx+bEAqLuEbucAgJUlVF2+/GI3z1F+"
    "aFR0+7/9Jx9+OOI++/3P/92a+uA7OF/ifJoH53Deg/OAA2egBiihFgrfWB4hGphhUYlBKApA"
    "DNUIUTERiBGNioWIRcFihGrnTej3OmU5Nt5693SrtLANRYErCpwvcIWBh935NEzzFW2Q2CQl"
    "1VBjTrAQsBiSWAhY0LwKFgStBa0EC3I6+NGtMgbpoQENOzgrwAooylS98+Dy68IMU8UkQIwQ"
    "AiqCSRYVST83YhKzqGJ1A5IAElxEnfVKpO7jy1SqczjnkvM+tcEXDnMOokAUzCJmgpmACqYB"
    "U8GiQMzVxwzWQIhgomgIWG6LSSCK9kvVsAMGmvqZvmJyIzqid2mcVLHGcgloSKJN9SaNIyG7"
    "0JxNANq4UgtWC1oFJMSt0kL4Svs7P7C6j3mPKzzOO8z7PTNgqmA66HkDspu8sT9ky5NIEgxo"
    "nWHqZH8MhvblZaka10J3G+oernDgPc57nDPwLrc/TToaE8ig4rArKiFZHIZ6L3nyQ+Oephao"
    "oVEhFM9Li3wi3W187CXRwuc7OF8WwRTUUq81g8ScNKbvB3dEIzx0ERXE0hrTGoOhwT4tw6vX"
    "n1Vxq9tyO+NmIT8D3NBQAjiMxgVLjmiqplmb6pBDACS3buh3UdiJpv8qw/bmFxVj3WI0jmN1"
    "bkEDkMR3H235odOAxGZNVVq0vRBxGECzOwkk1n6byEZZVdXLgkLMwFTy7ZfF978lbQhCSdXr"
    "EEzMotGOhpDUThELvAkbDuDJLy6tnX7Pf9dinTQbB/YZsAtAEt7jRBbOLlju936IJl6/cc9/"
    "+PfN8yWAVKEfPKCShJs5dEMAth8iu9BADDkxEG9aI8r+N3VdWx/y67iupNcXwaHg8pMvV+8G"
    "NlhiGHJhdyCHADSLa556O/ytGMSqAUCvjnUZ+xhDvR90YciCJtmwCxkAZQAxrHiYuBlUZruf"
    "iP7zJvxtpxj5vqmhhhucSh6nBcNiTqmkvQxizSOj8VlBm3P7woNizm0Hezxc3gjwAdDiW3xM"
    "O2EIUAMbwLH/g/qfxdfI9P0zcP/UhwAAAABJRU5ErkJggg=="
)


"""Small icon for remove database button on start page."""
ButtonRemove = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAhFJREFUeNpi/P//PwMlgImBQsACY6xiZPQAUi1Q7vF/DAy5"
    "v4GMH1BFbBBam5GBIRjINALibWH//8+CGwBU3BJRUmL84/v3/zdPn5a8fuqUJdBzJjB5ILuT"
    "gY3N7fevX++AhvQChbaDxBlhYbCAkTENyErTNjeXVNTWlvr57dvvM/v23Xn96lUsPx9fjqiU"
    "lN2DGzeuAzVnszIwPATSDFFAvXAD5jMyMvxiYDAGOj0cyHUyd3RU4hEQ4L937dozdg4OhhsX"
    "L54DOjcdqPkFK8hmHAaA/fwXaBBQwUw9Q0N1BiYm9tvXrt36/v27DiPEK2AgDjUAHgaPENEi"
    "ChSsVZKSkv7x8SPbradPbwjy8vJ/+/Pn0pffv/VAlvwB4lfo0fgDEpAgyRoFUVFTMWZmsWtP"
    "n97++PPnxPtv3pxlZ2XlAspt+4crHYAMAJqeLsHL663GxCT15NOnF29//lwIdPJcoMagR9++"
    "beVhYZEHstt+Q9SiGgD0t/xPBoZUaw4O+e+/fn098/HjeSB/zieg3D+Iy/Kf/PmzFmiREVCz"
    "I4YBQMW5hlxckjx//7Js/fTpPpBfD1T0HqYQakgd0IAzQDmPH9BEiGyAgw07u+SlX7/ePv/7"
    "dwdQ49nvQPHvEDm4IUCX1gD5r4De0ENJSNHApPwfkpTfMgPDAhhlD5BSIQNa4CkCxSSW/f9/"
    "nHHAcyNAgAEAxF3igbzwV7kAAAAASUVORK5CYII="
)


"""Small icon for remove missing button on start page."""
ButtonRemoveMissing = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAAUFJ"
    "REFUOI2lkt8qhFEUxX9r7z0mkijlAqUMKbnwPN7CO0yY8BbeyBQayh0Xahokme/bLpiPD6Vh"
    "1en822vtdfbZ2jtoH0ns8gdkcsx+p51lWf5p7HfaGW9KSbfbHSt7q9UCIAAkVQfjonJweX1e"
    "v4gG4Y574B5EBGGORwP3oNlofgiMsLiwXK0lvc2IJBklAnCzuoMaBFdz8wCsD/pczMwCsDbo"
    "k2WSWWKfBOwr3+SsD/oAFXnj4R43w8yQrCZQc2AmJNUCAMKdMhOUIJCsek4tUjJMxtn0DABb"
    "z88AnE5OYWaYCUOY/+BglFUSm0+PyBwzsT0cUiZkFmSRyITLKCjqApJxe3dDeBCjr3PH42Pv"
    "HjQmJtBPNVhZWv1az+9IGL4UDF+K7wK9Xu93gU/4dys3m++dmHDSOdzbGYv9jkyOXwHmFJKW"
    "bHJGnQAAAABJRU5ErkJggg=="
)


"""Small icon for save as button on start page."""
ButtonSaveAs = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAf1JREFUeNqMUktoE1EUPfOzEzDalQ2FBjV+tuIim5ZOFxYX"
    "igqK4LJ0U0GquLLgQnEnUmvppl20SLrSTV1IF65KYxeCCSglgi0IGmokaLQxDm/ezPPdl86Q"
    "SavtgTtv7pt7DufeuZoQAsPmExA27/7Abqj0jEzLY7nzeWH+00APzK37ERkZGR93E+CN2lnj"
    "p7joHejIynRUCQiI43dyw7dPXEvK9zgeLH5BqeJGeVqGc/kYZic3r4ogeLMlEOiMMXDZju/7"
    "MYG361WcuXQKH37JFnnzbqUK9A9luxae5m+FDvQgCMAk2fO8mMDhTg0vcstRrmka6v19yD9a"
    "+OYetCdCBwbnXAm4bQJjF47E8isPV8BzL9Fd+PoO++15JRAI3yDrTLpoF2hH5lAHTlpJlErv"
    "1yhvCkgHJPB7g2Ffl/1fgXvnT6OyWMOq5EQCYQtrzz5jL0gkEorT4qDZQrFYBC3WTqDhhXAc"
    "R3FaWzDJgWEY6B3NbPsTlmXh9eQ60uk0yuUyqJY4rQ5McmCaJup1F0vjq4qYvX40skzfCLZt"
    "q10hTiTgg1u6rquiRoMhlUqpYnpXMxJGTIB2hjiU6/TwhDv1eGzmVXXje40xoYophGhGeNcq"
    "wMSfpAxoNLRz+k154Q7Kjby/ZM7ldxqiw4f6YkOFVpDHDe1fU98r/gowAMaC+CTtivcaAAAA"
    "AElFTkSuQmCC"
)


"""Small icon for scan button on merge page."""
ButtonScanDiff = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAAzJJ"
    "REFUOI2V091rW2UcB/Dv85y3vJ6kWdIloTZpHRjaxq6FqRXXiyzaCo7pbhRRfLnYjS8l/gNe"
    "Kd45UEGrU2RDigwdtjAJdVPHwItRjGtr2mXdurV1WdpmJk1yznOe8xwvZoMvF+L36nf14Qff"
    "3484joPdDBw7q0uE5BSJZLlwBgFApqRg2c6s7Tjvzk8+XsM/QnaBvpe+yqiUTI0k48HBRERJ"
    "7PVD12UYnKG4UeUff7dUZVw8s/jpU+f+BaSe/yLjVZTpowMpz53tW8ifn+Hl9ZLErRZi9/SK"
    "N159UXp4JI1X3vu+ubbVOFw8+WwbIfuePqHLlJaOpPoiv8zP2UuLl0CUej4eshdTqZ7AxUur"
    "3dVNdujR7Bh54bkj9LUP8xXLFvuWp16uAQAFN3K9Pl9g8/ZtXLu6jA7P1bcOPRA89dnkO9oH"
    "x99+4vz0+6EHh/3HL1z4yZmbW8ZYOhGEbeZ2N6BUsGzU41Xni79ySW3k0/3J0siBAyucGx8B"
    "+Nblcl07ODK0oXpE/vTMD3Yy1KEQ28zuArJlNgddRIFgQnI5t4o9Pfs3h9PpNUUx6+vrC7nG"
    "lq3v7Pze75G2u5o1c1wlGiyzNdgGhGWAWQySrIFC4z6f1/DsUU0h/IYQgBaoKz6f11AVjRPH"
    "jVq9BcGNdgsUghW2aluQNc1m6OpnjCmsyqVAgCmBwN2ZMaa0nFi/2++3VzfKgG0W2gC3zNm1"
    "6oYZDumy133f2OWl7eTC9SvRcll4y2XhXbh+JXp5qZJ0q/eO93Yn5OLNks0tY7ZdY+zg67pE"
    "aGkodn+kaYBbJghRyl93x+U8ANxYtx6zzOBRv18j4VCA/Fg85zg3zh67ubrySfuQ4qMTGYXK"
    "3wxE+ryUathpCk4cWYLjQFBmd+humVIbhd9+xuEhDTOnP281mq0nN8qb+fYpx0cnMpTQqU53"
    "WN/jCWk+1QdCKJpWE5VGxS7X10hUq4jRgU450uHF5IlTOyZjGfLXZ4qPTugEJCcRmhW4+0wU"
    "pGA7YtZZObNNeOPNaFdCeuSh4aAqQUx9eebO34D/SnxveBzAyVAo2Ix1hrsXiiX8L+BPZD+A"
    "aQBdAFb/ADonfMWfJE59AAAAAElFTkSuQmCC"
)


"""Clock image icon for new days in exported chat HTML."""
ExportClock = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAAsAAAALCAIAAAAmzuBxAAAAK3RFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMiB2ZWViciAyMDEyIDAxOjU5OjU2ICswMjAwk8LXDAAAAAd0SU1FB9wCAgAAKExUReUA"
    "AAAJcEhZcwAACxIAAAsSAdLdfvwAAAAEZ0FNQQAAsY8L/GEFAAAAx0lEQVR42m2QPRJEQBBG"
    "29SWiEgkE3EAcwGXmTvZy7gAByCSKAnJEEnGW3bV7pYOprrfN/3rOedEZFmWpmn6vt+2zff9"
    "JEnyPA/DEEn4MQxDWZZ1XVtrCXnxIfBXPjHBNE3uMPzTgeCjKopTMIoi+TUIHFXRO01TuTM4"
    "qmK090Qfex6GA0d9MDmLXJ+MMddXOKpisa7rbrvAUZXWum3beZ7/ZAgc1WOxcRyrqsqyjNFo"
    "R3GykYuiiOPYO2+6ritX+r4p2UEQIO0jepg0AJyJFgAAAABJRU5ErkJggg=="
)


"""Edited image icon for edited messages in exported chat HTML."""
ExportEdited = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAAsAAAALCAIAAAAmzuBxAAAAKnRFWHRDcmVhdGlvbiBUaW1l"
    "AE4gNSBzZXB0IDIwMTMgMTU6Mjk6MzQgKzAyMDCZinEkAAAAB3RJTUUH3QkFDCAbTkEjOAAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAACWSURBVHjabZAxEoMRFIQT"
    "k9GpdA6g0jmHc7qAnjvoVCqdhopG3h8zIvJvY+d9O/vwHGM8Njnneu9CCMbYnKAdAwshxBiN"
    "MTnnm4T3fhoooJSeCShYCc75mqO9AEJgCCH3CbjBNFLKfTVauNb6X/BNAJjPO/Al+I/WmrW2"
    "lJJSAj9+dXVgjGGL1hp/dFS85gFAKbX+YNcbtEZUbc4oXmUAAAAASUVORK5CYII="
)


"""Removed image icon for removed messages in exported chat HTML."""
ExportRemoved = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAAsAAAALCAIAAAAmzuBxAAAAKnRFWHRDcmVhdGlvbiBUaW1l"
    "AE4gNSBzZXB0IDIwMTMgMTc6MjA6MDIgKzAyMDDonN18AAAAB3RJTUUH3QkFDhQehoXwLgAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAAA/SURBVHjaY/z//z8DXsCE"
    "xp81axaaCCPQDExROEhLS2NiIAj+g8HMmTOxkkBA2Az6qGCEhBhWDwO9ilCBBwAAobAv6s9R"
    "MfoAAAAASUVORK5CYII="
)


"""Help image on default search page for chats page."""
HelpChats = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAH0AAABHCAIAAAClA9+5AAAAKnRFWHRDcmVhdGlvbiBUaW1l"
    "AFIgMjMgYXVnIDIwMTMgMTU6NTU6NDQgKzAyMDB/0ECbAAAAB3RJTUUH3QkDEhAxeW8NuwAA"
    "AAlwSFlzAAAOdAAADnQBaySz1gAAAARnQU1BAACxjwv8YQUAACZzSURBVHja7FppbFzXdb5v"
    "39+bfeFwuIgSKZGULUuiLMlWZMkJ7LixHa9xChTJn2Zpi/5pkeZHAxRFURTorzZAW/RvDQRB"
    "07iA7cJIHDtxbUtRvMTaKVPiOuTs25t5+9LzOJRILZY5Q1tCg5wfM2/u3Hvfud8995zv3Hux"
    "Cx+c9DH0e7mTQviIBNC9//+4u65r2XbnGcdwz/dufqYoiiSIu61pIBjgfrd1+GzEMK1U30DZ"
    "wgXCc7WmLMu6rbMko6otWRRdXcdo2vVdra1SFH2bfqq1uiTy05fmbKtt2Fg8ItRqTYaiHM8l"
    "GcZQtVQmVa83mo1aMpFq6TruOsnsQDQkdavw7wjusGJJinpz1j/Wh4kk2bCb3z35l4/Gjz07"
    "8CSpacK5aT0e8QcynbqXP57WTBfzHQ8RoiS0GnWBZafnc5MTu2zTANzj8Wi14pq2bjs+TuCm"
    "YzuOU6zUY2Glpaquh2QlpOmabpg8RVcqVc+2utI2Hg4HuF+6dIUgMFiGmmHHFGqp0PaRGw/x"
    "87nq5ORouVCgaarZgtfYJIY4gTUtLyzg1TZGYg6GBbYWDodxgiCRWyzXKJrrz0QJipME7k5C"
    "T2AY5htJgWmrSKLFg+E9u2PjUG7xvHpgCndtUK9Tk2YY2zEQDt+8Igsk5nO8OMmBwnzVNKCC"
    "63lffPTJz0/V8tJsgHtIERGG6bBqCLyh6ol42PPcZDyqhGPgGiVJtB0vkYi32joN84MjzgXF"
    "3IFsVK1XaZY1TctHmCwJuO/6OIl8xLAC8v07CTpF07ZlPL8NjNjCMYR76M8nvgPlhmHATw45"
    "MC2WFeBerVYsx6OYwNt4nlOrN+DBVtXVv6o4jgdTSBCCIHx+2lYwHLty8bQYjn62/dZ0ZDhr"
    "zwrt0vgdnYOOwEIkyO68qN5uGXq7Uq0KghgORz4/3axWi2RYPp7o833f865yAByHJbuVfqP+"
    "rc0dQ/7WOu5Ouh1FMb/0xhu/SCaT7aZ6ZXZO4DhB4NRmg6QY27ZplnEMEwbguTYEBlgYDEW3"
    "NI1laI7ntbYG78ORbzn+0GDm9i8ivKtx9dL0xReefooHl+KyP/npjwYGskvzC4wggOdhabqQ"
    "L9iGFU3ETFPzfZIicZJlouHwJ/X7vZ+jX84FDyNRlBbQxyVU0IOfuyP6D/bm7wzoAWtE9tmL"
    "C4l0bDjbX6k1eJZ2gZcg5HsuWBmgmUynSHx9bsCRbhvZgWO+Z5u4ZauNhgc01POKpZJnux7u"
    "N+utcDiUHcy0wB0bDkvRpuWYum7abrNWC0VCsGCUkLwZ9dZwdxyvrZmq5pcri+Cga5Xy9MVz"
    "EKXbun7P/qlmKV+qaLl8iaEdzyaWFpb7tg994cGDn2pOqoEoDMWENdzvsNQbmswz0VjcMA1N"
    "La8s6CRJWJ5fLhVd20ql+qOJBImvM/p4PCYKXEttSHIolujr6l2uYxcKBU6QPddyHHd1bfsM"
    "LBHTwGnKtjyKJluNRiQe11qq3myu4e55fkvzOc6ORwVoIkjy1KHD0JggKccyMqk9Q7oJKwtW"
    "Ls2wR46zn6qHxKIBCcGYTpfQgXj7R8cLaNXn3jEB5ifLkt4yiivLgiAPbxudn1uSFQnHiexA"
    "Vm8bsZhSWs7FU2mWoW5u3q2PAqLBsBy4B4pkKcq7Vg7sFjBEvsuyrO+6BB4AaOIqefVvAkMu"
    "SSA8WKGErrXL1QZL4yyvIM9uNFVYlSTuqy0tnkhuRg/NQQIZmHmEgTH4BH7nEF8fM4nXajVW"
    "EiKxVK1Wh7D18YXzDCuXm/WhbHb6/Hma4lgpdDPurVbL9XvxhzDZtyi1g0LbtiAFky1W1yuE"
    "Y2G5uZm+wRHwYsAjO1PMclyHTvUspoPcDXGVwHzqblCabkehNmtgcOBneEEkSBY4MYmvkgwc"
    "h3gAn4q85rst06jVmzzHIJx0IRiQtO85wEvA+6/PgW1Xa1WWF5EX0G74N5erzcyIDz1EmY3q"
    "mr2/9au3PnzrtbTs1TRXHtr39a+/AAwHC94IIdr3fL+rMbw2g8bEeox1Nt/kBnFX9+pgwnru"
    "AZRnaApSSiUcpcnuJgACL/hoD7wDpAIQZwEKhHUo/xqmkMG6rmVZPu56joM5LoCEYdfRVkgO"
    "oBPLtMCRBL15vmmavg+5ZLC81qq+/8H7v/jxvz53II25i69fevbg3r35chFqgLeJhsV0OjOy"
    "Y3jzev/vAuLibVoxe0btNwVB97AHUy2815AAuIcV6Y2f/2zfvvumL8/puheLSsi1FxeXBUnk"
    "eWFsYvfwwK2DJzjVSCy5ip0HBnftE9hk5wHId4JmO8+dd/mrpnmtQmCyBBWPxzc2V1UnFOzk"
    "2Ou4k1DRdnb1j55ZaOxMEJmhoWgqCRwc+AxD06Iodjvsy01m+xZw35doo2A3secOAicD9nbo"
    "yEMCz0yFY5AKASjALffsaRerrXQyDu70k9pWK+V8vjg8suMG3DvdbqawE5Y7k7GhpjY8bF88"
    "vxCTpTXc789g/6NblwpLv/yo+d1v4bqmAuEFV+fjtGObJOYObZ8Iyfzmh73Qojdf+Wb5qMK3"
    "LWJ/SmV7DTQw1EatupyrIIFlMSIS5mc/nqu3jEg01Gg2CzmlXq4wknzw6FGZvTGuGobWaKjD"
    "27YFE++765/o+p+3Kww8JHZ9ISB54fxZivBlllmLqytnT5x9+/VIMjq9WD06tTtz6LpdIXBk"
    "NN0Fjj+7DCmZuTembQX6LUpgYkFiGcQ9luOBr/kYAYDCQAJvQFCQHgXbSrJ8jTFei6uFlRzC"
    "KVm6tZ2pzSYniOQWKNp6vhod3XtoaBIWxrBhBHzzeukKdJADGaA0DElSHOl11fCzFfKm/Rme"
    "39QWKZDO5UJJrTV5RcFdo23YpqaTHFMrVeVQaOeukcKZC5ppD2T7R3cM9ahb5+uVl//7P/79"
    "38KYXyzkRw8/9L2/+v5isTE4lPHNdq6oQu9dufi/eyvYJ/jaSP2Z4XpvalUNMqeRg5IlUz3O"
    "HEVRzfJKvqzvmbo3v7wiiUIQ+xCmmyZQNAIn+gYGWfrWG2c0y4+PT/rIf/3ln0IdoCIEhu07"
    "/AhJM/jq6ojF0p2aDdXoSitD14DVpGPRtRcv5vJGIf+lY4ffNHXT9XCK09WF6fNNzzIcxDRS"
    "8R5C61Y4O0d4cdYxHRz1ijuIpITbJum7NjjwQqHEUISh6cVqFfLGTF86lOz7RNwZVgnHIMN8"
    "97X/6k+wZtN77ODEA4f+huI3tfdyG3n7V6+XSoVkJHwtX6UMkq011FA4SkUiSkTZNbkTzMM0"
    "TI5jHc9dXFwKRyKi0EVoBRbYs36WixcNclTpzpo2CpAZRZRQofTRmZmQzI6NjsxdmU8Aj0ml"
    "IMMxdUtvlGdWjMzgIMfc6EVhYZw7d258fJeN+wUNjSdjA9G+UyfeocOJYq4US0QtQ0smk3OL"
    "OZ5haV44sH/PJrUiKToaS8KMruHen+k/8OjTs+0GFmOfee754OgVcizfBxcJlJ+mKUgcNp86"
    "MQTiqWCrFMN6tFaCwBKcp7kkR7q99QDaEhQdCodZ3aVYBgixKEu5hQWOVyrNen9f30ouR1Ns"
    "WzNvxh2IyIMPPBCJhnWfT2QesmgXy07cO3XYcvxMqo9haHAXgihKstzSDJbpIviNbN8OwGr1"
    "2hqf6W1sv2Nyjc8sLi0feuALUPJn3/7W2O4HQqFQLBp7+OH7aXqrx9GNWhkWE+B+Na6+8sqL"
    "L77YeT5y5Mg3v/FHmuHEYxFYcsGZrmXSnEDgmOe6pu1wLHP73l88jWaqCNaS5aAQbX9tpHrn"
    "QYTMJSSL5VoTMvh0Oo13s78ISzs4LUYoN78wFo1kIhMn33z72NG9tLjlwz8M6xwJreE+PT39"
    "nz/58ehRMuZPnDrF/Om3v/neqXPApjyMmto3derku/19/S5y9WajZlhPffUJ4rap5G/zqGmi"
    "ARk1DXQh734l1b4ruPOk++H7Z51Wa+e+8enz5y0HExhCV9v5ZlukkCBIB48/kokrN7cFV/DS"
    "Sy8psuQ6bpTBCdtgaGphYT7UblWKZSDvDIkTDGPqerWmErgjKTFDa8uy2JcdoDbH69dwpykq"
    "MkZl/sCPliipHUEEd+99Y612lucFyKe/8viXW7pFEsFcUTS1mX4/WEGXqxAeEYVRry3Kj2ab"
    "dx56D2f37p9wXIzEsceffArIDLfK3/V2q67q6VT8Nm1DIeXA1BRE4EKjldA0VTc926g3gbX7"
    "uKmzrDQ/e0UUFVM3FIUrFfKiKDca9Wgq0x3ulm2Xz9tvfh9h5Ad/+Oyultqq1OrgVXRdj0Si"
    "0WiE4bpgMoHeIno4gxomTABxrsbeFdzLxfy7J04ypBTL9oPXBj5TKeVZVik0an2JFHLbro32"
    "HT2WTd54hA0x+YvHHoYHYfeXZyP9lGNZ900Nju5WNmSwAwNDW9FtLa6ePn36xIkTnaLR0dFj"
    "x45tpdOTS2hZXf+Z4Ox7InfhoA9yjiBVCi41+WvUanVz299w6L7xEP/muNpqaydPnFDV5oH7"
    "D2bSqa0fmN0YVycnJycmJjrPW7xMAJKRUL/oxrmN++9Mz731JmuHBxukq6uR7733vigK2ezA"
    "zMwMEItXXn75+eeeEyQZki+1BT4gBIUESeqaxvPdeIIb4ir0Uq1W5+bmUqkUZARAYArlCkOS"
    "pXItGg17sCBxolapTe65lyY/Xf9/+jWi7PYf7yr3jNq8SjctYjyiE73aAEVR7XLNoKjhTPzK"
    "3BKkIKA5mHujrQX3eTB8x65JnrmOF/qBeP7q1u7CwsKBqf3gZoeHh8PhMIAze+VKs9FI9vXn"
    "i2XcM1q677sQJ8yvPv1MpJud2o6svxi6bjQaYCaAu2U6pfwKxwk+ImqlYrVRh5zbtT3P22zq"
    "37Jxx0NdnvOsi0B5EhW8rWfcAUJe5kNSGMw+FlU0zfaRb2ia1mo2bXegv8+ynRtwt22rUavM"
    "zs1zvPjEE49Dzjgzc7lerwP6QOEPHj7cqXbvWmUbprZbrbzVq3Yg6y/urJfO/TRBErZtH6VI"
    "wjANDKfS2X4YhhSKsJtOHE4UxW/YlQjTY7aJediVNjkWdnve5IGxKZLw0QdnlLS8slS6//59"
    "C7PzgsgnOd6wLMhOSrm5RcPaNrZTYNcSTj84UbIC41pN1KGEoentgwORUFiOxyzrxmOcm0s+"
    "VbS2WioWSXT1SBCmFCYWoDcMQ1VVmF7gi0C/BFI0TYsiGXBKm0894jwaUBBMEkX1aPAKj+9g"
    "fBynqV73xWDhEhSzY2ykabXHx3cCqLFEvLiyzPESGH08FrPNNsBqGNY13De2vXhxWhD4tOVw"
    "r7+Fi6L02CNUKr31yLdm7AT5+32CdSkXlwsriwuLOV6QVgoVoBrJUx9SO0aEiXG9XJ5VK412"
    "cCFpeGQ4m0n39orLl84UC0Xm2s1N33/PR++ucq66748S+As2+HgXcVwvPORffhOkrH9xT17q"
    "1VodD6tbRIhxyC2ca7M0BYs42T90gx/fTNvjx48psmw7PtmXJsMQJkLb1ARO0kA7qS6vu95S"
    "rt4X8znNMEuluaWFy8NDPkcsvnPqt47tQXJx38GjZ069W2g2+5JRD2G+pW+fODCxc9ttOr1S"
    "C/JVdbtJ9erf8y06Z6A+wU5zvUYIDKuWC0uLS7amn78801TtWExGtjm/sMRwrCyF7tl/YHRb"
    "9pZtwb+Dp4UHrS9hqE3Wd4GUKqG1HQXwFZ0r3T3Itczh6v2ZN8/+w9//UCD8SBLtOzj+nT/p"
    "f/SxJExs50R8MPtU4N9X6XBwUQT/dCq5N9aWqB6VA+EodzvtKnTvPRAEEY/HSVokGXLv3v0D"
    "A1lIwkVJdEy9qhqywLHcJ942BJQty4aZ+9sf/PU7b/86lUntGNvzzz/8x56V2dDz9biLCjYy"
    "xB4/IlXrjsDjrUZ9ZnYOMMYpJhJSstn+DW03FSqjjNtrTA2kbJAtixiLaj3fJwDUivn8hQtX"
    "IpmUo+nNWnF5sTCfW0mmkuVqJZvJlBYXaUn50uOPK9yNdBBs69VXX00m/o+dK42N67rO3+wz"
    "b/YhZ4bLDGe476J2yVptS97iOInt1I1jw02LdAkCtA2StkB/tChStAYcoAkQtA1gpDAKNDCS"
    "1kFrx7HjXZSshZRoiaRIcR3OQs5whrPvW8+ZGVKiRCumZDRpmwuCGHLuu/e+c8/5zvfdd+8z"
    "h6MZaujChfHO7qHl5ZXIWpCogkKtX3bNaE3WTIrkgahYKFmbmhqt5m0Nr2b3zvbDf/YXP5CL"
    "Jcl8Wa2oV+v0Q0ND5ODVXWPbveff3QlxWWXVfKKdlFsWa+3SbZ/X2ig0eLvdPrBzz8Z/Boa2"
    "cbmjxT44OPjP//Qv6UxOqRAb9CqxqBRP5Q18NEbc4mglXanVakXlQigdVyi2nQVrdjcYm+nn"
    "1q/vjDn1W1DZjbZtWfFrUmjODvJ2aBh0fALApCd8Ulobmuhno05jcw0DOu6oi/8j5/k+3ULS"
    "qaqJJsZHv3LEmkhH337tx08/+5zFYrnLlm/Oq78pN5Vz586rVMrFmQlFi7PVFJ9I+ddCobp6"
    "S7FQIOmj1evujN/enFd/U24qXq/38KF76iTI5yCoO9v1Ab93aW11NZbOphO533r6ibtUrmx3"
    "CqtoNPqrvtNfcdHpru+Nocz5+ONfEIvED7Vj0rMcz2VkxfKxe/YqjNs7fHObwnanNGL8+ENi"
    "/w9LZTO7qIzy3i99a3ElkM0WUl5XPAfR9hfCbmm59oHtXi4Vk+mMXCaTy+UloqPE0EtFPuzA"
    "ZwYLpTIKxdKWu0TyuRxdQ6Mk0OPjc7/2heyWzUOrQaGA26h9Mvrk5FWtRvPn//DDL+/pl0vE"
    "//X+yDeKkMsV1ZMb0k/wEOJjWq59qNo9/4/f/a6mvsFs0KpleZN9wLc4L5KLyd7pZLTF2bHg"
    "9au1+h6nZTUYCgQTxUxcLFVQlpeUJTv2HXDPjvr9uTqLKRENK1QqIv4jly7TzBG3Ver12VhS"
    "xKe8RYJSdvLhx7a7VPLplvkZrCxjLYUGE+xOTF5BZxvmPXy+e+jQ9WqkWqampgYG+sXFsiCR"
    "mjUCWXz4/fdozjQqWSIveubpJ7e1MeTWUsEZqfIPvv61QqGYz5fkMpGg0dsa69l/KydsCPg6"
    "uhKCWi0q5Q311n6ZXCoqp7J5pVySSsTKcl2z5QRNA6/oF/MqQZBKZGaLNZ/NlkisSyW5bOba"
    "tbmBgT769m72Ln8qxVCHaAJNWkhlSCRhNCEngkZAhoJAjtj6vnHy9wceOKmhW1ZrF0oK5IqH"
    "P/OFEw8+VH04wY9k73ok67rJsOmZus22SUNtHNbXrm82qR6fMtVtLY7Vm/f32O0tv0pj31Aa"
    "GvnnlxZ+yKNlqfyd558viRXhUHB3c5NKqVw3eDm/zRdw3NBy7cP/AlD+ny/EZ8LhiEQiOXHi"
    "hMvlytsb2tra7v6YI27l78UiSgVMz6GnGwQG1VcViCsfJBK4FqAzYOQiLCZ4A2h3oLunVmc1"
    "yNskEiGUtWht4vmk+okowkmMjWKgD2OX0dOD9k7G0FkXklEM7eQ61ZYDy7yaMHIBbXYEYgy7"
    "Hd18rInGU+29OtArV9DUCLMF09OUHCmg4LqGzj7kUujq5k7LJbz1C+zaDb2p9lC3+uB+7CIf"
    "rmjuRb0WyVhtVI89htdeRW83bK3Y8sEl4et7773XYrf19vX7fD5y/8bGxkwqvrwaVUnFiQw/"
    "bVeohOBq5PCRg3cGOrVu//avcfJRvPwjPHQCmRKWfVDLEU3C5cV3XsDkFA4dRDqFD6dgbmYr"
    "L0cwOsz3tryEWB5GMcZ9OLQLw6fxjb/E1VEcO45UFqeHodbCqMfwOUji0Nbj9bdw4RzkAj76"
    "CN/+e4yO4ehhZLNIpuBbgt2Ej65hZQpvvIO2DszNobcHBjP8HrQ68bnPYn4RfS1whXgwCwtI"
    "J9DViddew9IczA689CI0VqQiUGjQbMLoJPYP4OoE8qfwrT/B6TO1UU2PY86N+w7zqO4/vLVd"
    "ent7Ojs68oWiQlF5KQQxvTx0GhXlLItGHYnFFQqVrVnGJ0zvKMHW7E6e4nFj3x44e/Dqf8LZ"
    "yIf9HGZew5NUfEdnBIF2RycCEXZGcpOeVkwvoyxms6rL2NHPjZ14APV6yCSQKGA2QWvH/DzS"
    "GSiVMKhwZgSmeuzpwdIaHn4EgoJdW62DXgcFfSZxmIdSx9nvgQc5GsiX9RpcJGdvAp++E3Fs"
    "UU5ZyXF9qsBH9yVs6HuOY3aB86SpgTstiRFcRlMDVAIJItQ5kcpcH5VciUIa0TiPastCeLJj"
    "B69eemZmpJXDw4FAoKOjo95aM5f9E9n2toDjXZytb2w/ffr6v4JBjlDzeso8dAiREHT1UK0z"
    "eJcLDgfGxhAOb9Hi4CB0agTjaFpfRHK70NSCxXksLW2quXcvZSgUpDDprrfc4sDVSfj9Nzer"
    "12P37lrXtVKGa4mnZHj4l9/nx43qxn0iNz5fre4Xo7IwP+9bXiZd2dfbe/f7xZYWrq0G1p+v"
    "yuXY2Jc3MYH7jiMYRr0Js/PwLkCswMIMWmWIiWCt5zrVO9+5E/EYNDqkkiiWICix4GIUJu6T"
    "TrJbXbuGri4kk2hsZCQhX96/n7+NRtmIN5ZYDGoV8mV2WNc87DbOB9k0NqQ7Vah+rnadXO8x"
    "lUYuVxt8IgZBd/2hDF+iZfiimpkkn9vPl2pGr7amN95uc87w8GmSiiQYi8XisWPHR0ZGgsG1"
    "XC4vqAVedheVo7GEWi1otvlappvz6vPfhqkZyTAIxeIphEIYcCKnhHsRsQjeP4XuA/jwHRw/"
    "hNffha0B6TxXJg3ylT/EpTFMTuOLn8PwB+jvw9Be/Pw1zm+5AoxNmB6DLA9JA2bP8wsS5Bpc"
    "+IDJnMfDKXRxCWYdCjLUqUFsc+QKnGbMLDIE0VSR4aiaTIFFLw7vx/QE6ppx8litx88+iOlr"
    "iIa5xw9+AfccjK0Qp2G0oZhCMIbdXZBqMXwWR/ZifApGHYPhS//K0391HGIlnnt2k1Gi0bh/"
    "dY0kEvv+6urRo0dUgnppaSkcDpdKpdWA3+32SaWiZntLYDUkFRXKEs29xw7cmePX7G6qQ18P"
    "4lEUMggFSElV+EAJthb2KUFg8CFwJcchzUbwki0iGWG/Q4ldiRhLIgEpAVGRp4TYDv0mJnNY"
    "RpoXfd2YX+NcRy1Q4zYH/G4suPmrUAyEsVorK5cmG4xLWKOsqOJVQBpAMMDNpvyMy1MTjMiT"
    "wzhxhF2YeoxH4POjxYx0DtEI6yC7lj1m/BR623kwoTB2dsNhw7lh2HpQSGLFjeVlzMzA5YZw"
    "y4qUxWrRatWJeNTjW3nyi0/ScFOpFHm3Xq8niB8kIb5r112hDBBai+UKJaNW2ALfq+Hc1oq5"
    "ebSv76yJh5EqMRccH99U89gxdskNzF1yofEGwCUsoqAmyK6Gl9cNazNGziOdvrk7vRpyHbSV"
    "46X5DFajFbP6thj60aPwejf12NSMU6durlZX8aSVEGxVoVTGkptTMaEfhSkFHEEsNXVjuXU/"
    "cLn8H+FIWCYVqVQlieRpkehuz3ts7AeWfPNP/9hQZ0qlEF6FswMz19BgY28aHYXJiLlFvP1T"
    "dO+l9ILxMRiMzKCb7DhwEKUc1Ap41mA2ckzEiFyGMLOARAB1DqiVnJwzWZhMPDHEW959A2UV"
    "QkE4HWhsYJVgNCAL+LwY6mMyevE8xq8w7I5f5e4OVpjrvAtdrYwzrS3Yt4+VAc2hfxVBH4oS"
    "plg0B5cuoMXJmONo5TquOfTtYMAMRzl0tBpOWhfOclL58CycTtx3Au4Fnvuz59DcyphWLbls"
    "pkBKNJeNxRORSDyTyRjrRgT1mkS6kE4NC8IT5bJAhFIikd7xTyqV4KNLmUwNZ/wBJgZqA7Q6"
    "2PWYWsSbP0NLO2etNQ9hCVcgHyE2SQKqXoF5H2dR3xwIfnZ2oVBiZnn/fXAT0XZgMY2Il9F8"
    "NYKVFcxNQahHahWxEvw+NBkweAi+Zdx/DK6zsDZAY8ToO0zqS2X89GXEkmjbA1GeOavFytzG"
    "YEAyhEQBly7yn2ZCPy80nXjm87WRk7LraANI+i3h3IcIV9ZbLp1BwwCee4LrUBQuLaLNyQxq"
    "1oM6E3xuIuksA+tufXheLs/MzAwODrzwdyOLs5fz6aRUjcc//0a6WCeT5nt6+kYvnBcrNXKJ"
    "iMLjqS9/SbZ9HVuzO3EDgl0CU0oqrgAGdqGzB2shBKPMMUoZrtDWhlSUyQ+Jpp2deOPfoVBC"
    "J8H4ZXQPoKWBrUxQLmhhVyMXRjHPc7Pqg8GCXILT48I1fnVmTAWNglkdEZi+TkaARBhmG+an"
    "EI5BokQiCO8MiiehEnMIapSYmGE5LS/Bs8yesbQAGeHGCLIP8cDsTnRQgF6CUgZnE08kRUAx"
    "CZUOk6NIP8J1nK3o7mWZpizxUF94EVYbzozij762hVFEItHDDz2oEgSPf2ZoUJLJCvFw7sCx"
    "B+pIHVRKV3fnXQJOze4OJ1b9qLNyQrP3QZDjqd/eVO/gPRXQrOd8lY4xg2ztYPgm4ujxwb6+"
    "jEb8WmHl1T4CE/J0AhDpHui1qDOwaLrXxusB7iAzomIO+RRkMk50mTiMZhw8BEtFNEQiiMU5"
    "r1KnNHnjkyAY7OlkrfTMs4zshGBREv0RTvjVgdElpHvnZ/Du2zj5Gdia+fJkHHPTSGS4Dl0y"
    "fZWJQHsrVDL81d/czii8h7tCEKVlWyGOpZBEkBVLueIdbAC+qRTyuUw6k0mna3afqqx7UCIl"
    "rk3cLkPx/hJ6d+GVV3DsXsZukRiXJ/G973FNRRkT0+xBi27Es/jwfUyO8VqH28vyxGqFXkBX"
    "Pzx+aKQoirBSRoORIXXiKnQyXJzDyePQSjgahnYwJyGAjS1BXMblHFbWkFuDP49d/ZwGNXJG"
    "ucujkIp4weCHL2L3AagUuDIGOQnjLCJJDsGuHdjdxtloLcBAn8zjzZ+grhFJao3C0cnam9LP"
    "MrkFDUmGM2/i2jynGUItktByFb76O5v83e32yGSyH7z4ei6fk0hFxXz569/8fpVi3k2Zm531"
    "ej2dztb1c/ISODr5KMrZCxjsrKBKJ7xLGOhnJt5k5nTa08VwTzUJwZuA8+exsw9KgTNn3wA7"
    "rIGiocx4olVDTC5vYxYkE5CKQaVBMgNHC5RitCaZyR3ezXRQo0dzA2t6ayObLJnl97ZQUwEP"
    "X0tA0WiEWIWOXn4fIoFofy8iYURELKejWV78KYvgXeZRCWroTBwZNgvHAQFUMMTtxwucYPtb"
    "oS/BZEW9gaeNkqrVhEQcnYO8ZFbGpheNFji9pone/P5Xf8/SXx7a6Zw9k/e45gRBKRGLyGej"
    "sWRdnZGINn3Q67UbUZLJZLW6rd9hQE2m0+lkMikIaoVi/f0zpCEp5Im3UY4iYUmCZeQiVnzs"
    "1ASskRg7nUZgdyY5s6E2q1e98w77ON0q3Yakcv+Urwgo9Dq2QiTEazgULuH1w8OURSiIiedZ"
    "KNnmORmuhljfimRM2EkHlcQV+UbgU8/Pg/JFbor6oqt4PfKGQihEgONdYcq4Y0dNCdNvynPE"
    "iAh8KNo0Kl5BsjlZAFe/pd+nPoDBtOl9Tn19NP4aj5xfWFxyLdpbHM2NlvInftExKdtMvuy8"
    "vo6xqVwcvUBGb7E3CzR70vWnbgR5b/4ckTSPprMXQS9Wwzzu5TV+HSGZLJVFi42h4Ccvw9rC"
    "N0ZB2teGN37GR4RFUnQ3wDOLc5fQtxPREFx+PPUkLl6EzwOJHA1WluZk/blJeIJotMEcxNUZ"
    "OOpx/gpyac4W+w/ySuH8Io4+iqiXs7o/yFKu2YzX34SzGasBBiUz0SoNYgF07eJVZRpkJsMq"
    "IR3Fq68z6SQtJlSWwKgFCiCHHeIcfvQK30i9jtP41DzMDSgXUZTivqNb2EgqlTgczkw2t7wS"
    "rL456ZMUmiFrQyPFypbfqlRqYmskgAm+RBt5NZtEvQMz72Lffqyu4MgueCifTMNqZl7fYGZm"
    "Wcgx0M8vIKVAfzPGrqDVAms7Zt/Cvr2clqkmEXmSPHTbF8/hkUdZkRIuE03MZ2HR471RzEyy"
    "4iV8oEki0U+M5eA+/NuP0d/FsvOjywzW5GHs41qspSudhjmFhgP8fwpHd4jl1USFmyqlzGGI"
    "pFM1sQSXR3ixiKa5u6WyJkFqWcRfkSbwe3k1Zk3GlJRYg1jB63r1tq0t2NTwsTs7E8kkocRt"
    "tsC7FufNZnOpePNO5soxZaYN4UhErxKun/dYXGRNQZLS0oTRCzVJSXpVRgLHyrfnWmQM2dCr"
    "ZKDD/93Y1f0mDUXxAy39XDcH2CKMKSjJNjVLlphpWJYYH/wnfVtcon+BT84Hk4VoVBTrxzRm"
    "MGADxodjxZZSPL85eXAzel9vb3tz7rm/j3MLzf8eVYYse1FA6k3aygoqCqyUGDRqDfJ+/FmP"
    "/GUp6y1KJ3Hz1Nyp7WSYYuEwNw9A3y3jvIIpOnLmBJ+fyxJTi8K+8XDWl8XiaVcuh285XEyS"
    "JKBLUcm2z4lRPA7knLROu/m9267VqrFo1POcYtHWNcUdBtOavLNbTZjRS8m05w4sy+w2G/XD"
    "jhgS2MSJktQ7bPiBoOr69aVcpVK9ms0WCgVJEMMyS3yB94uuSu1Oj51TLG6yosmkUqfrxoDw"
    "yYbnZin55gnAmpPl1TsyDOh3+xmtrmFH8+yZxzgbFnO0+Qh21DlhS5aMmw/p7n3KLlDtC1WZ"
    "xy7T820gfrtNyzfJLlHuGkrtgYsosCJkJT5wSYjAeT7eoPV7FNFpPkVHA6qUQRu8wGGRTBOy"
    "XdKR2l8/0K07VN4DUS/coNg0bb+EQ15bxTnBxgMUR8VZ8h3KjXGM9fk9NK43hKz8nx9CDz13"
    "a+upZVkcdx+vqEQ6vX48Nhs1rSVZDthuKwrHna9UNO2CP+p1++xeZVm6ksn0jx1J0SZ8kZ5L"
    "ecOh74/GFJ7SFYYsVZGFiKhpas9Dboa+lV4rurF/gOOeZAYsx2nGicYCayySKsCC73ykxDz5"
    "LlKPd+7MFCUSqJKzdxVEKIT2EaoI+XUkeCjAQBikEdWalLKQVvUDLBKjMDMw48bIhztVNfQy"
    "kdhv6XYejCeeiBPWskGA+gzHzuDLFFTZqntYb94HrSb2lhQGxygMFyVaXiTJAIEzr0QMyBV+"
    "NLM0z8R16Nij6Awm/M9W36+3mg1dBwR3u+f/Q5c/GiUss8VS6S9t2jD6jnMWZyZNEMR0wvoJ"
    "Gi7gKHOKpsQAAAAASUVORK5CYII="
)


"""Help image on default search page for contacts import page."""
HelpContacts = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAH0AAABHCAIAAAClA9+5AAAAKnRFWHRDcmVhdGlvbiBUaW1l"
    "AFIgMjMgYXVnIDIwMTMgMTY6MDk6MzggKzAyMDBkFAP0AAAAB3RJTUUH3QkDEhEXsnm5BwAA"
    "AAlwSFlzAAAOdAAADnQBaySz1gAAAARnQU1BAACxjwv8YQUAABjJSURBVHja7Fv5lx3Fde6t"
    "eu+3zz4jISGhDSTMamGMfcDEISfh5Nf8mB/y7yXgOA5xnHMgYRNCAkkgjSTQNvtb+/Xe1Vu+"
    "em80PGZG8kgaJGFzz5w5/aqrq6u+unXvd6tu8199+Sn3w0gB4TiB5zeV5lsLb0uaZpIkbipM"
    "klQiUpGjMU4Qtn8QkueokouCyN+xCqtzlxYepki70kqvZwdhnOWFIhNJ5P0wlgQhL7I0F0s6"
    "4USSJXFWcJKEixCFPMAXRV0hnh+JIg9kCZFUVQ2CEHOl6WqapKLA0yRRNc3u2YZpaArp9NxS"
    "yQyDENgVvCiLfAYpOLzS90Miy7IkFByfpiku/cDH7OqGhYnOEhrFlOd5w7QatdKjxpwJvyv6"
    "HkVxnueCMNA2ptE8tCqMYlkmoigCRNwZ6jjWQJbhPi+JAiCOKVVVJUcJU8ZcHAiDlS2VQf+G"
    "T0FR+Q1FLXh+vQJbBAJurP9k5fg/rFQU/G3NH14P//P846Hv0MEddqXT7ZYsc9tbULEwpkAf"
    "2FTKJbvvATqADxAoDXlAygmSwOc5DAmrj5Uh8jkvEkxLx+sEMdNuTdOgsERWDF3d1CVeHDU+"
    "/MZ8cMLtotv1Nx4bbWF4/Zggvo47zwt2e+3ytVuUJhMT40zV8jxM8jT0zEql13UsXVhuei++"
    "+Jxt9++Eu2ka+Nv4aRj6vfThsVj4D1mYfZcUZaxRT5JE11VFEhwvrJl6okmcKNcq1thEXVZt"
    "QoS7tAITsWkJbyz80RJmLSADizO69gc2StgogWz8hFGCjeJHGhneHf05fNF37xoUDZtlLWOt"
    "MOOWiaI0dKobJmjjjbiLhT8cCBof7c8PhDt/+tMPWq32Tqrqutao17a91bU9GvmaZri+D3Sx"
    "2mGmc+bxJMwXzAgMPZenXhBjeGVL6zuBRGD3E4lZGlzycAaEyPCNgefKCvwqpWlBJLhoOlYv"
    "B0GQFaKly64XoLqhya7rSwNH6no+ZhJ9i5MEtg1ggRHBzwRwOVlWKRtuQOFL8AqaYBJzzIll"
    "6jSGn4ffzwaMi7NK5SSGpcwojUvlcp7EAc2LlCqqAtczOVGXRHEnEN0D7rviV8MwhkeFQ8TI"
    "hroDKgJAM0YtJGZ1eQ6DTNNElOShtkOh4BVAEOOIWpbuuIGiyBxzyTxT58HiYC5UEOF14Skw"
    "cjgHaCLID2YR9+I4BtwD7WYtqooSRWBHIiM5OafIkh+EiozXFUlagGXlbFZYOTrs+wEciSgw"
    "T6Nr6tYRoQnM3u5iPSq7wyMxpNDzFEULwhCjxvIES4kiCqBkyqc5B3oJz4Gfnu9oqgqijQEL"
    "eC4W8zSRFQk+GSqvyqJIlISC5gBBOU2prmk5pjIvCCFxTGE0FCICUFVTYX58D5Q11TRGX2OO"
    "qgpZbfUVIkRxUq1WsNqAL1ZTGMSqpiRpDhIJFSFsMnjX9YCsZZqYTFHYDPEPCjq3W/r+k9yr"
    "CHEUun6AK/AZrEP8X+d6I9LtdIMwCqA8QdB3PFaTcedkaaUJhXM9b+ghwjAa3ur3+71evxiU"
    "DFugMNcZa9bp23Gypf1uLx+8uhh247YEnue4XrfbbbY6jxqoXRbJtv2b17+xw1zKA0kvu53O"
    "3gP7DFm6cvW6rBm6JgchbVSsW0srzzxzFJHR6tKC7fpRlE5Ojtvdlm6YiI3ihPvs0zNEN0ge"
    "OTE/N12Dj4J38uLUUBRc7n9iut1x4EhNXbi5ZPNpXIiSqStrnd5Thw6XNHLh3HlJ0bzA0xVV"
    "khRDE/oenZ6aKhD2Utq2Pfjfaq3yqOHaNeHPnf0Y7ghBn7gzl701DhwSt9FCbmdBSjGw8neh"
    "CrDsCL4ExiazHXbvRyEEnmdlpSmIIFr8kMDeRXp2X1M1miYl04Cro2kKCnHf7+50eyAbgNU0"
    "tDtVqNeqjxqiXRZot+s49XpNYnokSBfOfZ7xckrTt/7u9Ts5crDAIgc19L84+21Ew+dffL5e"
    "K993D0A005S7deMa3IaikQOHjk6P1zZVeJD2H0+Bo7JtG7jz7/3HO7DRrmODrA33YLfdjOUG"
    "sRzWBP7D1w03DBDiIorhmVUZGg2w7J3Srz8bDYJfT0zNPWqgdll6nabT74+PN5i+G5p65vQ8"
    "zeQ8DY4/c+DqN8tA88jhJ2emxjc95vXtDz4+Hfi0US9VqqWvL8zP7dsbB76sGk5vbW7v4b7d"
    "QnTy0smThvKgkYGmG/sPPv2ogdplOXvmM1XVSpbG//d7//7yyV+EgX8frXieP7odtrtCiFyt"
    "jz94O4+VtJrLCMSdfncQNDv9M+cvjzeqgmL6vbUgTGbn9hx7+vBWK+A7zoefflarVlaXV2u1"
    "0hcX5l/42dPdrlcqazdvrPzy1ydXFxf7fvLGG6/9sNHej154pu+/euOtJI7iOJZVjcszREew"
    "49VabSvuaRL33UDkC04gPPh5WiAozwueSIIXxIYqh5Rqqm5axmO01f04yW197zF99+zu5euL"
    "MpcePfGcKouGad7psSxN5y/NW6Yxs3df6LpfX7ioWqZKFJHgpqAZFSGnrufvP3ig8RdHAXdX"
    "GO4ikWcm6rwopTTuulHtDpu93GAbcXbPnMQVodeP03xscmJmdiYKQqLIvu/ruoUQVbfK2+7w"
    "/SSjwuww7EbfC7gib7ebtt37/PRZL6Lb1iZE8F234IVKrUGjyCpZnebq/OWrQRhGYRRTv9u1"
    "DVNfXVm5dGm+eNRjewyF7W4XOTskwI84yUSu8D0fljn2HdD5O1nnMIxpmvmeF0URl9MoEVWJ"
    "4CmBNcjlaRrSFBMT+oFAlGLktPMnGcrNmzeaa2tjYw2Gu872poupqZpMAGA1CCOWaJHnWx/T"
    "DK1RqximtbJwU9FLjUpJV+TaxFS1sh5Y7n9ypPa2Tfx1i9N3IMCd/+rcKSKRR92fvy5hR3K7"
    "CHqn24PiC8L23B1WrbnWrNd/lDzHhxGghWH8mSARbFBRSJ5ld6/WbLXY4WenuXL+4lUiy6ah"
    "4wWkSDNJm6hZC4uLOS/DCuU0NqrVnMKVhmnOoWZGI8MsmRXryoVLk3NTYP+GUbYsrVIpJwnt"
    "tJrzV65TSvfumaQpmb9y9e1/eLPIsjCKHjWA38nVK99CUVRN9dzg6JEnLl++JchykcZFwSE8"
    "mZ6od207pJkm8VatPtGoHzl6fGxs7O5trq0s9HudC+fPg2cEcaoRcWyqvrTsVEyp77iqqsEz"
    "nvzFC0EQ8pe+OtNaW11ttkulkqrIHC+qiuQHkcQO4FPMSpFnvCCIEknjCIFVmrBTU54vNM0k"
    "RHRdj+dZCqOqKJizffv2RnC+cej5kcBSNkRJ5GiSV6tlWPtez56dmXrUgK+L47iiKPR6fd2A"
    "k5IoTaHTCpHYhp0oKTLJ0iQfJGQqioLJ2LPvoGVZd2/T7rV73TY8J0sgGeRtgd1RmmUpy2no"
    "2q5h6FDNZrPFf/7pB47r7s5QCq5Usu7SGl4pk79wX2LbfZokd69jWaakqgr+HvBlYD8RTbRB"
    "O2gNKhPHVIKILJlguDk83F7GysiLXJHlYW7Q+p5zkacje8gbyUNJmhFJYOmUCDKgeFuyG1EH"
    "1kxWFG6QqrQp13fYDpY8ujR8cFhCY0pksqWpbLiPjThkI+4b1r8nKCqVHZ0Z7E4eB+BDxCRJ"
    "QhDQSsViB4eilESRZuquEwBZiYgFwyjVdU2WxXbbBpYqERH0pkw7CpGolkbcMJZYukcRJ/nU"
    "WKVrO4AVy1xWZLgQICCIBBdWyaBxgsnCbHGDRKWYJpgWTVfRHCrFYSQQoiuk4NGNwBaEQdaN"
    "DKOJxtpdR9dlSZLzDIYURhJuMDd0I4yjkmU6jtd3vCSOeFEWuFxA5yUyVt/lo13+w/f/iDgI"
    "b85orBqGYzuw75yoqISznbACj8Bxy63exHg9DMJS2fr2yrVKvb5v/15xRF2yJPUw1EFqLlaZ"
    "IIpEkuIoIrICFcwHJ66EDCxmPsjhFfgszTRNHeQMSznLRypkmaRZBviKAh440QYZMsB9cMIq"
    "DCM9XMP532uK6c6PfO+v/n0IW5pJCqwYW3G9MEuSSr3mO06AKBYOtOChOKJQLNy62W7ZLFFd"
    "3oYn8SxTLIVtyZn2STD08LHQr+EA4piRhIQd7GUgC2iWJWvwLAk7HyTLAfWUJXnlURQXg68D"
    "MPY0TaM4FgfZjR4C4EGGMctHy/N7xeNe068fQro2/8n//QnshRvwa24k5fNOAoB2nix498qb"
    "7hbDgOIO9e9+d1gh/37aAQYEq71tIsI9jWJ3Bb1q1CuS54fjjfLyWtfQ1aIQg9CDl0Of6o0x"
    "mWzTs5W1NiyMH8a6Slj+HM+x1FmuaDe7szPT1e+fRKMysxkxbxiK6/r1sTrbfWu22XcJgqAZ"
    "Bkzn8srKRKMCFddNi0ax63lElsYbdeX7yQps6y2I/CjUVA0WjNK4Vq+XS98RO8f18iTz4wjw"
    "I8jBcCRZ4TJxtd1URF7RTSzlah2EtjTsGOKPJHJRU1FlDCOLY7NStu4tg/x+BP4DBJ1/7w/v"
    "gm3bHnCUC46srdz0g4RI3P4DTwHZrY/1bAfTce3Wah77oOcTM3PtlVuKUQZtVxTtqYNPbKrc"
    "bq5iFXmBz/FkrFF2HTtNhTyjRNGnpieKnIvCwLZ74JflekORxGazLfDp5OxeQ5VHm4IRiyPa"
    "6rQTZvGJCqcs8LOzM99NTBCCFV299o1plRC00IQeOnwwibPFpcVqxQKN6nVaVqV+8Mm9w46l"
    "abG4cF1V9SD0EZAEnnf42NHJsTvuge+WoJ/T01MsJfxv3no7ohyiB5Q2m2uww7phgd3xLMHa"
    "rZRMGNZSubRh8AAA2IuqazzLVxIQdU1OTuCqXN7mC4KFxdVGvaxp2o46VRRLy8sIx0GHwiAw"
    "TR22RdNNMshvQADiBx4CNITE7DMqXlQ0ddQMF1ly9dri5EQNDhzUoFpmq2Fx4RZciGyUDIX4"
    "YWBqOrwT+yil20VAKEsirBNijpnZuYf2xdnCjSsS+IMkcK4fVcpm6DlfX7xStpS1pl2rVlVd"
    "ztJ8YampcsHBEy/XS+vYrSwtfnHu0tRkrdnzFI6qZrnTD+TMO/jsyXpp84lHq9XudLovvnBi"
    "Jx1Kk9Sx+5euXs9DH6xcN7Vex3nplVfGG4zGJTS7fOUbVRYvz1+d2zMd0Oy3v3l99NPAgk9u"
    "XLvVbq26rqto1ptvvIbCTqspykZZIl2vHyZZEgaUlxr1ehSFX3x5MS/SsQZ0vLCqk2PVH9zI"
    "DAVdZuerr772elbwxiCXHDyE50HpVJDi1ZXlsYlJcTtPGwWhwL4TkLrtdqlal8SNJL08jpPR"
    "QMzuu+WytSuaBLqeZonAFYIkU7yG7RGVB0FDyBLh71HsHpasDjolstBM1NSHF0hf/+brwblH"
    "6Pf9FLiHvvNff3xfVcVuz3vy4GGFiz4/czZLijhyXMofPXLs5EtMbb+Zv/ThqS/3zo0tYVkY"
    "SsbzeVrQsO8mwuFDh+u18qGD323Dn/78S4WQ1177+YN3NwyiT06dkvhkYbk7PTNpkKzVT3qt"
    "NdMyRKK8fPKV6Yn6zlvDOl5dWbh542a1ok8/8fTJFx9qrg7T99fffIvtBbEwnUUnLAkUWoQI"
    "KF5P0Q/DQFW1DUpLoxBhnEIkRuQFBKLZgIGnUJ+teo3WEEbtUN83AhaQfbJlJ2f0ayaw+0E4"
    "JcKfk4FsVNv0LAKLQUCw/sWTyNgX+4wRLYBTIWZEIJaxlIiHRyvX9b3faa12/SOHnoR9/9ff"
    "vWcZpNePnjp81O8uBTSf3nPghWePjD524/rNC1/PE1mE+5JVo7e2NH3w2OuvvrztO979/Z/q"
    "1fKvdqbvcZR8+NEnrutl1JG06gvPnxjdv3Ts4H8//kgR6OKaOznZkHLfo3ISugiBVd2Eviui"
    "fObsR7cWlg8fe/bVnz83fOoP777T6riTs7MmAj61rGfeuW9b//LP/3T69Plur91eW1VVfvqJ"
    "E788efyh4c4N9f03v/370aKdfMe27ed6wzTJB4/0dtiBYZ1OtwsKsKknO+9Dp9Op1Wrcw/26"
    "dV3fe50mLPXTR56C7vzuvffLJokTYXp2TsrDds/NsqJkyrXxmQP79w4fW15cOHXqi8ZEDdHT"
    "s8eP37hyKRVIt9mpNErdXvCPb//t6Dt+/5//MzM1+bNnj+2kQ1GUvPPOv8GGVUul6cnajcXV"
    "8UaNpjT26dETx1WiXbx8MQltPxZMQy0SN8oUwvY5YvDOE//fzrU0x1Ek4X5Wv5/T87As2YBt"
    "2SCWcAC7GxDBgT1u8MP2V+xxj3vYyx6JAB9YH0xsGGPAtmyNZyzNu7un313dTdZIlsbClhsw"
    "TASeLxQaaTqnuyYrKx9Vmfn+B4jlv/nm64P9wdvvXt95+8jGfHXji4PBCLPCzvYlQZbno95o"
    "Tv/t049v3/7e88b+dIZESrXf/PjPtUb4qnAk74cdAjAusjTOwGnIMog+FiV0OSjoDDQmxx3v"
    "+ofBPIhSTRGjJKtwVlSUIMoFzkQBZbiyrWdC1ihOcZ7qeq3i4JIUd5ACM7AzikLi0ijNDU31"
    "3JlpNwWykTlTFcn1fFESwcAUFdPv7VlWQ1FVQUAQO2dpRLojcNzxUTt4ZVajCdEAz9FRkksC"
    "ykl9pen7XkUx+0/6zWZDVQ3x2TDtN8WRvMfBPKtog1cZutofjCAcrWiOotnQn7EIRET9af5/"
    "OJ9PpyPDMFmaaXfaZyxR351AWIyAT+jljhrwPYySKAzATCdJYppmxyCe+2HYBa4jGMnpZJxi"
    "spsUFmlZgTTo4InDbDnNFkkE8udhEBr2SdjZObcBvwVEpkFZyoRLYR1lKchKFAUZZjY6P8MX"
    "+vVY1GsjgVkkGYHZbzrw+CrPsKTIuian5Ijg9KkIy3K6oSHUALeH7IE/rw7xGKA9ISZg6mlP"
    "uA2YCMMwiiKnGQ6W4PJVnmMgmBJUJSXFseB2KxDf+56rqirEErA6QUerqoIEBN7XS59F3BgR"
    "uW6hgActqS+lf/V8B6vOSGSdgifZ3dsD3SLKmhwnT7oPjYaj6ebmRmf5M6AJuns93dKzJHUc"
    "6+bNW1tb7QtvbYvP20e7891uy7EAdUYDAf2db79zGhZDM5Is3P7/7Z333rPNI/0WJwUEO2kS"
    "JDkDUSsF8k5eq9FoIsni5oWLqiyB+Nf85v3efooznMBqZ2nevHblwu/JdyKngiQz1GKjHOxa"
    "Qama5s5mDIu2r26zYLC404ovJiagBCYBNTDo/NaWJIo4f37ygqbCDKY1RwMO9eVLb7C8VOYZ"
    "2HOJNOY4uQrSD9ZCkiDIzGFYkizBOnvU7UNoXVLsYTOV+tAMXRaF4WQC6kvXfqss/hd+08U+"
    "waeuH3XaTYjye48fJykYVYOuisN2LkTVcCzJJMgyCK5areZg4KoyGycZx9Gj4RjmiSfHeNVo"
    "cKBYTUXgCnIOT4mSpGva/d2uYxs1Tx1fExzZVboqpUVGDvgCDx72bFPa3X1kmA5TxnqjkyaR"
    "xFeP7gwbthHnTLPVjMJoMhyGGZPMR6xoTgZ9RtSKvJRVNOj3wRWhkeCNh9c//FDXqLnvQ0y4"
    "5vspEHl/OBL+9e+v5hHJPjB1sSB74uA6VBsdczB0D4/zD1NMD7sgwZ+yIuA0J6nYjj6b+nix"
    "xOklmqeo2o46HIcVtfzuKRt7colhmJat5DRbhFFaEnd+6WMV2FXwHfOKrnAuEOOJXC/ARaUp"
    "wjxMqafPOPr35LY/Nenkkm3IUZyxiMd5yTNFEBcvGtWLR16T7BkacDH++Y/PuMXb9Hia7PZd"
    "SRH/fmUzwbRjiru7T6YhtX35IkOXkqrJDP72h/6Xt7pAf/lS56/XLuwPwpYtToLcsS2GKsGi"
    "STS+e6/35a3Hy4/Z7DTaDv/5/x7UkQKE2Os7b2LiRRq8xP7nv7ei7ERrKwr/yQeXS4Y7Z4sH"
    "Q9/PmatvGe2OPRz5dFXduPlDgvFH7283bfXe/d6Nr7tnP2vnCrd1vrl9qT0aRd1u7+6DXp0R"
    "/nowNGn2QeT92p/+Mhx7y9NTnSalT79Xh+Z5ZMdL5wwa6unSqXOr415jv+xWMGwIBV75qM6m"
    "0YWYyPvmRgt+qDV+L4BdXdfdrQZrvq8Ga76vBmu+rwZrvq8Ga76vBmu+rwZrvq8Ga76vBmu+"
    "rwacosi+N131MF4vkI5hpIlPf+8V3tT1fEV+vfpxCKL00lrhZUwmU9L/PQ7cu/f38iTUrGbk"
    "e5bTmM/cME6shh3NvaJk2+caVcmwNI7TCvFMHMdZhhUFeX66dd4Zj11clRypt2NUVcM4V+Tz"
    "9QfxB0BZlPfvfX8w9HRNdBznYDCKo7jdssazSFXEKJhbthUlOWKpOMU771x1PY/sR+ZFqWtq"
    "IYmmpWsSEhUZp5luGSzLSogc2qmKTLKXydZxziHRJCVO6dhN7IYJ08YjwVLFIAiQILKLlJtV"
    "82EFAJHfOCdkWaqqshIopqELiHNYxHNc02lkaYQEGfE0H8SH9KRueFiv/3tNgPJ64+Ifrf/g"
    "2XiyP8AY16fXde1HCcAOpa3BkcsAAAAASUVORK5CYII="
)


"""Help image on default search page for information page."""
HelpInfo = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAH0AAABHCAIAAAClA9+5AAAAKnRFWHRDcmVhdGlvbiBUaW1l"
    "AFIgMjMgYXVnIDIwMTMgMTY6MDA6MjIgKzAyMDDw23fuAAAAB3RJTUUH3QsRDRgZE6kHcAAA"
    "AAlwSFlzAAAOdAAADnQBaySz1gAAAARnQU1BAACxjwv8YQUAAA5USURBVHja7VzpkxRHdq8j"
    "6+o6+5w+mJNBoJEQsneRkIUctkLSF2+sdzc2Yh229V/pn9jwh/3gsP3BS1irkAOQJYRkjkWC"
    "GWCYnum77juz/Hp6hGBA60EwU8zSv4CiujKpfvXLV+/IfNn0xu2b1BQHDgR/m/NH8xbjxUL7"
    "zi00Ofvy049/+/vzZ06t3F775v0PfvbNH6/KSiHCjGeOWJYOvNFfnn3v3373O4Zm5o8tJZbl"
    "YVI2yoGz0XXomqF+8PNf1Qz54B/g43PnNjubvtlvHn/9+uUrCwvVKMxoNnvp6Py5j7/gUTy3"
    "eNS0fJpODFX/5url0uxRkaZ/80//zLE5U0+DnQF9t0dD0wtc2+J5vqjr3f5Q4Ng0xSxCYRjp"
    "uiwW1GG/z/FClmHEMGESJ3GqKpLjhHAs15uyyB+89P1ux7QdhqaiBFMYpGWzjPCCZOhqp9NT"
    "NDUKQpYT4tjDKYWYjFCIRzQMBsvkSTro+w7veUrx4gF4z3XcX2BMec8HO341CAKMse/7n3/+"
    "+c2bN3Vdf/vtt+v1epZliqLEcdLe6uIEDL0WBgFN0wwnNOu1yf8ddDbXNraojClIqCApNENb"
    "owGNaEmuLy/W91V607TCMMxISjEsTlKKpuIECxzKKKpcKUuiOOkWBe6ly19LSvH1ky/nTfjD"
    "vAPpSZJ89NFHMzMz4Frv3LljmmatVnvnnXeAd5ZlKyXddX2B56MoFuHJePH+LURJhv8F46FI"
    "4si0+EKByuDBObmA9l36bf8oyUoaR5Kouq4FXIOugPMci/AdBElZWlzKaJpkFDjh5wE71KRp"
    "Ov4wjl7C+fl5INpxnAsXLsBgfPjhhwix6jagj6bru24haxr8mZw3Wq3tfxcORnpFVZVtqSYw"
    "DP2Hes7U9/fNe1Ls2Pc4juG4srICRub8+fNra2twcu3atcXFxbwl/PPEjr6DggP1o9EIzLpt"
    "23AE6iVJAoNz9uzZvIX8M8QO7xzHgXaDhdna2nr33XeHwyF8nJ2drVQq0JrGyaXLl2szVc/x"
    "SuWS43qB73McOjI7p6mKa42u3fg2xZCyZH7g+RH++7/74GCk73b6tm0yDB5ZcVETMxY16s2C"
    "JOzqloTeV19dSTP2J2/8lHs+IriHXN9gMDh9+vTMNlZXV8HcNxoNuA554MorK3RGDL04TmgN"
    "A4PXyjC/7V1FqTC/sIBYFqeYZATc7IFJbxRVSZZEgS9v6wHGhOcf48wZxB2Zn6cZ9jkhnbrP"
    "exRF7XYbopqLFy+2Wq3NzU2wMDAM77//PrTSDKPIP8gm4oWZWi0X6XkeIqyxduu6/ie6sYiv"
    "P59+VRRFMOtg5avV6o0bN1577bV+v3/mzJlbt25lD0ZkUzwj7Oj7uXPnQOXBl0JOBDE7kA6a"
    "3mw2oQmuEEKSNGVoGl5kURQguEQc92AcnBHc6XQJzUj8+GWHKBle/jgOaFYwDG3/ImZI9DKK"
    "liUBEwgNmMmVAiQQj8C1bZyBIORPvxkHzft7770Hkfv6+jqYb8MwwK8C1+BsJ341itLr16/E"
    "cQpDUCpXO5udYlGN04xjqaVjJySe/fKLS2Db4yiOAsdLGCpJRYQjJIaW+8t/+BWi94t5243a"
    "d2/Vm/VuZxAmKY8YHIeSohUUbWlh9n43SGg//eQPXpgdPbbw+qnX8uZ8jCefj8wyUGg234nU"
    "Q47v1z2eADTNss9Hrn2YMVXbfLAnfU9T7AcBQ4EPo8IwNgwdQvv7reBUe71BlGDXHIiyqhsa"
    "jxAE+Joq97pwPfbsES8qelHPcJokaalUsi27VKmwTz1HZTteHIEjlceuHnHgNhkGSZK4qxtO"
    "olu3bsu6gUM/YxhVVXuDoSYXHMvmZbVUVJMo8oJI11R4Fi+IwW9ZlgPxsSqLDMsTkiKGjlNK"
    "5FnLdgqqUi4WD4J3iOtN00qiwA9ChmGlgvQg7/ZwcPP2XZLErmUJAuOnQqtcMP10ea62tmXS"
    "aeS7Dly/ch2dXFka9tqdzX6vv3X6rbOGKj2l9EkUw72ybAAO33IDWRTLtdqjvBOCR8NREKck"
    "CTjEBVHCI/r22h2CYzkh7qjT7jqtZrW3tUkwLjaOeLYFvMcpw7NMoSC4QURnSUogU8SYYucX"
    "556e9+k6Xw6YrvPlhinv+WBvfjVJeoMBh1AYxQSnqiIrmsGhnRqU0LW//PraiVdOFvWDLqFx"
    "/dgZdWNCyyLnuB7HIYoepxZghlut5v10Dfzql19cLhgVRCWcVBAQC1FAEoearkd+wPKsa0eQ"
    "aFuuj3i+wIt32xutRmuzfa9aKcYEAgQDXNq3175SSo1GvcIjdvXb1ZQQSeQLirq11VtcnPU8"
    "LwwjcO+Q3FQrVdd15mYbt2/fi8Jo5dXjMWEqhvbEvDMAmoabchzPSSJkTruaZ+p1KY/6GZIm"
    "8O0Cy3K8KMvjyAqePYjDcvmheTrwq7KmixIvcBKoC8RaCof4ohGGAYwNhECqhkLfLRUNGDy9"
    "WH5ZlpwoabZaiCECDTyjQkH6yRtvmaMRQmwSx6VKlcpSuA/48Eq16DquomkiL8Q4zQiEfIGm"
    "6aABMzPlKExcL242ds8bTv1qDpj61dww5T0f7Mm+Qzq3eneDJnG5Wi+XjF2tnjn45MIXHMs1"
    "mjNrN1er9Wq51lhenDsA6e+ub4ahG/phqWTYjgPZHGTEhEYiYmcXl3i0o1VJ6H3yh/+23aRc"
    "U0lM1KLqDO2UpiSWCqJs8fjR2PPam1vgB2RNS0l29q/e3G/J92rfHcehx0t6D2WqE2RZNp4H"
    "zzJCiOf5iiyBkxP4fS+eAWD4SpxCzMAJAjCO4XNGAj9UlEKhoDDfzUPgNO50B7KsyLIUhqEg"
    "jKuAMpLRDI0TGCcap6kkSUEQWpZZm5lRFWVfxZ7WpeaDqV/NDXviHd7eOI5TeJEJgSh+Vyu8"
    "5kmaTirOLMuCE9fz0u+uTPFY7M0Kk/Tq1WskjXDG0JCncOjYiRVJ4CaNo37v6+t/hOPy8jKk"
    "bY6fRKEpFKp0bC4un+gO+n999uy+rfQdVjwz+w7aPV42pigOfV9z+agTnoJ6tvYdKGYR4h4g"
    "+gBId10waIlle3Due964kuFxwEnc7Q8Hg2EYRpMr99bvDgYjE8yibcNHczhwvQBaXcfa3OpN"
    "+kzsZHdrs9cf3v+6B5ugs+dH3/fs9p6Arv2mZl8RhEl7406S0N9GPk5xqVJdXlp81KbZJhBr"
    "mubIHPQdz5PlIssmiBULigAs99udxuICl/oRLQgI04x84bwTpZks8u179149teI4kWf2UpxV"
    "ajXXC1mWYmh6OBpolSPEH6YYi5JypDV7/fqVn/3y16rE7UXyPdkZgrG1XdWUJCkEvxzHw/F+"
    "a5rEtm17QWQYhqrksKvv0GGv9QQQz3Q7HchNwiDCGC8dXRSo73l3LOfW2mp3q7f8ysnjU973"
    "hmnelAN+VP3M84ROpxvFcblo+GGYZZmmaaIgPNoNp/FnFz979dRfbLU3UkzM0fDI7OxwNOL5"
    "gsiOfRzBPM+Rfr8vynqpqJojRxAFsSDW963e9nDnq6pezHDiB6FjjSsAfqiClmGYarVqDoeu"
    "4ymaLssKojOgmCa+5Uc4JWHsR1EE0RBi6I31eziN2u3NZD/zvj3ZmSiKb66uFgQOtCF0bUhb"
    "IXWaaTQnS32gaJc+u5hmNKaoer0h8cg2LdPx3nzzp7kNyPONvdoZiGQMTZMkCd6+JGB5UeQf"
    "js3LtRoEZXEYSRKfEkrVFKNayfvpnmvsiXdIiVqt5uRcfqTKmabphcWlvB/kkOFw+1XLjaz+"
    "RphSakGEbNMPgiSOVN0QRbHVbNzvhpPo/KcX5FJNZDGSZJFjwoQkka9rxmgwSHDi2gESEcOy"
    "b54+fTCSH27eWQoXFE1lEeRxQhhqmsoLfByniHsoaSSEtBbmRUliGHpcT0AomaEEYSYMfEVV"
    "JUWBkVGM8kGWOe/Jr4IaDYdd0xovORUkzgtilqYV3TjSHG8asvrd//n6xt/+zdmnrzN9QbBX"
    "vyoIrCzL5XLFMoeiVBCCUJElhHZSVq1UPnXyRN7Pcsiw17olwxgvZ4v1sdHctUWIZsa70fJ+"
    "kDEwxhB6TY67muIwyBhusurrjafGduYzMoJJRv+/21fGs9z0+EmzNEkIJW5PTz3NRPfhtu93"
    "11bbnREnonpJpzih1+kkccIgNiPj3XDV2szc7BHoFjjD3/7Lv/KCgAReU6XIxwyfxV4U+C4v"
    "CSynBmZPNMqx42QMUY0KWE6jVHYij3ihKMksm3VNRxVFVVcGI2e2Vd2811FUyc/4f/z1z3+c"
    "5Hudn5nsk/shhfIcGyKER0s8pngs9mrfkyi6t7ERhkEaj7fDI4aqzLRmWzs7cbOMXP7q8tbG"
    "8Be/+UXeP5d2aDCdj8wBh34+0g8i37VYTtB1LY5iUYQgPoLjrm5pHG71hgzDCogaWX6lrHf7"
    "5vLS3FZ30GzMZCRdv7NBIbZUKrmWiQSxMv7tC5ci6faO3dCLs6IijWxv9kjzWUl+uHm3LWcw"
    "GnI0uB932O9HCeFYOhtP/OJ6a67V2ImyXNO+ce0qyVgah6yo++5g9U73fy99xgoSpCaVssJk"
    "yHZcGCBaVMoKd3ujc2RuwelsCaqiaMXQ7vWsqD5TeYa8T+1MDpjWi+WGKe/54HDb97Wb1yDf"
    "ZMbZZjb+SY7HTRCBbzx+4kB/vvDu2jcY76xVTfw8xmSSEqcEHXvpJeqw8+67o//49/8qqBKH"
    "aEmpeGafULSsKI5jjkZ+qai+8dYZhMSn/6Ink8pzgPff/+fHPCI4EyHmEhXt3XfegCZC7whz"
    "iHmHtHl9o/vqqVfuX6mUdn7TUDe0o0fHUyj9bqdUafy4+/84EELWN9o4TY+//NKD19fvteFY"
    "a+xsx5jGMzlgGs/khinv+WDKez6Y8p4PxvEMmPm8xXjh8H84tYAynbQ/qgAAAABJRU5ErkJg"
    "gg=="
)


"""Help image on default search page for SQL window page."""
HelpSQL = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAH0AAABHCAIAAAClA9+5AAAAKnRFWHRDcmVhdGlvbiBUaW1l"
    "AFIgMjMgYXVnIDIwMTMgMTY6MTM6MDEgKzAyMDCprw7NAAAAB3RJTUUH3QkDEhkaBBFPsgAA"
    "AAlwSFlzAAAOdAAADnQBaySz1gAAAARnQU1BAACxjwv8YQUAABuzSURBVHja7Fzrb1zXcZ97"
    "77nvx765fEoiRUkU9aolO4rsyHaaBCmMpI5jNEGDfuinAv3Q/6H/QYF874d+aAonaCMYLgo0"
    "dl3DkWHLesuibIkSaZEUH8vdvffufT87dymtKL4lMSJca0Atzp6dM2fOnDkz8zt7V9SNLy6k"
    "SQzP6dkSuTvx9Ve3b3OsdGh0uFDI7bQ+3xYifQO9la7iXK2hacpOK/MtIhKnVM/Avq4eCGIQ"
    "eUhSoCkIAuA4MAzIPTgAtt1CZlkWV4xvmWCaEEYgSoDhqlhw/CBZvYVRGFA0YRja8zxBEJY6"
    "LcuSJUU3sAEUBfkCKHQIUQJeBAUxe1W4lfpip0AgjIGhM0WX3i69Qgp+Ajzz4O1aA5GSBJo+"
    "SAREdqeMTtM0IYS9fqWXkYBNM9MzDJw4Cc1F6O6BmWmYnwPLg/5+YOHq2PUrha4ejmXi0K7N"
    "Lmrlcs/AkMTstVoQp0ARoAFy2tzn5y5nUoBK4tC0PVVBkg+Pjl44f4EVWNt0h0dH6jOTC01j"
    "956RoaHeRh3SGPwQpN3QnYvg/AywEphoFA5u2eBE8Gd9cGMWjBAKAggMRNhTgU/uwZ4ihD7M"
    "+BCmELnww0H44DZwBPwUEg9e3AVX7oJHg0RBmEBIAbjZhqkCmBH8xe419uZZ0dgXl6jr1y6O"
    "Hn7hWc5qWy2WFzl2x5a944R234HFy4q60wvfecrsfv4sBAzkZfBiICkMj8D01C1e4DFu1Bbu"
    "VboHbNMyLaO3/2ifmFI3aqDx0K9BXnQsSKYgi+UYhyUABwBDd4IuDWBAFndG0cwADQcMP4u/"
    "KQWml0UhAaMznc3Xpxgg5vI7bYYdsHsKXf1ZgE4T0DCoUsDzUMijJdI4SfsHBsMg5AWuS66q"
    "CgGVwGgl49ZEXYeMawBAb0ui26aHTAJUAApZU7ez7YSiBIaXRV40dEkE3QeRA4bKImxOZL2d"
    "tsGzpxTtTsGu3Sv7K12Vdcfk75c0+SUnVdp/6/F2WoPFh70Dj/BIInzriMq89DntAD23+84Q"
    "iQGuTEKUZuc9DMHzoG8AksiSZNnQKUWGybsgCRBj9M8BYSyGtEugNE0zCJIwNMNynO85giRP"
    "TSL6ARSImAZTJc85hONdx0G8hLgIwQKV8jOzUCoALwJnAlXJIh002imh2M4TcTsx0O122M4T"
    "bjtF59sZu9bm7wII2p1qu3NpVKn9Ua1dKKAoq53bq+0l1jMGPwdxCIv1bK4oBE6Avl6YvguE"
    "h3IR4RYVBUFKoYZJtsA0CaMEYRkukhcFvU4sOxtSn505eqLv6Y0eL+VV1CamAfMkEUDmgXBw"
    "4fLVkcMnU5rB/UAw6Qbgu8DLcPPmFy0nLuTkyPfsIPQsO5fPD48emb49tmf/MaPFe3a7tkG5"
    "EkxN3c6VBwKnYRjN5qKeq1QPjRzWmyCpELtAEOhyoGmZXUIWYhkEPzOiSYOGpkd07LUZqGxj"
    "zAC0XW0TJ2DyoHHZKMsAeS9QaGsGTNIWNQch1xZlZO2sE/djPrO1w7WBqpmVUbgiScmA4dwi"
    "VCowX1s0W74is5YdmMaC4yW4QJEjX8/XA7N54tTpOCwhpyRBvdnSEZ0/EUmSdN9lM6+FHcBN"
    "31waGxt7+ee/zqkCnnbC0Hg8mk3LCza6zd0zUPrNP71dry9ygnb69On7cnYEN31zSRRFQdZO"
    "fmdv5PpOELtRUm9OUEy6wRDCSs2mPjM9MzisPdKP/y6eAyJCGmR3LHgk9x2AL69+LshSy/JK"
    "pVJBk27evMnwueMnDl+9QJk2MBEkIgz2TYx/NV6sVKIEMBaKDBWGESeIee3g5cvA0llcCiJ4"
    "7fvJx+//d75cZRnKD2NMBlZzUdZKB48cun7xvKxIph2Uu6qqyNy8eVtUiy8cOzj+FYVqTI5D"
    "oR9eHAnhwiz0aBlSm8OQh7GZyV69IFP+aFeGA54h/foff5Gkywz9q1c35ldlXpKMarV7RX8W"
    "Z3p6Xwij7EZw6S9fAKNZpxg6DmPbtmmG4Xk2jpNiqeJ7BCdNY0gQ9PC257oR5h+GoajUNKx8"
    "AbeU1tQ8ZiHMz4SF0INKd9KoN2k6zfIeRbmIcSmW4zlVLbl2A7NtFIaW43IsiycXl1SuVBt1"
    "mmQXa0DRUMRYj7uHLdzJRTvTD1EboUFmM2AsPtPzOjExUS3l/vDB2VKlmKYJYShDdwulQhSF"
    "r7xyar1Rt74aC6OQIfyBAyNLPTtzL/bNJbT7nj171vyIotY9dou12SSOzZY7vG/fUs/z+P7Y"
    "lET+5xeuFvJK4lOMzM5OTL/02vck7rHN+Nzuj0dhGPd0VxmWZlSCeW333iGOZZ5AznO7Px5x"
    "PI/JzPPCcjmXz2lPLOf5PcHjUeA6U1PTeqvlBcHTyHnu749HvKx85+TJKIqw7XkezwvUE9Wx"
    "z+3+eOS1jPc//BhBv+fYFMu+dOrVSuFJnsMgNJXeuXUdge+KSmh1z5ofddpb5N86w2rJ68nZ"
    "ePg2kuUEg4ODP33zL59g7Ny9r2m4H5oEniMts2WY0zTNxlGYpEmSAsMwAks7Xox4CLVHbGzb"
    "HkWnDE1jNse1JEmSARn8SBKjwI+BRjQxPbeoybxuepLIIluxmPdc2w9TSeBajidwLMKtME4J"
    "IaLA+Z4XJWkcxX4QSSKH/JKM3aztesiEkA1rBNuNCjm5abRQB0UWjZYniiyhAAcSwlFpKAiI"
    "dV1FZPWWK/IcneE5VC5GXCXwrGVn32P19XZlt6HbRHFEhb595dpNnhCgacLRUZAePHwQnXfT"
    "sb7vZpjxAVHXr55DVZfeoEFpepNMuxWejQduUQKyIZp9eo/dRq8PIurQoaNN3YC28/E8Hydp"
    "uVTceIol3HTt6qXualenE72BIcy6Ud4PghVLZ57UfToDtyjhiSf6kxIhjGnoAi9W+/s48hga"
    "EsKy7P3HsNDz0e6kuVjz44RnWdw2PwhFgTd13Q3Cnp7e2dnZXQPbcNP//4YC37McNwgiwTAq"
    "peLWB9q2c2d8XFJzpWLu3uw8aXdZUzNzthPIcgYKsFjS67O8KEvK88dUVxIniH29PdkXKVZr"
    "2nG6e/vIZheigeehha/dGD+wp+fWxMzrr2c3aMSybDeIy5VKeRlrqZQhMccyXfdb+JTFRoS4"
    "6fr1sWpX2WlZCSGSVijm5I2HzMxM37lzm2NSXpKH98rTUzMs1geXzn+Spuve3KcZwabpLYrj"
    "rZQN28sWxwnD0NsiaotsumFpW7gbiKKYLAv9t27e9H2/t6e7XC6Jooi2nJ9fIFgzsOsrH2Op"
    "Qz2yuiSOzFZL0Qrk0VEcy6B0TPFrysHCkWs/Bty5RTJMU1PkIMSqYI3ncpENJ8KTqCrZw1Bp"
    "mvh+JAgPHw9eKs/ZZUogz0KtWe0qrRDlOZakaJ1oUK8v5gslZpUn4YyoUk7Tlj+xjFI9LxCE"
    "+4vK5xSOzQbiSj3fy2k5ZPb9jGH5wgOgkG1JGr49cujAsi0JL126XC6XHxuvRq75z//yr3/3"
    "9/+grnpG+j/fe/el7/3g7Ifvs6IU2H7KUWIamkESJTSJozf/6m2Ju2/izz47N3ZrXOEYw6EE"
    "ykqJwvJk+MChF488VNE1679790OGiilOzOeExqzOi0ySRkK+7+03Xl8x9e9/f8aPvEYjKqgJ"
    "r/W/9ZM/79h1fHxiYvKOFwLDUm/9+PV/e+e3pZ6++sxsvlIJcS8lokjqj3/0g1Zj7j/+8JmU"
    "WEEiMiT1nVZE0X/7N7/83W9+GxGCde8bP/lp7sFj6Gf/+L/jk3O5UtnV9ThNCUuxnPT2z9/k"
    "l/nBmTNneI6nOYUhNBO4EUNs03z19dd6e/s8z6UuX/yUPI6/r0lBGC85cqNR53hRkaWN2bYo"
    "bTXhIYhTdHMG/R1bLKG3LgpB4cJiA0Mztvww5jl24xlNE9ObXyzmcCKx7fJPrz/L8mHo352a"
    "IUmcBPG68R3RQa5Q7hy0dWcKQq69jFyxeytsHcr2NU0ohk2T+H7BniaeH246I/KFYYQl8f03"
    "aYKBZ3UeWpoRZ1kSni/3duZdjg9WK7bmWtZk28oyOxT6DtodsvsZhl7uNBfOXzz24vFO9EGz"
    "TE9OYtFPpdHE14v5Yv5nb76xWpzrunfuTBw6NLqxQgiU3z1zJl+uJDHNC3y9Nm970XC/emPK"
    "EehQknjCqQNdsiJr56+N56uF0Gzcm63tP3zMmJnQHef0j97s61IfGDr95KMPTI9RVX5+Ziqh"
    "YGj/CyeO7V+t2L+/8065uyunlc2WjuCVZbhavRYm9K9++Rb9AGe6ln75+u1TJ09sor/rYmLc"
    "1O4bsN24fnW+1sRPqYuff9J+1Oo+mYYhygr7IB2jv/cODCmKulTYYMQhZI2UgOkF3afjfesR"
    "siFPFEWYEh0HlRNomvZdq9HyquVSEgcJkDSN0WtphnAsmyC+jrNLhaWKi82Q3cP7OMdxsCDD"
    "lu36siQybVo9IwUUYTEuta+VsDh2nKyowAkw1T6QlumPgJKQTfVflnKfhM2xzSgMTKOx0t+p"
    "2GtfUt1PmujvRqPRsiyIo4bucjw5/sKx1eLQCq1WS1U3+UEBst36cqxQ6c5+8+J6tmXqpqPy"
    "SSuWHGtCU5U4ZTSB8qIU52I4SKPQ0I3de/fber1Rr48cO1HQxI6o2ZkZ03Y0RUoogpWDtNaD"
    "xcg2PTkeM2oS2IhRDMsKfZ+ChqLlBvof4vA0iWbn6kNDuzfVfyt234DN9+wozBor89L8Qn3F"
    "JU+SAk8ILwiiwG0waxxv4UewacxwfH1hfm6+hpXlQq2OHo2HffLrCazL5mZn9aaht2z0YqwQ"
    "ypWunKrkC8XA91mOLZaL3KPnaebePYSCOMrQTXt9fIfZgmPpdq6javMLNLoZwzZ1fYWp+M0z"
    "ynbSyjgTxxEewY7l23m1KitSu1ym/cDvrlbXWJvnYTTgOG7jybJTT1FLpS5GGwxZGArTyKd5"
    "NY2DpZiYxGHLckNMmlEQZQ9CCZqmtQw9TtJiudK5ikKfCoKgI4oCrFYZnlsjzhiGoWkqlg8U"
    "QyRR6My7PARjYee6waY/3/0TxplrVz5T+kcHu/MPtgGatQVdz+67b08sHD1+ZE2JaIWJycnR"
    "gwc3VgiP83+99x4ryknCiJKwMH8vTslQj3xxvF6QsUhOGS430K1Wy5XJ6Ubvnv7INr+4crV3"
    "cDj1WniaBDGnVQudGc+d/Uh3KE0TZ6cx87MjoyeOH92/WrFrFy/yilwqdTfrC64fcCw3tzDP"
    "cNJf/+JnnbyKez/VaPb0VDfRf/vizEp/X0Ho77uHRhRlk6+ynsARHrQxhiMu9dAl2z/XxxSa"
    "oLNvKq3j71jRY329Xt22WrE1VX16R94i27r+voLQ3+/dvUvzHETB7LwpKeIrL59c0wqLi/X+"
    "/k1ujNGyH37wfv/uQSqlMIj4rq23vL4SO9sidORoqkw4uSvPxQlTq7eAo6jIr9eaQwcOzt+d"
    "YDnoGTq2q+ehv9+59WXCqBwTL9YWKDotVodGhvtXK/bHjz6SVLVYKIdRgMEQi2vbwQoo/uH3"
    "T3cyGR7EycmpkZF9m272dvn7ysrp04//Z+DQd/uKDwGnrCqiJCEmkdQCNtabbAtIB08X/d1T"
    "p3TdQMypMHKsKb19hE4CPWj27x6KIz+lOAzVWk5OaFLpqniWWenq5jCn7x3U8nmef2T2XKHo"
    "+xEvqkmaIEuhsPaN1fD+/YoqpyltW60k+zmVpOVUTNztU94xPCaP0mbabyNRK+8JwsBH9Ege"
    "3AxglTK478imBeLTA4rHZUOf8n1/U+979optzGbodd9zTaO56p6AIgi5g+ThN664vE0BEfJs"
    "5WvMbWRbiu/PcsZtYetU2yvj+1djY32Dw8oy3DQ3PY0VIsThfK2Vy6vHj6+NmxqNZm9vz2bG"
    "Ss59erardxeVgB8GoedgfK/mmZrLUVl8VxhWLGskiujbDYvC+B4HjVpz8MCI1Vy4N7dw8uXX"
    "CjmpM+Pk7TsL9bqqyISX9+zqw8y8pmI3rl1M2Lxv6XsGd8/MLWChijYRFfXQyMO7T4zvd+/O"
    "7N+/dzP9ty++o3dHy/7fn1xO/b9urvu5jeMK3+F6Qz0QhQSLKCmUZMoyVYYzmfyiKPbYymQm"
    "/2iKHTsxHYVylIw8jmU1S+JIIiERBAtA4lCu17wFSAASARwnpspkfyE4WOy+be/7XtlFbimq"
    "q2d8XqR5UOvAuUiWF4WB9hGK/cMshZlOYPZPz5zUNNUNAKopgAyYLQb3ykp1Ipf2XTvAMc3y"
    "YxIvur6clkG/C2KMIPBYMg3WJjDNbhcwtkazmU7LAM8wjzaQmiGCidEEIJMpsgJKFaFBZ0J9"
    "x3ulPnAiMSqGy992XITO+4hq3RATwlWw37pfSPEkxfCRrucX99A9PAL3vYAXBJahhw2v4/YL"
    "EShAV1XAEIUmQUsQBKHrhmdr8/NZEvNEEQXMwFBQFBVMMEWpAeWiQBqONXXNdjGSeGVp586e"
    "AXLS+RVBkMMEy2Qztkcm4lHYGSdOzHRIMwyqv36AtnzwVucdR9ejevr9wd0fps8s5FIHbj8s"
    "0NVWO5vDK65tp/KZxbGxw6oLuqlWd1OpEErgufYP33+HEbBCeDshoubj1HRGuP2oPC4Df3cI"
    "Op5P84Xx8XK5livkI5bxbGUlPz1LYc76yxfRRDrZp2dufL0Em8J3TFqQL188X5jIY4MEW176"
    "Jpqe2FpfX7g4//2dB5IoRnCMYJhPrl3tVXMsVVXH89kQ+T0v1CYfXc04SBx6Z3ZTd/o6YsDH"
    "bjLeUYhKv5+g285hNOv0GJqF+NbtJuV1XH36+Ce5MJuU9mkQHJeXq6twwiOBu75Ry+Szi1cW"
    "Bs7Czk5lamoyZLJ8969f/mWiMOW4yC9raFqtqZ7IcMUaRnqGFBUoJpqNExQllrdqEY4MTE2p"
    "NWbmzrX2Nner1YVfXhsfi3V7XPnpnuFxPItXt7d83M8Wznx4duawYP9YWmIEXk7lTFMNQG96"
    "vm7oLc38/e8+69lNnv302dr5+ZD4wfHhaoD2O9CM7heNRiMW67mHYL9H42mh7X8H6GJYJhEf"
    "8GiJ2S7xeMh7JlAHsLXebEUIAgWIApemWdfSN3eb+YwMuOphBEkQLE1opiOnEkB1Vc0Em8h3"
    "LR8j0nKq3/8OKwF/IyQFKoLnOBASkOBwj0qtFo1FYbpN0+okd6IUO5ZLxGM9/7uhm7YXj0mh"
    "8h+nn4DqCxcorul4ksTvn18QF4eDD0zZ9zTDATga5hcDpXYU5fCyuJpMZ4gIoWoay3Ctlk5F"
    "nHgiBQgJRNB2fJ4GuV3bcrd3Kp7rmIY1XpiEI6nrquXicYntNgU8h2GjvmezDEsSQFHwwwIg"
    "2lOvtxUuy/IcEcE1TWdYWtfNXDbb7ydothpHkf84/QT9+TOlF2vjp6Nd/h6gaKfNwRCxQK03"
    "8OH5pGCkhQoE59kwreLqKugZjmOV2h5O0tk49ePTlbwcA9RlhWQySqcSCdMw0tkxR2uoDVWp"
    "N3kGV1stVrTbLxLsl71KpaFtxuLidrkUoeiZ2TlMHuDIBY5oOxZFs2vPn8O5pklaaSgUw82i"
    "KEfPT+BjQbj8x1fCcTU7PtPmakhEMFwHpvLCyQIED83y7cT5OtG4Tlaw3T5JERo0idPhALD1"
    "dAM5sICMQR2SJGmacWzLR5uI60auO7gKimI0wUCCUaTrY6OfM+sPDIyu9qb8kd/dupE5dbHf"
    "/7668gRDT1S4a8XK3PzZxUsXDjeHAnjPnp8/Pz9aIJjTL/70R06K+T4hiPz21obt4ifHpfvF"
    "hkT7DEMSVHQqJyWi8SerW9nJnNPcKxY3pk+fLD0v8hK/+KtfZ9M9XL21fHOzWmUogmJjVy4v"
    "TBbyAwX7+os/CPJ0tbx+6fKHt/9zPyoKoF4ojr3+yce9aq75cKV4ZdDQXmvtTfnfAXz6PYtv"
    "jkcCGLabDSxDb+qWnEz4nhvgKE0MOj2Kqt3Z2clkMqBGUBR8SIrPWyOIR6w2bL8D7q03dO/S"
    "hXOd/2G/72yWq8g/41b3VEkSP/powKaGWVCUei4XYneg6M+9O4mxPPLP2HZ5o1RvmbKEKza3"
    "/uIFNE6QbFIkACGLNS1C42DP1HZrU7OzrXqr2aieu7CYjPfsJsswf7x7TxJ5kuYnJ3IDNwc6"
    "iE8e+mTUbCmFqcnqXg3WFflnBPEXp072qvluqbR56tSJUPnfFK4CWSQO8iY6haSQfwZRLpxm"
    "BX5YEit61ScIg6bAT8gZW1eBL9Ik6RH0mMxxpL9erRTyad+xXc9TrSAuSYLkp9KypTZ4XgQ8"
    "SCYlAFyWo/u7UBQlHouiiz6uZzvesN5RQrkUszgaWFqluhuPJ2Ao0NEr9YNAjErh8r86Vz+n"
    "2ut6xnHQcnQdv8ivJKU4gfN99OAeTkT6L4v0ltGy4Ieh6giq+Z7XCZ504suaprmmirExEvME"
    "AXlagMUrShOtgAdN2sBDEL+E02HpSTnLMt0EMYSr0EjnVyN6JPDAciMCPwp+YRs2VSsth9wj"
    "GJF4e8RqQ/XMt/+8jePktav7L9SAntFbLaCSEdzdqTQphp0eZJTCLFSru7IsYyMLHOelvy8h"
    "9uITBBmpK7tBhJ7NSbcflTMJHg/cCB2fzIiF8YlSeS9XyOOe/vTRo+xkwWhoAeFLsbF4rMff"
    "v/zz55bvB65FC6nFKxeH4eo3X30eG5sqvyiOwlXPUdXaQA/Pa60dk57B35l/5uB2GcpsBMLY"
    "ftVrX4wj+mdQWAeOzhDPzP8s2But1mzsAY8olzde3+8PHj4yDfvKgRMG9ntprYjTJO67NcXU"
    "bfP6p78ZOAuVSnVysoCNLGi//+2riclpaNbzPVPXlKY+Ncau13HCQ3EPkhazCTIS4bYqdZwh"
    "MFuv7dUBVxt7jVpl/fLV69mk2O3x6eOHNsaDhVfZ2cbwYGz89AdzU4cFu3njBisIqdSYZRmw"
    "PK4DhjcKbP/204/7/DPO89UXH5ybC13sn7nfK9vGtzeXQTHgjx/ege6bzVbnlYNSqWS72OxM"
    "4aAJzHE9gNYAbUMHec9TicPNARq3Ws1EYv8rpT7ghaH2KrqWadQbTQyFUWDLexRF+46xUWnk"
    "s+m2E4YiCTwq8vWmJssp2BqqYbIMC/MCYqQz2X6+uLdbBb1P0gwyZTmAgZhwKP8bUGJ7awsQ"
    "wvPR+UDGX/t4MQwrp+XuGfEcq6mZiXhI3hLITxDhNwYGVvNcgCvn7t078AHsUPzZyv1bt/5V"
    "mJzKZeTQFo9YShubCxcvH1dr/x9F11rLy8vZbLZzP5JsNpvwj65r6yXruPqAhVVbjXc90ver"
    "bG1twTxL4j77Alz9N5x9iiJVzRQELoL5hoWuRbWTocF69JCeIdqMvq0OCSJiWTZKicYCsBOR"
    "udh+kzPwUZ5XgOMcyxiGiRKdCZwgqVqtzvMsTdPc2038fM/L/sV1TTfRpWGYURd5o4BkdLIk"
    "UHqo61q2g15ZwwnbsjTNQDFkH7EI0J5AMRGXa78LohsWwLTXaaBd4DPHsbA2KOTov1WH33te"
    "/gtqRDNRb9yo9wAAAABJRU5ErkJggg=="
)


"""Help image on default search page for search page."""
HelpSearch = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAH0AAABHCAIAAAClA9+5AAAAKnRFWHRDcmVhdGlvbiBUaW1l"
    "AFIgMjMgYXVnIDIwMTMgMTY6MTU6MzEgKzAyMDAqPnlpAAAAB3RJTUUH3QkDEhEPoRUhUQAA"
    "AAlwSFlzAAAOdAAADnQBaySz1gAAAARnQU1BAACxjwv8YQUAABl1SURBVHja7FwJbxxXcq6+"
    "75mei8fwFklJlqx4fUB24nh3rSycIFhgkQBBEORHJkiAYJHFIotdr+NVJOimLlMk5yBnOFd3"
    "T/f09bo79WZoSlxvbGNX1lAxP0iNx9evm9316lV9VfWazMP7N7M0gTN8C3ijgGVZbDAMo8hS"
    "lqVfPz5NMz8IJ5dwHCcK3PEpHoWept9w/Rkm0HTDsoeIy5feGNp91/UyyDRVxVODgc1ybD5n"
    "ZDgbGYsHnmP8IC4Uy5qmo4g5lul1DuqNfV3XCmaenfa7vGb4/HfX/+Vf/53EMbZv3bl/+87W"
    "pD+MIm6s16jEGSsJkn58CSdIoqhAlvl+sFNrPXj4FDv5ab/Ia4Z//qd/dJyhJMv+aIhqiyYH"
    "FRzNjqapPE+Fie2U+BHJBP5Ipz3XwaMsCbIslQt6oWDSYVt3r5/ZmW8JnhcFURg3mcD3UOJf"
    "Px5VX1YUjjtSbn/kPr/V85aoFMtzOF3PL0tSYIC6hSz1/AhSIohiEIY5w8CzzfpuQkLPGymK"
    "Mhk/eQ68QxRFSZLgnRiGkyRxcjaOI0EQCSGoFzgSG4IweQc46kwTkmZxFKlji4lICNF1ddrS"
    "fg5CIvz37cezLBOFwaQdBCHq++T4XO4o9I3zlzvN7XrH/cXP/03SC5osjtw4iCxWUH76s3/w"
    "Oru72zutISkUix9/eNVxHLNQZLKk23kQBeGTZ00uC6M0YUCsLi0VVVDKS9v3bjGy9hdX36FT"
    "Atlw6Ea+d//BE44jAq839tsXzs13nOjyxdVPP/3dbDlnzi4LXNat77z10SdFlT5Yr9+btqhf"
    "GvoDqzo/a1n2zEwZVZnZuneD5YSV1Q2WO2I5jmPncvkTF6WJ7Y4YSLHftu18np5tt1qExOPT"
    "WRQR8WgBTuY2kGU5CHzUfmxMxoRhKEny1zxZmsQkQ+NIvrwEPG/IwDes5dcM2fh/ljDN+rPq"
    "4tq0H+f7gvZBvV7b7fV6dDmHvtfpO7IkyiLX6uByKO/uHqyuLna77VKxnGaoc2RgWZqWEwQu"
    "DoMgyZxer7q4QF05WnWGy8aqKvIsiYmi6aoi/8lP+P8TBwet69dvLC0tULnHUbj7bBuDMUni"
    "GBB0TThsHxqmgYS022nfufdElVhZ03wvnF9eUdmovHze6vZu/c8NozInMaRWb4kcE5AkHNla"
    "rnT+4hvnVhen/YKnFHNzs9c+/uHuXv3MzrxStPb3ep1Wrd48i1engyMeiaaG4YSGw3Is5CXw"
    "CBgCuBHIeAwhLwNJIUoAQzCBg5CAKkDfR3IPhgQtlw5A4oe+OiDgx2DI1GljOyfRH8MERI42"
    "Csr4qMIoQmoP+LsiAqWXQdCjMPSDUFGkMAwEQR6NPF6QWCbDsAB7OE7gee6YJp0GHMk9y1KM"
    "kho26ycwr0PdwVgLFAGGIYQpvFEGN0byQ9soKRyzUYCuB3YE7y6A5dMGyr3tQJDSuRnFYEpg"
    "h8AysGrSkcOYThhJQOZhtQBf9CAdz9lbsy/nNdIsJXEcMDTUSpIgjmOM4VDWeMT+JMnSjD+N"
    "chclGnNeKUAUQaEAqxokCbAcqDK4Hug6TgxYPWAlyBsYhmHwCRdLyNNBkmCuivEsDPqwMgcY"
    "izEsdA6pLnN58H1gUzB5EHUomRDEdIlgeJuPoVSm98Hfgv/cIegGTKJunN04BQwGCIEwxNga"
    "yhVkXEAAdA26hyCPo+PJIzkOihUKRRypFIqKbQMGHgyTpqnmeZ6EPIHjdF1/IQY/LTiRF/vV"
    "f0GrCUEI5zbhoAWDLhRnwe5BpQKeCzkTDjtQyMPmKnx2AzQdFpfgP/8DBAUUlQo0DqhJWV2B"
    "1TUMpUBTod3FXhBV1EcIRxAn1PgUdKr4dh9WN2HnCZ2SoQOVEqonHLQhpwEjQORAzNAZTdEi"
    "8ZBToN6E+WXwbDoTWh6cAQxtWNkEiaGnUgJLy1DbRwWCa3/5+P7WA9QkHsPfyPvok5+amjRt"
    "Of8+TvCZ7e1tq99XDV2UtCRwLcfTdF2V2Z4Vzc+Xh/ag07NKlZn5itlsdd+8dHHaD//64Q/z"
    "GYzUZVXN4kSSpTCKNE1TVU3PFWSJT+MQ/epCdQ7GyctJcucMfzRO2JnN889VeHH+ucurVGbw"
    "uPTCyEKxPO0nf71xxt+ng+f8PQW26dIiiYkcBqk3ksgIFJ4esQc5BnpFdJscgKlCGEPbhTmD"
    "+kkkixwDGQNMSi+UOeA5yhd18dW9BvJ313NlWSUk4nkx8Ee8iPwdfTWPpxiOQyKZyxnTlvZX"
    "5E4IYXmpNYRRAnMaNIZo60ERwQkgmvB3QpkDRlKDADZLlOCrHCX1TkgZPZIZjocKsg4bVJGy"
    "/h9UX6ncU1o0IVEUYxwiivg6ScbEk4In9mMn1YvThBN8Jh6XaydloyQOepZfKZssy7XbrYWF"
    "xVPIgl87HPOZE371N7/5NQTdX918srS8oXN+3w7jNL7y9lXrsPn3f7cAcCb4l4YTcr927a/o"
    "8W+n/VDfA5zxmenguV9lWNaNWCQwHAtBApoASTpOxRDag41RRHNe/LidjtOTEneUKZu40DSj"
    "lohhKb3BTpGnbAfHvBLHkEVhxHJckiTIYdBRCaKIHvX4NMMwx9sXTgOOeWRAUvZhXx0ROGdC"
    "3aWkEClAEFMe+c48zTjKLPRGNP37VhVqFoQRrBSh6VDhMhksmNDzwIspn5FYsMbZ4zUdWBkq"
    "330eENlMp9sTJTEKfFUzLGvA8aIsUUGTKESylabZ4kJ12tIG2/EG9qhYLJ7gMx2P8nTU1mSS"
    "GsyoqqIGiTyHPUyGBzZJU1T5jOHgqLjKTkbyNJlO97dyY6XjxrsTkEEj95d/b1NaluJI8TRp"
    "3yuDbfXCwHfswQmRdHYfdjr77a79wQfvP3qwtbAwt7K28dmnv8wy+dzFcw9u35Y0c6ZSsntd"
    "XlEMMTm0kgsXzz+6c2NhZe3Ro6ezi9VR7zBmxeFgUK6Uqqvrt6//Os0SnpN3tuvGbFGXJZET"
    "fvTjHzabBx+8f3XaQpgmTuh7GIYp3VeZKbKMJpKQWJIUEscZpBxHjaYoUkOO6gxjUkmtf8oy"
    "KeEFPgxxsEijFwxZxrWkvu1lSZTPm0cLh5p+jBsTSRQsxy2XitN+9yngD+u7JEkvtic/8vzR"
    "fqaJ0BEvOqhxSxx3ipNzx7eYl//PCt73U+gv4oxHTgdH+j5yHZLx20M1SmE1D60R5AVaFMVp"
    "6YzgUoW2Ue0HASWIG2U4cMDxYTZH2U42rqniUImBnk+rsuh/l02ovMIdpWi+2u2Wni8NrV4u"
    "X7QGPUFSVAXXXjp0PEmWwyicn5ubtrS/InfkXiRNJZ+aDDemdbjDmErQJ5CTwYmoKHFKRG5c"
    "1QyolPNjsfIMEJaexRvJAggRrV9nCXzTFuWXD7NQovvQdQO9jpHLczzHMiy6lVwOWS57qpKR"
    "qCdf2neG4TlOGe3GwShXMPf3aobMW14AMUYjYqSpG8vVru1f3jz39fdbNqfzHuixZZn6oYlP"
    "+nJn+ASnaBvBMU7Yd6QuhpFLEtLvHg69wMzpas7MkLaPP6D6fjLu7wgn63ybm5PGz6rLXx16"
    "Vtl7iTjjM9PB8ac3tDA2CDn0k1YAukQ32mHsQ1JaY0Inaci0tqeKdF8YOtggpj4WmX2c0MLe"
    "YPwxiSFBP6C5Gnk8pqjQwhMOQ6/LjVNjaUbHI2jiJHs5O/QmiMKQblRSlGA00gzDc11BlAQa"
    "eWRhEAFLvZemadOW9lfkjn6fEPK4wyFNTFhKGXMi7A9potEK6TbHEQqapcNmdCpHf/yVD4o1"
    "Y2AlRxM7LoH3qnBgQy+gRAjla8rgh3THGfIfvBzncpK59CKaxfzx6st8DYyy4xjly0VRREPn"
    "8T49yKhDiqKA4YQ0SU+T2L+Uu6xQ3bugtEulEs2ZMkyn03nnfBkbjjMUBF6WFdcdapqOLC0I"
    "ouFwWC6XWJZJU/oVIf1WluYCmKrBhWE4/kiMbln0o0xVxPGAbJyLBc8bcRyrKAreNhF07B/n"
    "0dJJrjgldPMriSO6zU+QBI4NAt8dRaVivt/r5UyTlkwZZlI4fRGyosgTEkO/bYTJl28TGMap"
    "YpAn5T5Bo77z6afXeTbefPPtg51HtkskIeu60WxOrswv3H/wMK+perG0srSw9/RBz+PeWJ9/"
    "8uSLjTd/sLt10wkSVTOv/ejqL37+y7X11dv3Hv/NJz/qHjq1douPHI8IXJawTLJy8YrdqiHX"
    "brZaI3+kydrlS+u3tp4JrCCyMRqKSmX22V4tGFqMpL175cJurWEPwyuX1+/e2ZIVURRlRTf/"
    "/P23py23PxVn3x28UnyZF+uf8ZmpgDlhZ5rN/YNmXVK0KIrXVhY73f7s3Fyt3lBkWVOVL548"
    "rC5vcjykeJoaYm5xtlRrHKCFyKli4+Dw3PpGfXdbzRWYOCzPL3b29zhJLRfNXt+WZXk4dFiW"
    "0zXZC8nIHmi6Xl1etbuHgiQ+efKsUDDQbVRmZhKSyDzbtWyzVOk29xheJHEi0kwLev6sOlOU"
    "NM3uDwKSlAomeoXa7u784uLebo1l4MKlN1uNvbyZC6OUpFkSBpLEt3tWIZ8/f35j2rI+gRP6"
    "LghCZXZekuScoSKDhCxJ0MWJGKhSX2fkTU2VJFHmBX40CgyFi0EgMSER/VMfpUrFddxiqey7"
    "Ls4L8oql1XWRzURZy5IYb4ERvMALrhskNI+PTcnq972h22630SWiz5VlusNLEER7OEwIURV5"
    "YXEBf1R1jc3SMAxESeR4jhAkL0RVtSgMWGSjDNtptSVZwpug58cJdtyRIon0o4OUFCqzlYLJ"
    "8afuzzCceKCtu3fyZmmntlcoFW3LanesZqMRZPL6nEZYbafRHg79nM7X97vACkwaf/bZXS2v"
    "5lRlf3cHFOPCWvVZvafwpN3utrr9lCTFQr4yU9h6vJ3Gj9+9+vbTh/e0XOVgvxPFwXtX3926"
    "fXNucWVgD5GM3Lp5b3V9Y+/Znp5Xu4f9t9997/HWfUkrDFo1QVXiMHPsrpozXdf1rHbGafPz"
    "5Z2nz1SzILAsasBura1ofGOvCVzKScaFlXS73vvJtQ/xpfKG/kfK5rvECb/qDp1arTm3UI19"
    "jxUkfMdyudhqtatzMwf7LbNcViQJFwGai8NO19A1hmUdyzIKJWFSXM1SASmkIAT+CFngoNfJ"
    "5QuKzO+3rZKp4yJAIpgmNNuz39xHC4bajaobxsTQdcsamKYZ+H6E3DuKi6Ui/opyqYQqjKsQ"
    "79qoN5ZWVnDZjUa+mc8hX+/1cXkVkMtiJxLQKBPzmuSNfFynuq67ttWxhudWl/H3GqdG9Mf1"
    "phNy/+/ffhYFw54daIp+9cP3nUH/ycO7mVBYm9UO+qP+oLO4vOEcNghN/wFJ2MZ+569/8uGt"
    "G3cuX7mwdf++mK/OFhUSBrXd2szS2kq18Plvb+qm5vqZKoR9N/v444++eHRXN2bTJMT1ouq5"
    "R0+/IBm3sbaAMy2p2uef31y/cHEmr+6322GSEW904c/ebOw2NtYXbI+gh+i19tTigimxvUGf"
    "FhQ5cWNj7f7W46LGr2xcatR2Wq2uaWobm+e3Hjwtlgxcl8V87tLlN6Yt8K+V+xm+axzL/YxH"
    "Tgcn/OqDe/cIUgW9aPUOc7o6irPNjfXD9oHrBTmVc7xU16R8ztirN2D855tKBT1J2DjyfD8G"
    "JhN42R32kSBmnFzQ+FZnWCkbzogIWRQjq4sDQStD7DEZOK5r5AtZjOyGkSW+Ol+pN1t4oSZy"
    "thuoijhEN2rHl88vZKzY63azjElSghYjzdJWuzs3U2x1rFKxmNPEncZh0ZAbB+2Zmbkowmu5"
    "3iA0NLFU/t92rvQpjuuIz32fe8JySyABupBRpSLFVqIP9mf/v66KykpkiUgWFjJYHALEtbPn"
    "3PflngVT3hQfUnESkCv9ZWdq5r15815P9+/X268rUZI5Zi/yg5n5u7JwpTYG5UPzzgkigeWG"
    "7dcqpTDJOAwMrAe+r1qthJ49OV4+0U74TARQKIhcmlE8zxqmB5ivbzmz0w0EpQSBgZfHSIak"
    "cBy1dMtR5NLB/sloo9Hz/BFFAszZ0To8zwMgjXOGZVKG4YrgTo4wDBt4DgDBJM14gSeYYvdp"
    "HAZ9w5JFkeP5PM/BaQNUZVlaVcGpQ6t8cnIyT4Jpkk5ylKHIOIsEDuE4DqAty7IYkoV0QKBX"
    "rqDUkH3/8d06UJWOHTx8uPz06YtaVQVIM7+48O7teg00189ZKq83Jo739lhZ6WkHSU6KAssL"
    "5SdPHl32i3wach4nGJr37e0dSRRECVBaVqTnkWScpI6tO0E2NlK1TFNV1SCMEgBuUcyCdlFM"
    "msENoNQiEJg0zUCBC0SXp4bpqmXVtW0UxcB+DOLMAEBR2y52wxB4EYsHdJikaeT7nChmwFPT"
    "DPTU94PQd+IcJzAwQVRUkCyaIDDLsmiGPyvBmCMkRdqmiVN0kQWl67D4JI5iRU7k/3CXyW+Y"
    "9yE7Mz4+tru5bvv1Dz+tR0n0py8eb6yvtzv98UbFtdzQ0zstvtXtuZZH0LQs0hNTM8CPUs+w"
    "3JQict0OHj18sPJipT5SEQV168MHT+9P3rxlvn+vSNLR0YFUHvH7xyNz93fWVkq10RBMktnL"
    "UPHW0uLR3rbW6d2/e6/Z1ednp4E0YQyrcGS73UcoNnV0kmeznPKDYGqs2mpq6ugEGnpB5CdJ"
    "YvopnmXw5am1xp3Fucue239JhvT98PCQYxhekoHGn6o8EBm8oEIU4HXbA+5O4zhoLQomGH6Q"
    "4g+HFDrxfQ9FYS1IoPtF8SwUBdJEcTLHkC2tVavXoDdQTfCQsiyDshtWIPBFScUigp/ntmWD"
    "UzX1nqCUqSIXJwfdh6fChwW9FY9AMRgJdAJWHlrBJc9xSIYejAXttNtg9OFm+AxzDC8p8m+a"
    "kv+mXKzviqqahvH9N99MLiwa2nG725GkyvTMxOaHjxWB0O0oiGNwcKrCHX88ZtTSaEnc2tln"
    "eBlLHbk+ZWhHfb07M7+E+IbteeWRad/qxoG9svIaUNDd5Qehb794/vre0o2dnYMoTfI0nxiv"
    "d7S2HSZx4IHPrNRHBaJYfoSVro9Vtg86PJH0dUsqVSkisewQjAnNsNevzWxu7RJkTuIkK1cS"
    "z3jz6tXswq21H97OzF4vLX8C0fkhfW+3WmmW12rV82qG56UJi8qDcQT8EB8gftAz3bCAr5/e"
    "BG4gCgOG5+GibVmSomD/34V2kVys7ztbmx3dxHOMZkjAvxlB72+scuoIkkWOqecIu3BnUTve"
    "+3jU+urJo5dvNpc/u72/91FkyU7f9j2PYSnL6PFy+fPHf6koVyUkckUF9D3/RZqapvd7oPLn"
    "EoAj84P09CRL/CA6PYxj4DzOr27MB38lXyxwSWtqp8dFqzQCInZ+WtSJyzI4+KdWAGzOx2D/"
    "cjUMPMBTpw3Pezi9pxhMlja19lAvadztm78e+eWKoXdbzcPt92+H9B2Ao2naf3/67bU79/ST"
    "g5N2C2DJ7OzMxvY+mRaUdO76bLPTj+xOjMvdk93Hf/5ie+cgDtxyrW72+xmKLd5Z/P75S6A3"
    "aBbdWbr//G/fjYyP97utqanJ1Xc/6lqz1hiDJev2kq+//lI7Oni9+hOSBDjHldUqlnsnx8b0"
    "dHlj65gTJCLxPv/yqzcvnuEkj5Co3Xfq9dLstZkXr1aJPAsSpFTiOrqnMNjM3I31tQ1e5R8s"
    "//Hd2pqmSLsHH4v4JIkvP1h6ubLG8dj8wtLc9cl/Vzn/44IO2fdX/1gBcF2uVZEMPTrYYzlu"
    "fGqm19YALzueSxbgmEKRDPByqVoHbD45NXbS7CkiDYtRkmXA8q4fJGHA8UK1Vvc9FwAOYH+r"
    "1yE5fqxebra6tXr9hzerd5fuAs4BHwB6evP27Y23q6Ik5jiN5zFBMuArvMCneZklsk7XkEQx"
    "iiOMYhgs8RIMTaMojCRV9SwzQ3ERABKauQDm3VgQhchzUoRQBNoNkpIqtzpNGMv8jenLnugz"
    "uTguVquPMjQJbwWWoT7aIAki9F1nkHkBdAkaFJgSRzCShi8XIKXjRcXu9CJxgwbiAxYj8gPA"
    "GwDptHaXJEkDVL3TIynKt504zdE8zZIIcCq46JZ2ggDPoSmgY7BIaJ6RJBVHoeV4DEPJigxr"
    "DLgQOgkHRAywY7la9Rw7zdCRehUIWo6iFIVTFBOHRW5SUSgHQABGsCxTHR0Ft64qsDrl9Kyo"
    "69WSIX3/7tlfDTu+duPm/Nz0ZQ/s9ykX77P57A8PMRRLst9XSd4rKUPzzgxqpl3pAMcnLmBg"
    "we21Wq2heT85PqqNNIoS1HnS6dk0RYALq1Qqhm7gJMnRZJIXyaUkReODRFMARp4LrjSqlIt4"
    "GcvS59s8gGrZtsPz3CDpD2hX6rghzzFonhhOrEhnKaln+XtI7npekf0qCJrW5ASRY5lB3fnU"
    "AxCbJdDPIGCQnfZ82hQOPdvyU0ThadMNS4qkaa16vRYFAQZeIYxFgUOvGH1zXffZs29hSlHA"
    "kgx7lrFpmzqQ8narLauSotZ4jtrb2nTDfHq6oVtOHgcAY8ARW05Q7JCPAHdj1WqpeXKMkozC"
    "seEguxFJ/BwhREnywSc7DoUh1dFxx/Ni39VNh8QRgqT9MJmaqPc6OhDfImGGwRMEFhOhOTaN"
    "Y9d2qo3G0f4uAKpKpba3u8sLgiwrpmXFEeB0rKJKpmkAYIfH0RQmitLRYZMiEIrli+r+OAba"
    "o3fbpdoIQJrLnuohOTo80HVdksSfAZdV50AWNd1bAAAAAElFTkSuQmCC"
)


"""Help image on default search page for tables page."""
HelpTables = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAH0AAABHCAIAAAClA9+5AAAAKnRFWHRDcmVhdGlvbiBUaW1l"
    "AFIgMjMgYXVnIDIwMTMgMTY6MDU6NDkgKzAyMDDSmuHXAAAAB3RJTUUH3QkDEhEazMjFugAA"
    "AAlwSFlzAAAOdAAADnQBaySz1gAAAARnQU1BAACxjwv8YQUAAClwSURBVHja5FlXd1tXdr69"
    "F3QQAAGCFKsoUsXqGosez3K3s7ImD/PgH5Bflfe8ZMX2eGyNPWNnbNmSIsmKZY0KRbEXgES5"
    "uL3fbAAURFESKSlW8pDzQGIB+56y93e+/e190Tu3rkdh4Hs+RhAYivw/HBTNRFG0hwH8ahim"
    "IPD7TqVrOk2TTzyOoE84llhfXr7zYEFVFIblRVE+c+rIKzpes6nwPGOYfiwmrK2tJeIJjmN7"
    "vxq6StAcTRKvaPU9RuAHs7Ozsig4vh8hWKO6TvMxSeJdx6ZpynGDXF9GUVSSwMD4/v25mCT7"
    "YYBjiONGJE1qShOnmHKpH8PQjUqlVCz8cgsZG0VIIowaTVSWPQSlSPzS5k+n00cV1fuXf53/"
    "548PEFIifkzgOY7r7OEVAj4IwqtXLisGkozHSv3py5euOF5IYKFh6AgllftzpWKOTsT+9/0O"
    "o1KpNur1IIj6+vslWVZUA8VRU1EQHE/39e20VDVdVVSW53ieJwmy0WylYtLi8nqxWMAeek83"
    "4LBIG0KGiYgCghGaZ3y9/v2YPBQT5aOTcVEg0RtXLypK61UfLBaXSeLpWAaKwwkcRf/POK66"
    "WXMcZw8D4BlZllRV23cqDMP6C7knH3/ydESluhWXJcvzI8+hOFHkmJfc/VY9m07uWCyoN1Tf"
    "c1PpVKVawzUTWAW29bRH0dAPbcfBcYIk8H0X8vxgXzM/CGA2FImeZ6p4PP48ZhzHP4+Z6wV7"
    "2ACvRFG4vr5BQChoipi9f19RdZygZ35z4uX8DhHF8UdRjUJkfWXB8rCwffsgtUTgdKC5Z27a"
    "hYuJdw08z9us1RMSb7kRzzEwM2wSxTAC334czBzTaFmexLM4jgW+C2lDFNnONjCyfcND+Isi"
    "Ydfedd0w8BZXN4cGCjDPzptXr23GE0msHSJgYWJpaSmXyyPtgLUxCv8wFIWNAbmvrm9kUinI"
    "nLzAYx0FQpJYva7Dug8XRbr7N01DabWSiZSmao7rSrJIU3R7PyjqOPaNGz9nMmkCxVDbR8tD"
    "I919NICAX9Lx+K5nBw6Mdz9Yti3w+4uB3oCdrW6sRC6AlvFtFac4gsAPHpqaGC73bBxD/ebb"
    "yxH4lxE40rfMAEUCyzJixbEPZnZD56sLXxqOi2LsLzeuCKmB99480/up2WheunwNxcBf/vT4"
    "8HdXb8ZEemW1nu/PBW5AUSHLJ96cOdeorv309wdu4xtGytmmChm41VJ+/9F733z9vYdCdmU+"
    "/PADltoO58Uffmg0WgwvhF4Al8D3LYqX/+mjd9F2SOxisWhZJvrXr744evw0xBky8rXrNxmO"
    "z6Tk2Xv3T557PZP81bLc8sJsvV7bA+8g1EiK3sOgN+Ai72vmeh5J0j28/0+m+nXNQEr4fkiR"
    "aDtEkiTl83lBlCYOjjUaDUGKnzp1gmXpl3fz04bnAds7Dz87Fy9eSvYVpie275nj+tlc/4Uv"
    "v6JYSDA4xwpr68sEzQ/mY3cXq/mEqHuYxFFAFMA89xe3RA5fXlxLZfs81/JdS9etmbffEzmq"
    "O1sQBFcvXazUGrbrHZ0+eufuPQzFeQ5zvaimaDOvnwW11/aU4/z0n5er9YbpeIcmDm9WV7wI"
    "8y2DovGWGZw7czKfy4KZZWir61sjI0P7+N1xATn7eCEKgILVVqON9zffek/XdUi7V69enZ6e"
    "vvzjd6XBMbVRp3jR0lvZXGl8bJ8lnxxLS8tAvPPz66lM+uDEKOB9c3MTuPiZm3a9sYOHaXo7"
    "2JoGvCl0zqxTnNBNHFHgr25UM+lUz+yxsUM0+L4PM8Ri8rOW69ratv30qZ5mtq/iAjOG2UeV"
    "mEZba4DftykJx3GYemRkJBaT3vjt26raKhRyYRjRJB6h+1+xJ0ccaJKFukBiBfHhEo/yqufa"
    "n336x/6RyVNHDna/CcOQxNELX3xRGhgIQrgQgQ08qlsZGavoaFZiNDsA3ivk+2xduXevybLR"
    "7O2F/sFBho5CL6xWq6fP/y4usT2/37196++3LNswQGTEacILI033Pvr9+zsPA09++u9/7Msk"
    "ai0rX8ydPXn8qWcJPHt+YXny4NjeRwbg7ut3xzZ8r/2B6G0Ukv6DBw+AXv706ediLBZPxJOZ"
    "/PTkPos9Y4RXLl+PSRw4DoD8xttvIZ26ye2dJEB+M/MmQVI91eX7gReE52dmVE3DMBzCAEoU"
    "KnjXMtIomZQFw/awyN+oVKF6TCREgsRPnklCPuR5rtVq5YtFyGqAuN5xBspDGdsBZ5A0ZZsG"
    "jpM4SXm27e30ghu88+5bpqkPeKEg8r3Hdw3X81OpxLN+fTSb4+x7J4AAux+2/U4QBOB9YmIi"
    "Fkv84eOP4XHPc1EUjx6WsFDdgLoOgxDFn6eLg8389jyOYSEIjjCgOhX2Trz7SLC0NA8X4vCh"
    "ie1AhZBtiCvXr+VLZST0HMezLcOwfYnxVZ+xNaVlOFDKl/oLge+YpkszyOLsUr5c1jTFaKqK"
    "rh0/OxN/WHyA31eXFlXDhJKg2N+/VWsKgqjUF9fWRNuxx8fHMqlEF6GLD+Z0y0JxMvC8a9d+"
    "KhT6q2tr2XxGN93x8dFEh6nCwK3Va9lMeu8zvwzeIQ6GYdy+fXt6avLPf/oSpTmgVJqiMDzy"
    "I8rQlFIhU6mbJOJBwI6cOjFcLu29wOzsLMdSt27e6x8aOnZ4chfefddPJFKJVHYn3iEvlcpD"
    "hq4hGCyO0SwDxI6HkIMjSZIJNmQJZH5hqZDPoliIRnjpQBkUvSTHActyOkVi4U68O+0lEqZp"
    "Qj5PJJKgqfsK+TBEbYclQDp3LNsIxfF4ItFue0ny4anJhqoPDJagfiZpHrDQNbMtm2GZV4L3"
    "Lr9PTU3Jsdib77wDNQ5OEPAZC30vxGjqBdpVvu+pqpFOp7PZ7NCB0d73O/GOIcRWdcPygqnx"
    "4R7eaYbR1Va6L4e2Gxs25GBNtzEklOREFIHKMMR0cnR02HUshmIJKqzOr2cKWUVR4QK2Go18"
    "f5lhqJ7fk+lk+9b4URiFHEOYZoRjuGnYkA8ClO4CExAKp27f6QhqM9CyRgSJBXxBEJD9gGm7"
    "XgS8q4a6L5Zfkt+hSgS8Hzo4ujy/cOPWXWBRy7JYltE0O51Ly3zs9JnXnsfvylbl8vU7B6fG"
    "Ll/8vqWZAJa3P/oQaZfR7UW6NiAogYLgBpjWNohc1wdiAfgvzj+IAPAYqRsqSdMCGa0ubw0W"
    "+qDe3dqsrKy4A/05RW2BK3lJUBVVhL+aBlEFmWrbYe84q8vLLCciOLZV2YCa2QNJDxEzLZql"
    "xIT3EO/2wsJyti8N1SzU7q5tIu21cU9xKEYAm67f4QjAlq+K3ymKAj0jSIn+wWho+ICUSFLP"
    "0S15cqRyxZMnCASny2fPdb/p7gZYi3kobwOfSaVSsWRGErfVju24DMvH43IynYE6HlQg1OIQ"
    "hsgz0yVRZIharZnOlkBfmYaRzQB2I0t3WQ7QGgHPMBwjCVwv9YDfh0fHMAyp1ZRMecBzoGR2"
    "QTLYjk/hIUU/wvv04UNRGBiWl4jHNVWBdA7fMxxrWW1R2MM7F6CvBO8QB0D3wsICQ+Oba+uA"
    "98GhwTfOn30JvyOdJLw0t7SI+opq2rbz1gfvIx0m2YF3W1GaboRlU7Gep+D3Wq1erVTcIKII"
    "RjdVAF+cQ1ebTikd9wmmUd+Cb4bK+epmQ+SItdUNXhQhB9W2FByN4slsr26Ccf/eHSC2kKSb"
    "tS2KYZpKS+BZVdE4gR6b7PXvoutXr2fzOdcwtupN39QcKHQp2nVMkpGKxf5tozCCPPFyrnjW"
    "eIR3kiSLxSLHSyzPf/jh26auzy0sJiTRME2e44m2DAPW2aceA1qcX1gAujwwPsaROFxwwCPd"
    "0TM78yqCkv39RZxiH9ORfjQ2PsayXDcMsCUgXMfUDkwmBJaCz3ynyaO2lEw6jmFReXAQI3BA"
    "cTyWpBgKuGInz0xMTu1CX0tp0BMS7IUgennVPfraMSjX21BwrKZmSoKgqWoykwaMOA+JBSgK"
    "0sCvyjPott+9zqhUKpLAqi1dNfQWUKfAzc3Ow0ajwBHlVGmwzPbto6WAHDzbwWjO1NVNSHm6"
    "7ljumTfOI4/nVdvUvvnbd2MHp/rS2z3YTt0UXbjwF5qF2gePQlRp1XGKy8XJuYo+kJI0DxMZ"
    "HCqxcim3uloTeGJ5YQ00UYiEvuNRNDZy6IQsPtKR3379Zx0048M+AWhinkVdH6m3tJnXzw10"
    "+gTADLd/vrHZbJq2NzkxXa2s+AjumzqkbdUIzp09Vej0CQLP2awpULLtffYX4pntPgHcI9Aw"
    "c3Nz5YHS4vyinEq6tpNIpeKyhLzICHzv1i83EUAhQ8ZlMYwA72g6k15bur+rTwBpEMQi8TCF"
    "dPsEUCIQONHFAdw/QBDoKYoV2i/VHJeiCEi/UE/B1QFgBX4AkwOnQ8yCMJJluYe2DmuFkLG6"
    "EItAP4I2fKLwAAhDtUiDXCMpHHtmD8O2rShCWXYfn75In6D5SCC2++NAlli7mFTqdQOqCYx8"
    "cb/7oIVZFjeREDhX0w3btOOpNp/uwvsXX/4lUyjPnD7a/aaL988++USUoVTBKZKpbq5HGNUX"
    "w++v6yOFVMMMkjI/WMoTWHR7bkMS8IW51UwmidNMYBu1Rv3df/xDUnqE9+/++pUZUJJAzz+Y"
    "Jwn06OnXD44M7tptFLpffP5tKi0cPv5GNvlMl0Ghdn9p/WinCtljvAzeNa39Egt05Njo8OZm"
    "HUGxIGy/PQHVTNAgFmgUIyzTPDAyTDy7t9Ubq0tLqm1LLAPADBCsNFBcX5p7nr5Yb99dNgcc"
    "O5bVxTsEBvhuvbKVSib2PR74HekkrcBz3RBhSKLz6uQpeIepYGYM2+tQzwPk5zTbjXeyMwqF"
    "AstxUbjpIxiFhvWGBkSjWg6JIRuVddADcJ2fR1uGUSSLUuR7luO6lhUWi8gTeP+3zy70lwZn"
    "zm63ojp4Rz779JN8vgD0EYWYpisg73JxYqXhFxKC2u6LycVCzmzVbtyo8hw6e3shm8vgFEGh"
    "aICgBw8fSyfEnt+vXf7Bw3iWjJaXV4GKJo6cHBsq7ton4P2zTz5PpISpo69nk898M+O71t17"
    "D478qnh/VDfBWFpa4lnKDwBarhn44CZd0/rLB9IJeXgUec6hNrdahkvaNgW6AaBEMl0w7dQz"
    "GMn9wwfv+2H0eF8Meefdd0E4A/oA/jiOkiTd1jMkK3EMZEjIoNXNWl82PTrKQmVVLh+AAHff"
    "ozquBz7e2ScYGh0H/QdTJTNZnqUZlntSkDhu+JvzZzmOJUl0D7kCeaRUKryqugnwPjw8LEgx"
    "ttkKMZpEPcjyHCSTMEBeZHheKIlsLCY7UOHYHu5anVpkN97/44crxXzuyPQ2iLp94G+//dvA"
    "0BASoZBFbdvUTTfBR3WbTIuUZnrxmFTIZUC+VKstqGnm7y1kQG/gWOi4GI6KsaTE0z2/L9yf"
    "xVgZj9z1jSqG+OWx6XJB3LXVKPQu/3hLktnxQ8cT/DOhGvruytrm2Njw3gd/GbyDfoDMfufO"
    "nalD42rL8BGdoUldBT3J4fWNm78gAoNJ8cTE+Mi+fgec/nT9vwbLJZZuAxCkW2k71I/h/dj0"
    "IYKkd/WBj584obQUpP2amOR5JttHhq5FBXgmLhlQamLh8spaPpdhGAJCODo53u7q0BRIrwBO"
    "HYJ+j3p+F2TZsl2omOKJOEgRnsaehne/PDggSiyO7tUGgJ8ESfhV8R491gdu98Vgu6Yd4ZBL"
    "SaidIxQPQj8Wd1le2Nfj234nqWNHD7PwPI7b4EvPwbbjsaMP7DnLqyuiHM/u0O80Rc3N3kvn"
    "CpAfLKjoXa+lWSzuuLikQQmvWbIkDA2VfdduC3w0Wnmwki1kLXC266EEJYhCL/W0STMIQFBt"
    "1RqyJAoss7SwFI+LjboGiMwUx7MJvovQwHcMA6ms342ns66pGabd1ps4BnB57dh0r0+gqE0m"
    "92r6BCAhbt68OTUxeOnHn1mJCv6bmit/jqPaztP7PjM9+6JdtiSvWAZsAhgMjxd4IS8JqSRV"
    "SVXen5Uf8kP+gDzKryAQCIE8woOAAwZjY2QsWftoRrNP73vn3OnRSMhYsl3ISa5dU3K7u9V9"
    "7nfP+c53zh3HSaUzUiI5dAUPOKR4st1RIN0P+9qD58fSqDMC9bQAJnfs7rEsD7QZ/HJ0xHGR"
    "LgYZ7PZWBWg9HsPdwCMJZAO114ln0yzPA6n54c7iWLlgWmYsJDKFjAtuXRI8igKGrqk6M5xX"
    "oLOaxjCAcx6Ifr3ZwnBM102cQGsJqPJQF1M1U6ZRIm5bpqYZwOJDDA+By1H0Xl0MnugIdeCZ"
    "mZlkOnvpRRbAz5K4YTtRFvOwI5mQREniaNL2gNTYdJ8+Qko0xHtIk5jnCJLE7haCMZYTWJoc"
    "GTlG4ISm6RRFGIYd842pnMzhQbOtpLK5UrlsmYachKkJW7W2nJFdzyNx0rB1wNpeu+dyOfBs"
    "kP2yDNPpdICVthp1MSH36wrEUBcbHS0FQUinOLjGcZ2MnGo1mxCKARn9XImJ8G7YwVHpwODf"
    "FxcXadz+7lbFsfUg5uUy2Vq1pmh2RuZ1yxN5GgjAhWefGyb39xuJZOL3H/1BEhkVIqPjvfDK"
    "L6KptndCtGMhCYrptO3kINZF3QaKorZabeCRFEH31C64LJmNrTVXJkt5D6fu3rkDIXpirATe"
    "Q+RJy7bqtTp4XrCR41qGYeF7+glWl1cgSvR6ytzcyVqtaZqbiTjtaWa7p57nOZeJUllvu1p1"
    "gqDbVWZnTgauu1apYp7tVW3F9BHPIZFs5zpgHBtC4KFYPvScMBhEIOzdf7lCUDwskKA/cByD"
    "ZdWAOc9kXNuG7BHJQ+h0DFCTTCQ4QTggpYZ1g/cTd7BjgPJ5cAD+L155pdPYaraaDLXbo7xZ"
    "2RAkWY4PwgYk7LMnzmqaJsUTOOqrsuFWwCldSyV5WeRopafAdMLNVaVnGA4GTMX0KIbEMNw1"
    "LZLnSsXC3n4CsBRNMyjzsl3w4jQPi2l/hzQsnUazw/NMPC5T9299MXQVMolsvzR4wHiQvElT"
    "uo5joX4CgPm3126IogjPOj8/f+36rTNzk5lsFgsDSFRdCPmO9f3tOwNjbW4ecNNCoZTKpClw"
    "yQQGC4VkeBYcB0n4NvKMfaMM/DvA/eqX1yamZ548M7eDBIiU/vWvv8FJIggxLCQUrYuTTE4i"
    "VpvmaCZuhhSHo67H49NjW7WWyBMbq5WkLFMc7egGTjH5QpHdY7uvPv9McwmJp5ZX1gSBOTP/"
    "9PHJ/bVJeM7vb96S0+LMyacy9+8XAigBIIqF3ME27depDnHL2I5GhL3/r2+VR6cAmgB5QDRE"
    "f0U1UqkkzB5FMzzHAJUG8FIP2pkeKt2eabscz8WlAZYhLahX1/fpBEqvF2B4Mj7wM5FOEGlE"
    "sR0dGPAObJ3mJYrA4edImTIM3TRtWHIuABk1TeKAbQhIyaS8t+4R65O06CqzX9XiWBoejAVP"
    "6mIsQ+4iNET0aXga3ALicVRT3YmrEOn8REI6+M0frn+GIqnTp0+rqgphAew+Wkp/db23tFoJ"
    "rZ4bEKFrirnRbmXtr3/z9/QDNBLAiv70vz6DMARzr/mdqM43PXsidk/e9NkXV6ePzQxbASNd"
    "7N133uXRbBEUydQbVYxk8wl8saodK6a7VihL3Ggpz7Pk0npTFIj15a0s0sVord1VdfXVX78h"
    "7Kmvfvhv70OeoDqhLLIjpfyN724Dnez1NI4lnrr0WjIRxdXgym/fzBbylm5kRiZdpbndaFMM"
    "7Tkmzibe+LPXBr0Ujllpd/L5n7OfYKCLAeEDoDUajXRK3tyoABemWE7kuUMXzv7fHfi12raP"
    "Gr7Dcqk0PP6A/WLD547ghgqytk1zEFGQKWHetxstGWz2wLoYuioW1hrtXDYNrNTzA/AtfhCL"
    "1L0h3oHjOjbK8aI1Cv7C9zz4xB4YyA942v5+sajOd/fuXYaeubO4BOta01SMln79+isPZXcY"
    "i7cXMJIfmxjZV9feh/d/+Md/mjzxxBuvXo6ORHi/8uZvU5lMCPkswTSa226AlVLkUs2cLqba"
    "upeVpcmxcq+59fGttbhILC2sZgtZWGJCIr6xsvrnf/ubzB4d+D8+/PdYGOsa6KpCLvXOF1dT"
    "stRs9uISdebCL8eLSN8GvL915XeZfE7t9kpTs2a7Vmu0IBrDRIQE95d/8ScDvNv6tzcWLl6Y"
    "PwRzD4F3bIB35M0pqlqtAt6rtXo6nQbAbtfq2VwOpl16gC1Vw9FtNRXTBYIO2IEICc/DC8Lm"
    "6v66R9jfy4XvJBoR3mHNIb13oAMjlmXqOsUKFIkH/eSyUt3O57IA5N0Mpf8TcPD+wV28A5Lg"
    "jdBVntvs9HLZTLTvIkRSGjFsfIxuBUPXVNcPIbzBbKE/GDZst+8bhyYOE8AfRQeO6nxbW1uS"
    "JFmmeePbr/rlGvzO0lq5mD03/8SD2x1Y162bixPT4/VqVVE1WL3Pv3T5Xrz/8+/emTo29/zF"
    "vXUP7IP33x+bmADLu45v2kDfrKyE1XUiF2dU08+mEuVS3jZ6i5Uux8UWv1/Jl3IYQZpqB6OE"
    "F1++tFcnuPnNtZCOU5hb2arhoRfg89Nj9+yACdyPPvxESrCnzj6TzeXv90a+a6+tb548cYgk"
    "+yh5E9EfYHSSwCDHKxZGeEHUdSVbGBFYolKtUcjDBqlMlr1nn+C+URqbLI1NrK6s59JyIUD9"
    "QBSxv66NU/zrv3oNJ/f3Rz5/6VJPVYCIhqhBUqYZxjX1FOqPlHTLIWP+VhX1RyaTAtDt8xfO"
    "QX6AmjLocbgPzBK9J1/N5AuO69G0hJMUz7EJkf1JXWzuxGw8LjBkeIAMAGsxlZKPRCfYHRhh"
    "ar1Ws0dDmsFwSrdGI8HAdSxNkJLnxPihdo/m3rbQNajOZ1qJVCq2rx/Y1K5+8d/licknTg5A"
    "1O+PpG5ev14YHQt8D1aJBSTD8iTaUwPONVRFt8FxjY2NoLYBC5J4b3VxHfBu2o5hKBAVn/qj"
    "l+N76trA0HzPr+mtuCSAW1lYuJ1OxZuNHqR9hfFTxYwYIbTXbduurSwtZwplR+8pmgHuBSMw"
    "N8CfuXB+qIu1qq3Cz8pnduMqDHhW17VjBC3LSUESSIZ76smH3s6q9tqrq1Wah6wJYRFIekSr"
    "f9QPHBJT09O8GN+Ld9u2cqWyrvRQf2QIMKVlliNDMLMvCCLOCByF3V1eGSkVwpgfC4mRiRGY"
    "rbQkzaaP+54DYXlv3QM+YckiXZWItbs9cNyQcyflOEni9M6Z8BsJEnVT0bkM+Bzb8yVJRFkf"
    "5HcYuVcXg5OOBO9Rna9UKnGcAFlfR9NCLRZoVtTv8FDDgqSGxDOplKGpGmT0plkql/fjHVnZ"
    "YfndjSlgQZblbNPIF0vgmwzDBAOpmgmuOZPNE7DWVUOOZ+dm0o5t8ixHUGG9upXKZWFWN+4u"
    "645z7umLLL3rZ+JxybIdSeRFnveChiiI7Vad4uKxwAO3N9TF0qmE7bosL+BIpenKcqpZr+eK"
    "eVivQDolgY/wrh5dfySM9fV1gec03eR43nVcnAxv3LzR7RjPv/DM4cXsnZEtFOHv4p1F4MgU"
    "joWcEMk5SPlyB4j3gDoELkRd3TCjI5EODHnj8uIiLBESp3pql2bYOBOu392aGi15Mbyyse54"
    "/sRosd3tCizJCrxnQybpABgxG5JKAw+o4eusraxRHNdbWTt+fEZVjdp2KyHRdq/b1YxkMmn1"
    "vSUgtL69Db+us7w2MzPHMTTwSDwMWo1tw4klEokoMgHevcA7EryLogiHpqamRCmeScuagUxP"
    "U2RPUeSkFIYPvZNbFAV4BsMkaMfsM8MYw9Acu1OHc6xMrshxTDIx2Apjov5IMZtJp7M5kiB1"
    "TcMG/ZF6YTKO+iNb3Wx2EtaHrmvFPIkTIWd4DEeD1QDlQDp5ABu7a/dz589f//oa8NTlpcWn"
    "n33uJ/flAkJnZ2eufvUNeMKVlZXLly/9pN0A72KA/6x4D3fjKvCZQn9P+MjY+PDgaOwRR3FP"
    "sjp4eiRSDvDu2tbq8iIfT6flgd1hLcD/16q1SqXieiFFMlF/ZErANtvOSCbpkUyjVgUGPj1R"
    "rm63IPpsrlWScpKVEkRg266fzpW52G7Mv71wW84V8f7WYfG+mlewcPuH0dHR/v5a5n7YCoNQ"
    "07R8LvOoxviJ8fi+h2FvXA1i+MjoBCfwu3HVdtyFOydwjGNRjgZpD8mndaA1tjmTTfE0CcRI"
    "6KPJfPvdwto6BL451AaG+WGMxjHIalr0e/QwbyLJ6csvsZoS/dO+dg0+FU2lWYHEwmFCZIfh"
    "uO3EFfQ1AYCJysZdkWUhJqWSiQAeYOc0x3c5w7bazYNf0P6p/fD7jZBOxvrq3uOz+964arvG"
    "8spqKpXIZ3bqqw5OefZ7n3zKIl9EAPw6WpcgmYJELDXNiZSouFicIWSRn1GV1bV18EWaqSV5"
    "wfaCRL8p1ZRzk0M/Q5Iff/2lapiW680fn1tYXcUwgmdiro+1VP3F+SfG+6IueIZbtxcaqmLY"
    "3qnJ49VGNcApyBgYllSs4Lmzp8r9Tjc/CLY73VL8ED0S+ZnD7G6jPUZoDHSCozb6fl0sDC3H"
    "pUiUrEUHHNudPTZH9esm6J+eR5Ok47q+59Is2vSDtgKTJIRRwjJC24WEP/ADkqJ838PB2Qch"
    "xTK7+WoQeKLIUhRcRfc/Y2gTO9zQp0gc1li05coCJtMvxew9DSwHt40aNAc80rHRvh/6kMRl"
    "eLcDhuEBTvxdXewxjL14N7XOBx98KqXSr14etNgjnYCn337rXSkZ9Udy2/VKgNFF+d7+yOC7"
    "uxXIrFfubGTysud7NMPVt6p/+jd/t1cXi3TgnuXBVeVC9vqNhWRC6LRVXiDOP/ursX5nWWga"
    "V968ki3kDFXLjx2zu/VqXxfzfTtGiX/1xusDXazX+mF568n5Mwe/YGiaLMcdfI7dbQFvi/2v"
    "4f2esU8HhjgGFCsMA8swov5IpKMFXqXayGYeoj8SXeR7jVYnn8tisd3v5xjqYv1bIYnO1DXb"
    "Dwd1GPSVELvO+gh14McwfuTf9d477308Oj19Yf50dCTqj3zn7bfLoyM+6mmP6aaq6nYhSVS6"
    "QUnmweHm0snRcsFU2zdvNgU+7DTUZqcjZ2Uk0hvqxRdeL2V3+yOvfvG5a7uag3Yby4n4++99"
    "UCykt7Za+Qw/MntxqAP//sOPEulUp96YOHFab1a/bLQhx8YIyKjJ1/74pcjwvmPeWVw+e+bk"
    "wS/4KHWPozb6PrwDCHvoq7jYIaPv4/0sABLoBLj4CLBol4WhhQSbkDgIkVg46I80UYMJ0W8t"
    "JAcb3cOowx0f2l1RejzPW7Yb892uoqbTaVQ7JAjLcSApHdY9INuiaQonqF634/pBQpKCMEDM"
    "ESOSyURkd9PQ/RATBf7gd/x/gHfXNK9+/vnYsbknTkxHR/p4x//zkz+MT06B9W3LRXvcTVcW"
    "go5Fp0VahZ8TUqmYDzwb3DS8oNJWm81OPBXvdjosTT97+eW9edPa8tKabsBkTIxNtNtdxwm6"
    "7RrJoH3Dp06x+Swi44DQzbUVSBJDnEwnkj8sLZXLo9uVjXy5oOn2qZMn0v3v3YK8abOC+r8P"
    "fsFH0Qkew9ini50+c5bh+B/rwP65c/O9XjfEcUhlGY7KZJEsRvB4To6Ljgd8ZW19o1zM0zTC"
    "diqXK46NADSPTU9omuGjfZTDu3kA2EKxCCk+sPxSuWzo+sTkFMAYeBRDkcN9wywnCImkaVhy"
    "On1RTrR76uyJOYLA4nEMlsRw3zAvCkeyv+kxjH26WKvd5QWrlB9srev3RzLLi0uZYinqjww9"
    "R1FNBrdiZMI0jZ5iyMn4salJ17GAOIILVjtqZb3Lx4XQcVmeA4bOsrv9wDzPbm1WpaTcqG93"
    "e+r8k+clcb+jAIS6jl6t20Bp2o06UCMfZ+Zm9vf9ov5ItXskuthjGB7qq9/Box9jaSIpZyx7"
    "sAaifcMxgqpubKBtwxhhuzYiJGTYbGyz+QzFsmDxhR8Wx0YKumHwPgnZTU4o+q5N8OD9neDH"
    "/cDdDqRduN+XHxiW0Q0t+iLCH1kB4rHpADF1bA/YvtrSOYm+F9eoPxLHjrjucWQDxcCd/fYe"
    "joqdtm3x3ADvOO7Ckuc5JjM+BtFP03SSJMAoMd9I5JM8FTZbvXyhOD5OQCRMyzJOhpbuqpoi"
    "JSTPdniGAd49rMmA3UfHRrfr7aQsB67dVY1EPH4vGOEhstmUooeZTBKmPZfPejHy3tMA76Zz"
    "OJb/j+Id0O44O7qY6zUaVTuMjRazwyMQ6HtdpdVseUEM5khROzj5P91dW1PbRhTW3ZYs+QKy"
    "8YUAhoRe0qSTtCTTZNLLTF771r+ZaR946EuHlhbah0xnOr0EnF6GUgMGy5IlS6u7enZlHHNJ"
    "YDItNN0HMPZKslZnz/ed3fMdhKJIbXd/n6uVQ1awNjdiim7O1A60nixibR+QGgBVgWUAGxvh"
    "sPAHRczqaavlemFPN6nED/1oClDhRDAJ86/d3h0MXAvVSjm2e6CJ+WI0fVwvGZE9mZGpPq9h"
    "WeFZfUaFWS9u3GEGAl0bXpWLm/PXJFkZvcMwuDLA/MK8rCgwW+EJsSzrgMEja+71kiIKpjXI"
    "5/FHlmlUyhMkXyxEnjuh1jM84zge+KbRbcMQ3Lj5ttbrlysqzA+YPQVFPjkocZw0m00v4sqT"
    "RauvlytTfnjK2MXEkF/Zcadphj70v5H/9Glrcqo+0smDY2RwtaDvgJTH8DKiTdtg+Wwlz/2x"
    "71xR807MSlzCc9zi1dmdXU3JscgOwgAYfWcCeEhXh7PlxGe4+s3ql47jRnQmw4G1xnfv32uc"
    "ED3DMD355WdNMyW10ZiQ2n+1lclK7d6dY90AQeyBkdYae0GDs6VJhi9o6DDCuBweSXHi7du3"
    "2OM6G+qDjz7k+Wf5kQBlcRS8I8ppxZjUewK3ac6J8Agd2wEczklZoEml0iTR2VCjcV+6+94x"
    "b9vrHnAZCQ4cJcEBEt68tZTK74GGTgN6MGxnr5MvFiLyBYbjHkb5gvKq4uo4j7SN/Uefff7m"
    "jZsP7gz3zdN8seXl5Ryu0DHUDVNsJs2PvFZX0/zI2elqhqd/3dJkmY182uwbfhI5lnlt8bVy"
    "tT5ef2ZtdQXI+LhOPsOHYczp1mBcJ//42zXNNB03eGPx+tbWb4IkO3pPLkrjOvnQQ+1u7x+t"
    "t5T8R9fFEN5oFKMo9N1hfiTOWKbi3Y4GMeR51sUQchXliCYrDIJjavlTI3uAFl4Qxu32/7Mu"
    "ZhsHy1+sVxszR3Xy1KePHqmVMkZYmtf0Az+kaiXw7+7cVFF3Qoha52YaZm9v7cm2IjN9zTZM"
    "S5CEgiIZmv7w40/UwrN14B++f6ybph/Eb12/sbmxyfFCFFgUmzPM/vsP7s9M421IsNB1gAHf"
    "B/a/MNvcaG3kS6rZ7ZSrqm56YO91sj0SevaPP20uvXtG0tx57B1wwnNRt9u9NHsPwPro43Uh"
    "ADaBT+K0SAdlxSzDMK4zYDM5CLIg6gIfvLO3j1d0IYzBBYPhF5NWHsb5kTw3rvfQdaNYLMDj"
    "JKvBWFIDBCkhWZkcXkAbroul5yHdeFzNDmcdpwvGDN6CGXZDLMD9WanR57H3vZ2tr1e/UlX1"
    "4uyd1MOmbduBqyZJ3Onsw73kpCEDgRDfQwPL99Ni/DQMVID6toMsPRHy07UKIhULphtVZ2D1"
    "LRt4J/BIN/Dziuwjj+aYSqU20srA+WESUEkETiXwEUStDJ9J2RRN+Pgh3ww6BzihDEK2OMKf"
    "4mEecq44Cqm0Wxy6htGfPLM2fQIu5LnYG+FCYkGr1YLbl+XcxY07RKFr6+sLVxevSNjtNufz"
    "YHtdAM/D1t1vw0/TOHIUToFHGioADwGnP6CIVKVYII57tN9J0pWhX/h8VVfkOad+mJ7K984o"
    "pyRlufTqL90c21pZWalWq2mJ3Isbd62nwVVpOgF7Td9BCP253X7xUfBs6vX66JBXt+3u7sLt"
    "jzLaL9LeuWKpxFIJfAPwKgFZM1DL5TgMeIGHCIhIDBNi0LieDw0BPikwBUBk4H/8EqbA5RPB"
    "FctyGYFFLrBsJsE545xuDERRBCp55gb0pbRiQSmOKaS4en0KQO/fvmrge+DNQ9cLSI2MMEpo"
    "BqMcgFiQ0FmaCSkAt4ghgJnN8uYAJUTUybAcia1pCLsgpIJngw+PcR9Sjx/+4igaenKyLPsE"
    "q9mX0jpfcPsb0i0aSxTJwvQAAAAASUVORK5CYII="
)


"""Icon between database info on merger page"""
MergeToRight = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAADv0lEQVRIx+2UTUgrVxiG35OZ"
    "ycw1arj+3VSiqUhwISW0AcWaUBA3uhFxYxddVnR1sZsgIZtmERCkF4RCQnd2UaSg4A9E7OKi"
    "dHEx4KYoRdtICA2dyV/zP2fOnC6qIVWvd1Po5r7wcpgz883DOR/fC7zX/y0CAOvr61EAXgA5"
    "AEkAbzjnPwUCgd//E0AkEjmbnp7+uFar0XK5XNE0rZjJZPKMsbhpmt8Eg0H1seJIJBIFkFhb"
    "W4u9DWABAMMwwBizGIYhK4rS5XK5hsbHxz/q7+//3DTNH8Lh8GePFRuG4TUMYykcDi89CaCU"
    "QhAEWK1WCIIASikajYY0MDDwocfj+YQQ8ioUCn0VDAaV1mJKKWZmZryU0qVQKPQoRACAyclJ"
    "78XFBW5ubv5UVTXHGGvYbDYRgARAcTqdz0ul0nClUpn2+/1Wn89X9Pl8f5mm+eXIyEi/2+3u"
    "v7i46Pf7/Tg9PU086AEABAIBi2maLwAMcc6nAUw5nU6n2+0epJRKsiyjVCoV0um0pqpqRdd1"
    "AwDm5+e9hUIBdrsdu7u7Cc55bGNjI/YAcF+rq6t2xthLRVEWxsbGhq1Wq03XdYiiCJvNBqvV"
    "CkIIOOdQVRWiKKKzs7MJ2dzcjD0JuNPKysoU5/yLvr6+T10ul6Otra3zbd9KkoT29vYmJBqN"
    "xsR3AWq1GiwWC+r1OrLZLFRVBaUUjLF/migIkCQJkiTBZrNBkiToug5CyNNXtLi4aBdF8aWi"
    "KAujo6PDhmHYqtUqZFmGLMvNH9TrdTQaDQCAw+HAyclJghAS29raigFA8wQLCwsWQsgLAEOE"
    "kGnG2JTD4XAODg4OlstlSVEUCIJQSKfTWi6XazZ5YmLCq+s6enp6cHx8nCCExLa3t5tNFgFg"
    "bm4uyhjzyrIsdHR0WLu7uzt7e3u7OOdttVoNsiw3Li8vk9ls9ooQ8iOA14SQG9M033DO0dXV"
    "haOjowQhJLazs/OvqRZvB8a7vLzsLZVK0DQN1WoVjUYDgiBA1/XC2dlZUtf1LYvF8u3e3l79"
    "rnh2dhaSJOHw8DBBCIkdHBw8iAzxLioqlQpSqRQYYxBFEYIg0Kurq3QymfyNEPJ1PB5//UhU"
    "YH9/P0EIicXj8UfzSATwjFKKYrFoAqDlcrmSyWSKqVQqXygUfr6+vv4un8+rAD4AYLZa07Rf"
    "TNNMnJ+ffw9AaXnH71YC4LnH43llt9tHOed5wzD+0DTt11wud5bNZltT1ATAABj3TG/dusdu"
    "bRIAz24tv2Pw+P0TtJjde+Yt63s9rb8BX+/h8JD+o8gAAAAASUVORK5CYII="
)


"""Icon for the Chats page in a database tab."""
PageChats = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAK3RFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjYgamFhbiAyMDEyIDIwOjA4OjAyICswMjAwNrNp7AAAAAd0SU1FB9wBGhQHFZJfkT8A"
    "AAAJcEhZcwAACxIAAAsSAdLdfvwAAAAEZ0FNQQAAsY8L/GEFAAAHIklEQVR42q1XW48cRxX+"
    "TnX33HZ34qydrAO2sTcsDiECJSgoSiLBA1EuWJGCUCLxEIlHeIUHHnjiAYU8RAr8AEBCAglL"
    "8YuRkEAKAqMgBwUS2U7WFpeNN/HYa3Z3dmd2pruriu9UT89tdy2ZpKSZ7qo6fS7fudQpwdg4"
    "fWnNn7t0HSurbWTdDmJxyKMIHpPDc8GHVQlzGTz1IePEMnqpVCtYumcWjz+wgFMnF2QXyfdP"
    "X/Bvtht4/HMVPHT0AGrcqUYeGUmU57QSbvCxeAkvQSEZYzhQtBxbqcHyP1t448I67jt6B14+"
    "tShDBb575l9++SbwzCMLmK310OkY9CSGxA7VzJG1TEqnUPFExmRwJqcglRSFNUzTBuUFNokw"
    "y+22NTj7p5t49KjFD58+JvKH5Zv+B7+7gQdPHsPhRo6tRh1R5JDYHH0kiMSNcwrsHXHOjSD2"
    "OSKntkfIJaFeBomzuxRQv1hiWXUxsr7F9a7F2++t4pVTR2D+SJ9X5w/DNirozNQgxkDINOdH"
    "LlKoDSyZF4xypJEJAo1PyDemMsK1ODhKAjhETFKugd+BikbBPYZ73dggrdcwR2RnZhfwOmWb"
    "d284NBu0NFHLyNh6pFTCUwGFVJwqYAsLXS3Ewq3ovGSkrSPODYWDgrthTelqeU4jiFylgtlZ"
    "wcp1i3gzjXGnI5S+EphKMNujTyYV64Zz4/Z+n6bba14OS3X8IKKFxuxkYKZ5CZBSB/RpkVL4"
    "TC0T9EO+FXM7CGuhWTF5N6vcUwQIvDMe7ZR+Jp2b+k75DDPHO9WCSBrVADmnsXcaIMIJFcn0"
    "O1smGNGSMI/5PN40uHfOY4nPhbqgWRnFmKURG1RgtQNc3vS4sgV82LGDqBgpEFGSC3ZoXDCD"
    "KDu2jFpNI8uIcQ5D4eWH9zQMnjhi8PDdwrowTIaJZNMQVaUOE+0vHhJs0ZDfXwXOXVNkZMIF"
    "oVI5dYUElGL9o8uQ5wO23gwLymOfEDy3aDBfmyxEuzN9cm2WGj13wuALB4FfLztc3ijLh9YK"
    "W6RycAVzzAXrCUiuSgif/GWCT80Inv+0BOH7Cd1vlLSLTeCbnzFomIJnRjRSyskyylSDabhR"
    "+HNGg/50Iyd8hhtfOwHMVW5D6j7jOJX4yidpHHm7LKOBPvxUnsJqIuKfsjxmWUTtmNtE4/55"
    "4MEF2VX//9/xxHHBwRp5B3QzdHOGI9+ZNlrmVBvCQp+U2j1wFxNTbg/2W407qsCxOSClkV1b"
    "hdhthl7KtEwQa27a4IYiFjT1nPvoQqeH8lTjkqwNwxqgMcADn+62dhiEGYtMPwXOX5WQGR+X"
    "CzZ2gIstFjrK6tHdfTuHKNNs6GlV9UPtctaCnP7564rgjav+Y3PBmUseK+s0SmsCS3/KVOyY"
    "BBX63uhJ5QcuyNIiE9pdj5+/6YPmH3Usr3mcfscRWRra59lAOX2eEynPHl+kof4x4glNziDJ"
    "cp7cTJG3WMlePefR2i4Y3Y47StoLLY8fv+5xbbOoAznPhg3boMtpve+zMlotxVoQCgW0ULhI"
    "I9CEivjaPwR//yDDay8m6LE+1JORANlDaLm23gV++ZbFmYsea9vRkCbyHUg2C0mIOo9udX2s"
    "Qajl2HGW0f8aE4Ed1xSZd1sOKxuFTf9ZLxidmBccmhkJ1JrS2vZ477rH+fcFf2MMXf5vXrQh"
    "Y40h2xdkIQMknAXhMNJSLNpiWfW/0XOisMYX/tF5Kfxbv0p5AhscSGIcarJdZQsmREpL+LVu"
    "iu2+fqzdEib4lMOiGto557SPLghiX/TYhQK5CpShCwyjROcq/MVfjPZu9BgbHaA8vEq66e/K"
    "+RABNVQVcUVT74t+wAUotRhl1o+KkCKAYv7Cz1xh1WAvILMH3X7zYZyINjYFAkF3ahBLaDvV"
    "XxXttjlLgn9iTc+xuWjQ7PE+TbfXvByOLYmaLbQmM4XrTbNOv/dSVKmKjXa42EOFh7V24+Pz"
    "/d6n6faalz8FhZmOGu8SWhOa1R2YhxerWFtbxYG5iLlJHFgkYtmh63oMjBkkbMWNsagPjumM"
    "QXgrukQ6iJhN2kFL0kXEy4uuKZ1jgNbYIc83LDZba/jS4p0wLzy2iH77Q2zzLnjk7iqZOPQY"
    "QMIPY15NchaOjNBpd+ulEg6S3M0gZXfTM8WaXl6cFhilMyO6zDUKuqjOFr2CerSNxbsirLW3"
    "4Lvv46lH7i1S+UdnL/pXTl/Bic8/irkDTfS22blos6om84pWRNDkDVHGnuWvuDOYXVVT44wJ"
    "iDleflrtG/j3hfN46RtL+PaTJ0c9809++7b/KZXo1w6iOX8ILkmogAlXr+kh47qITN4dZfdZ"
    "HgI962J97SZdt4nvPX8/vvPVz44up+X4zZ8v+bN/WcWV1ibhI6Auxqgv0ugvM4vXL+3hdVdK"
    "ChOu6SZ0u2PlWYq9iNeyxSMJnn1oCV//8tJQ7v8AarJQ9wfXBWsAAAAASUVORK5CYII="
)


"""Icon for the Contacts+ page in a database tab, and
Merge Contacts page in a merger tab."""
PageContacts = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAK3RFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjYgamFhbiAyMDEyIDIwOjA4OjAyICswMjAwNrNp7AAAAAd0SU1FB9wBGxc4JCGlYgUA"
    "AAAJcEhZcwAACxIAAAsSAdLdfvwAAAAEZ0FNQQAAsY8L/GEFAAAH6klEQVR42pVXXWxcRxX+"
    "zsy9u+v1T+ykjWPiJI1TN05apWkoaUhTAlWqlCoqKEIF8YDEWws8VFAkJPoAlagQD0UqLwgh"
    "HuGBgIDKSChCJSqNWrBACQpxrQgnad3GiY0bx7v27r13hu/M3fWu7RDMSFf3ztwz5+c7P3NG"
    "0DZOXZzxb168jqtT80iqFUTikFoLj5XDc8GHVQlzabz1Je3E0vooFAsYHujCkQf6cWJ3v6wh"
    "+dapC35svowj9xdwYFsvSvxTtB4JSZTnaiVcY7N4CR9BIWlj2FC0OW7VDSb+NY23LsxhZNsG"
    "/ODEkCwr8I3fTPqJWeCpQ/3oKi2hUjFYkggSORQTR9ayUjqFiicyJoEzKQWpJBvWsJo2KC/I"
    "Yosu/p7PDEbfmMXhbRle+vR2kT9OzPoX/3ADD+3eji3lFLfKHbDWIc5S1BDDimvnFNg74pwa"
    "QeRTWKe2W6QSUy+D2GVrFFC/ZMSy6CIktQzXqxnOvzOFV04Mwpyhz4sbtyArF1DpLEGMgZBp"
    "yk3OKtQGGZnnjFLUrQkCjY/JN6IywrUoOEoCOERM6lwD94GK2uAew3/VyKDeUUI3ke3s6sef"
    "KNuM33DoKdPSWC0j48yjTiU8FVBIxakCWW6hK4VYuBOdl4S0HYhSQ+Gg4GpYU7pSmtIIIlco"
    "oKtLcPV6huhmPUKfI5S+EJhKMNujRiaFzC3Pjbv992q6282bI6M6vhHRQmMWEzDTvARIqQNq"
    "tEgpfKKWCWoh3zx9Z7B3g2C4hxbR+jrD4tysxySDNUOLbvW+5nw5c7xTLYikUQ2Qchp55wOT"
    "1FGRRPdlzQQjWoJ7exye3m6xs1sovBVXj2wSnJ/z+N0Vh9m6rNnXPm8OS0kupKfGBTOIsqOM"
    "UatplDFinEPbJsF9vcCXhyP0FtcGdoEIP3yXoI8fP71IJZZkhcDBLoMnBg0GyvT1gsfpdz2u"
    "L5q8Ujl1hQSUglPoMqQpdSICCQtGkmiKAZ/ZYW4rvH3sols+NUAEG/uURz/3PLvH4hAL3j3d"
    "wGNbBM/ebxj9dB/RSuhuF1zBHHPBeh8W01R/SmC0t1cwtAHrGo9QwOaSNBQQPDZgcHdHC3wN"
    "g4Ey8PHNTFnKSRLKTH0op5HCnzIa9EGAJS+9e/roM1mfAhsKwPYuYHIur1Z9RYQ4WFGLON1Y"
    "dDSQsHs05DGoLfGvZwof4eHPekDDY52y2yIcYV/C/TzTglvbh8obn1HrE1RThiORRpZomVME"
    "+CNrMFAlSF1N/LqFK2W1nu+rc9/pSYexa2pMfiCx+uLN9zzOXBZUsyIkW+B6nUrHiDQ3s+CG"
    "PBaa0J257HHsHqAU/W8FpuY9zl3LDdFxnfMfjzmcHGHtYLqOE5Ffjzt8yCyI/TzT2YUY4IFP"
    "BbJsOQizLK9UzHice9/hLFPn8Z13dobS//Ydj/c/zPdZ43Bwq8VnKXwfs6BIA4YY0Dt6hP2G"
    "w1/eM4y9TthkjvRLVIDO0/xX6J3mJx8VqS76yZjHZuYxe4j/6vfRCY/XxjWobNj3+LDg64cF"
    "5bhFpzVjP1P1vk0WL79exNvXMlRMjL5aqgHp4RsuSFhjNUXq9fy5Muvw4mmPX5wDZqr0JVFS"
    "v+r7Ki1+9azHD99wuFnJ923v9vjqwZXCl7OADw9cPP9oETWeE3WePVqZ5ch33/bpphGYqJNC"
    "FZ4sT0cFlPGg8wJr8CeHDPZ9hIh0Cq7MCcamMvxtSoO3Rbd6X3PeHDEhq0SCbtaM+uK/sWVx"
    "UkuxFgTmvqYiq5Szmj96KBkUyejBQY8v7rc4vEMtazF75kGLt656/PzvwD+mRCv78j7jVvJp"
    "DusrkKQLEhN1Ht3q+hCEGknOaZGw4YCwrNcHtnp86aOCT+yKgw+zVVnZSTiP3Ss4wkw5cwn4"
    "2RgPpw/yU9B4CXxCgrY1hmxfkIQMkHAWhMNIS7Foi5WpH42eE3hij8H3nmLtbjsHVlfF5lSj"
    "/PgIMNIf4blfZZiYpt1qkMnrQ3uXnKEY2jnnJGRMqIQagKpkUICB+EC/wbeP2RXC7zSkkYo7"
    "+gQvHbfoLZlGQeMBlep361lOd+0jtRI7VSA/g0Mx6mcUf+fJiO/1CV9WogHHw9sEzx8l1CYX"
    "tPYxIdtUZN6zOO0xtO1MEfEO8LWjBvu3/r+nQGso0y88BJzcr1xjGmUbzWr+OLYkSiRUMLW5"
    "66OeDoNbS3WYYideGK3gm69ZNpSF0MemhUro7XSeFBZv+72aLp9XydyytS80+uF8aJvIOEeJ"
    "d4lazaOnuAjzsaEiZmam0NttUaB2BWUki8yeJVJ3ImYrbkyGDjYaRism+8Y70cVSgaUU7aAl"
    "rrI0J2FN6RwRKbFD3ljOcHN6BgeH+mA+/+gQavMfYIF3wcHNRTJxWKJjhBsjXk3SrMwct6G7"
    "9VIIB0nqWLRo4ZLJ1/Ty4kiXKZ1p0SWunNPZDrboBXTYBQzdbTEzfwu++i6ePLQrz6aXR//p"
    "Xzl1CTv3HUZ3bw+WFliptFlVkyVqObjthiht7+aT3xnMmnukRgQTEN2sxdPzN3D5wl/x/c8N"
    "47nju1tty6u/P+9/RCVqpU3o2XgXXMwgItzWre0LpF0XkZV3R3Fr6TXQkyrmZmbpupt44Zm9"
    "+MqxPa3LaXP88s8X/ejZKVyavkn4CKiLGnfjnLSprsaz1x5e/0qTwoRrugltXZOumaI0hNey"
    "ocEYTx8Yxsmjw8ty/wOBV2HGcjvWTAAAAABJRU5ErkJggg=="
)


"""Icon for the Info page in a database tab."""
PageInfo = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAK3RFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjYgamFhbiAyMDEyIDIwOjA4OjAyICswMjAwNrNp7AAAAAd0SU1FB90DBRIcBgK8oMUA"
    "AAAJcEhZcwAACxIAAAsSAdLdfvwAAAAEZ0FNQQAAsY8L/GEFAAAGdUlEQVR42o1XS4tcRRT+"
    "TtW93T09D/NQEyURHAnxhWJACTHgRtBIEBTRjWtBt7pQfCxciGSRhf4Fd4bgJooLweADQXER"
    "iNEQfESiJJkQMpOeme66VeV36t7uvt19oxbM9Dl1z6k69Z1HnRLUxrEzK/GbM5dw/sIq3HoP"
    "mQQU1iJickROxDQriZfqV3+kLixjotVuYc9tCzh4/w4c3rtDZkReP3Y6/rDaxcH7Wti3ews6"
    "/NK2EY4iuua0EaFSliiJSAZJbcHK0OFYGxic/fUivjt9FXfvvglHDi/LyIBXP/ktnr0CPLV/"
    "BxY6m+j1DDYlg2QBbRe4tEzuzk0lEhnjEEzBjXQnm+YwLZuMF/jcYoGfV73Bia+u4MBuj3cP"
    "3SHyxdkr8a3PL+OhvXdgZ7fAWncO1gbkvkAfOayE+kpp+UCcCyPIYgEb9OwWheS0yyAPfsYA"
    "9Ysnlu2QwfU9Lq17nPrlAo4e3gVzkj5vb9sJ322hN9+BGAPhogWVglWoDTwXLxcqMLAmbWhi"
    "znUzGiOcy5KjJIFDxGTAOVAPNNQm9xh+W88MBnMdLBLZ+YUd+JJ7Zz9fDljazpPmejIL6yMG"
    "FMwJq0KajMk8l6CMzwh7SHJHHtQl29Uf8MaPPskZntSji6xg/HCjPK6rx9LoFAU2aGzWamFh"
    "oY/zlzyya4MMWwOhjC0Ybi7p2BF9mUPLhxFvwiSNdKYaysE06ik/HGperCJaQgcbDsy0KAlS"
    "2oA+4VeJ6ChIaPsp30reV2E9oqfGRmjWU36UOTGoFYhqrKY42SwGDRAhQ0Oc6vlhgkGoPORF"
    "zAQ9PfquWa+eFZY7hXQOjQtmEPfOPKNW08gzYhKyI6WhYjP90sniBt9uxJcuSJUq6GEkoZTp"
    "P7oMRVFVtmimqkmNvxH9f/iUIVorfJnKyRUBWUinJyBFWVLr1UtkXM2G9EeHZguNjhc/i416"
    "EwZUpdLonnrgFANkCkaD/qV8iVKV3ZgWGvKi0MXmzXUUDo169SJuUICFkAag2o8GWOI/4Kxz"
    "trpgKp36rVLjnz0eE338GTthwMDHf9VLyNCowCLlMUcEaKV3jAGvCAAaxLHCbfg7zCDlm+j6"
    "cIS0Sa8+tLANbBsdXOe3AdMy1xgItCGmINRYkNrq5WbSSM8Y4NCoNxkErKduFYY1QGOAFz4N"
    "8H4UhN6XlUpSlaOPpMZLnKCnR39wAz2ML7NoWK6ZHUHmYd1Vft+kASwGmv8KYdD85N/oohaM"
    "ed4BdXomCAvbrFcHgoFneYENWh49k2Nrv2AWoPRdcgEFygJRBo+kU5e8mDBBT98FzsVGvTpa"
    "md4NJoU+Brx7YpmGmhZMGcJfsHaHVKzLYxgZ84ZFpE7PBmGz3gQERKWXdbHIqtuKa5TwWoq1"
    "IJQGuAHTxFano69MGPPGT9IzBvhmvVQRq2FjD+IWIDkREp9cn4JQIyaQc/SjxkTCjnOKzJBX"
    "1Ov0bBY069VLItsXuJQBklyULqMQS58VPIFTF5ix67RkDnltver0dKF3rlmvnjCezYu2cyFI"
    "mSGxKsVKJAM0E4yMoWSUDPmULTV6xoCiWa/ugkwPqoaEsuaWQaidjk7SEOdjdSWX6Gn3N+TT"
    "qUj/9e5sAOr4451W+r397Um9ej+vTatHiUBKVVqQSWo72VqjlYKWXVzyT6bpWeNFgwY5/mvI"
    "lF5Ws4BVIB1bmIrOlK7PluYM1jYHaLfn4e1G6uVy30p9bNEa866id785prNiKNcb8TKlF2v1"
    "QttExjk6fEv0+xFL7Q2Yh5fbWFm5gC2LlrlJHHQh2aDrNik9j5ytuGEJnXNlAXTsG/9NLpce"
    "LHeJbDolX4fl40XnVC7wMuqIw7aux7WLK3hkeSvMC48uo7/6N67zLbjr1jYXCdhkAInRq7LP"
    "4Oyye7Opu43SShdJEeYxYDHaNOWcPl4C5bzKmbGcC91Szs7xfdDCnL2O5VssVlbXENf/xJP7"
    "7ypD+b0TP8Wjx87hzgcOYHHLEjav87GhzaoemU+0MoImX4hNnZ9+DXoRzcRF0ATEIh8/F1cv"
    "4/fT3+P95/bg5Sf2ju/MDz49FT+kEf3Odixtuxkhz2mAqXJ+asG6LSKTb0eZrVIp0N06rq5c"
    "oeuu4bXn78Urj98zc1fh46/PxBPfXsC5i9cIHwENGWS807jRgF5Mrox5GUqY1FOa4SVU2Vjq"
    "8CB8bS3vyvH0vj149rE9o33/AZd4Hw2lBI2bAAAAAElFTkSuQmCC"
)


"""Icon for the Merge All page in a merger tab."""
PageMergeAll = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAK3RFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjYgamFhbiAyMDEyIDIwOjA4OjAyICswMjAwNrNp7AAAAAd0SU1FB9wBHhQSAqlbZbsA"
    "AAAJcEhZcwAACxIAAAsSAdLdfvwAAAAEZ0FNQQAAsY8L/GEFAAAIvklEQVR42n1XW2xcRxn+"
    "/jlnr77Ecew4F9ttXFynCVRJoLRKQtNKFWkhQmqpgPLIGxXlpQX6wFMjRZBKRYIH1Eg880BU"
    "IdREIlLUVqGQQkFJFMfN1SGpkzi249vuenfPOTN8/xyv13FMj3R2z8z8889/+f7LCJY9R0cm"
    "3ccjd3FjbA5RpYxQLOIggMP9j+OE87Pix7L4r3+ynFiaH9lcFoMbW7H3yz04MNQjD5C8eXTY"
    "fTpXxN7tWezq60CeK7nAISKJ8lwphF3cLE78hxdIljFcFLTxzNcNLl0bx+nhaWztW4PDBwZk"
    "SYDX/zzqLk0B33qqB635Ksplg6qEkNAiF1mylvtP56HiaBkTwZqYB+lJgZ/DSlovvCDJBGjl"
    "8lxicOzUFHb3JXjrhX6Rk5em3C//OoGdQ/3YUIwxXywgCCwySYwaMgjELufk2VvaOTaC0MUI"
    "rOoeIJYM5TLI2OQBAdQvCW2ZsyGiWoK7lQTnLo7hnQO9MB/R57nODUiKWZRb8hBjIGQac5MN"
    "1NQGCZmnjGLUA+MPNC5DviGFEc6F3lHijUOLSZ1z4D5Q0MC7x3CtEhrUC3m00bItrT34kGeb"
    "zyYs2ovUNKOakXHiUKcQjgKoScWqAEmqoc17LHwRnZOItAWEseHh4MEVP6d0+TimErRcNovW"
    "VsGNuwnC2XqItZamdFnPVLzaDjUyySZ2aWzs6t8r6VYbN56E4rhFRAuVWYjASHPiTUoZUKNG"
    "SuEi1UxQ8/GWjpNFWK/8Xkm32ngpcpxVKWhJoxIg5jB0VgEiHFCQSPcljQCjtWRpLGJW/V5J"
    "t9q48QQ8yXrZFReMIJ4dJkSthlFCxFiLZZsaG5lEiPjOPNBTMGjNCP8FbdkAtSSlyQUa5wHG"
    "F4BSJPx3uFclOO39AqgLfKayqoB4K4X6Q5chjhczmzNLewha7FgX4Nk+YGOLqEsxUwNulR2u"
    "Tgs+L6eZppeA2tIm2LtB0JEj8GiB22XBB58DZyadN3WaPjRXJGkoe1cwxqzXngaJVQjhP19q"
    "sT4n+OGgwStDgtmqWgE4fs3h0CcO5yaA/Q8Jbs2l7/P9grOc0zWlUVrd88qjfMlDeSnPqE6r"
    "8Jwo4pmqMAUzav6YIuqrCzFxsJHAfW2HoKvgcOSMw8iU84i9Pgt0ZoFta2nqOtAe6utofoft"
    "neLXrs86VHnI1RmHd7m3O+/w2k4hT+aVKKKCzr96no+G594+61z3ANa0F33CKISCV7nh8fWC"
    "mapDmZJvagXocizEqSlzPFgFmqik6bG7KB4HcxSqwLUW5hRmcNycUzc6bKZ7/jPu8Pt/Jygn"
    "dWKhgLl7JawvjRIDiUrDDeoTWmP/w/RnB/CPMYcz4xZTC4KffFVw6qbjXFqgfvqEwXsXHT6+"
    "mTp3T5/gxUcNDp+2mmewe7PgabrlD2etj/udLH7Pku83egO8dy2HPEqcrxNTGYQam4l3g2YD"
    "hyc3Ct6/5HDimsV8zaF/jfGg+nDU4gb9vaEVPq1eniTyq6lFLk9wL7FSJkDvlJgh6ZJ9/SFm"
    "KmoFh/PUvsxq+OQm8rk6j2o98RhgwacASbIEQs37ivSY4fXdIYMr93wyo3UELQyJzS1AXxvU"
    "aJirMIQ0Yrhrvqp5xGFDQQsUXUA3RiTazMgh/vBQB9eTlNdsjRO2DUE0zWioQp5+65+u1jWE"
    "fKHIPGDw0laDl7fDAysTiA/Rhwm6GcY465APTfXxtWmgvlj4FB9KM02Nw+B+GnXtmrwvATh6"
    "3uL9YTJimMyUFtC78F8V2MEtuiAhMv9yQfAVArCF8Xz4FEtyFODtFwSfjlmcuJzG+S+eAf54"
    "FrgymZbawS7gR7sMfv2RY55w+PZW+n2jwevHGJIZizf3GZTonuPnEyYv56Wss/a4NAz1h6mT"
    "jULMw2YXEhz5xHpk/2xvSARbXKAPH+s2uDqZ4OxtiwrjWYvyCEE6cifdX2EI69pV5oMvdRr6"
    "3XKvw8/3Goaz4MhpR7dR86RIl2eQdTVGA3Njoo6xqQCaKCLafIQl+g1Kf2sGOPhNgx76/ZFO"
    "xv96eN9eJza2dmuoOY+PoS7B6LTza9uIeKXtIVgP7jcYI3CV18hdTfUVCMNaSOdYujX1exBq"
    "rFiOojjwBUJj+7M7DgdPCPYNxvjBjtD78jcHAozeS328nVp9bVNaFfcwxKbp2ndfDLCFCakl"
    "m2Lg0MkEH1wRv9endgI2YhVMYvG1wBcjTcWiLVaimdBonUj7XQcfnseGLU5eipnvQ/R3xVjL"
    "gjTQGWBdq8UzA8b3h6eva1JyGGXGnKaZb7C/HCtFdItZ7BMbxSjn2zlrtY9ODwpd2mOnAhCI"
    "1mipML4oGaJExxFbm4ssPCMTafHRhsSaxBfYJl1jLEv7GnwaT6iKqiA2bepd2g9YX51UW/Wh"
    "bfSgCk40x8at/r2SbrVxs5nWZia1gC+8lIApQ9MPW2tkVXiOMt4/oYbnsrEoaFb5Xkm32rh5"
    "lwi82sL6EJnU9aadPrXVOgsMk06wwMkqsizW2o0vH/+/75V0q40brxqFkY487xKW+aI9twDz"
    "xEAOk5Nj6GgLGJu0A5vJUBboOib6uAUZtuKG/i1E6nsyYN/4RXQZKSNgNGkHLZkKs2fk55TO"
    "EpB5dsidxQSz45P4+sBamO/vGUBt7jZKvAv2rs+RiUWVABJuDHk1iZk4IppOu1sn7Jx9M9mC"
    "Orubqknn9PJiNcEonWnSRbaY0gUFtuhZFIISBroDTM7Nw1Vu4vmnHkmbr0PHLrh3jl7Blsd3"
    "o62jHdWS9nNMzaoyr2jNi17Tn7Lsv/GmdwbzwD1SccYARBsvP+NzE7g+/C/86uVB/Hj/ULNn"
    "/u3xc+53FKKWX4f2zi7YTIYCGH/1WuWm1ZRF5P674/Kr3DIBTFTB9OQUXTeLN763Da8+91jz"
    "ctp4/vS3EXfs72O4Mj5L89GgNly8G2Ox3W5EljYakZ9z0qAw/ppufLfbjMB0DxVh+hzozeA7"
    "uwbx0r7BpXP/B3BdFieyVMNyAAAAAElFTkSuQmCC"
)


"""Icon for the Merge Chats page in a merger tab."""
PageMergeChats = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAK3RFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjYgamFhbiAyMDEyIDIwOjA4OjAyICswMjAwNrNp7AAAAAd0SU1FB9wBGxcrI94utjQA"
    "AAAJcEhZcwAACxIAAAsSAdLdfvwAAAAEZ0FNQQAAsY8L/GEFAAAHcElEQVR42pVXW2xdRxVd"
    "e8459+VHnq4bsNPGbZrSAhKpCiGJQCKVaCurSBUCCQnxDarEB3wgxFc+UMVHEI8fPvkBIVIg"
    "iCAeSilpU5UCoqpIXJsocZJa1Ildy49r+55zZoa159yXr2+MMtbxnTmzz+y9136OoGucnVrw"
    "l6Zu4+bcCrL1OmJxyKMIHluH5wsf3kpYS/NXf6SbWDqTUrmEwwcGcfLDo5g8MirbSL519rL/"
    "x0oNJx8v4ej4blS4U448MpLomb1CuObH4iVMgkDSdWBT0NZYTQ1mrs3jjctLeHR8F743OSFt"
    "Ab7xm+t+ZhF49tgoBiubqNcNNiWGxA7lzPFo2cqdTMUTGZPBmZyMlFMU3qGXNggvsEmEQW6v"
    "WIPzry7i+LjF6WcOilyYWfTf+eMdfOzIQdxfy7FaqyKKHBKbo4EEkbjuk8LxjjjnRhD7HJFT"
    "3SPkklAug8TZbQKoXSyxLLsYWcPi9rrF29NzODM5BvNX2ry8937YWgn1gQrEGAgPzfmRixRq"
    "A8vDi4NypJEJDI1PeG5MYYTv4mAoCeAQMUn5DvwOFDQK5jHcW48N0moFQ0R2YHAUr5C3eeeO"
    "w3CNmiaqGQ+2HimF8BRAIRWnAthCQ1cJvrATnZeMtFXEuSFzkPF6eKd0lTynEkSuVMLgoODm"
    "bYt4OY2xxxFKXwqHSlDbo8FDSta118b1n7foDNf7qcT4QA2HqnRgCrlJTGbXa7ix7rHYICIU"
    "xzc9WqjMRgZGmpcAKWVAgxophc9UM0EjxFuxtk237p0rnWr3xAhw6kCE4VK3Gxoc5/+VFPjz"
    "ex5vEm1Lpp5oQUOc7hV7pw4iXPAg3fS2FWBES9prEdN/zr9TDxo89QGBke3hqmMXhXp+XLCL"
    "fM/dsARO/YIRRN6xpddqGFl6jHNoM+/osfN8jLY8cZ8K1ZNYehKN7h8fjfDGvMe7q6qABBRj"
    "/UcTIs+bmc2bnmzSte4zPzYiGEjwf4d+pnSfGDG4vkT3sWoKh9gF7QlIXgDayl76wUgNODgE"
    "PL5PUI4L52EY48r7HrPLwMIG8MpNanagOFx2EECPrdPEf7lFXvy1qnDwAXLM6Q36IMAi2FMB"
    "npkAPkW7qVOZnpNPHZTgWBdvAX+45nDhBjD5kDBpoatCdBiH5MXJhVmHa0supO+CHwWIiH/K"
    "9JhlRcJ4ZA/wwhMGY0PbD2v7Nl/uKgPPPQwcHTX4yVsOd+p0tEcofHU7/ftE6lczDi9TgNRl"
    "dPoqEeAhDIkYVhEAMkK7u+zxlY8UzIGdIW3tKe2XHhO8+LrD3+aAz1GIyYdN2PvdVYd3Fj2m"
    "WWfeW9OQjpghy6hgjaZOiUqiPsDYDGbw+OSDgkf37cS2/9BvjjEMz007/PhNHwR4+mcW24KS"
    "fJJshQi64AMs+BTA2rYTHhw2uHf2BRrjQ6wJqWru8Jmf2uDMgmLd5m+Y0hk9TgYQZUvc39Ss"
    "6kP8Z5TopSs+OMu9Dv3mpX+rYzH/W310HnWti8emFJWpP2USq5sEJdreqOP5pgmm5x3OTd27"
    "AL+dcpi+zeKUFk+WOjp1Z916XIO1g3warBspa48vwpBerfWAEiqEZy7mjP8IJ+kPqpm5i01a"
    "e6/Nenz/IuuGerEvsqNhqnborDsfGdTjGoaIUMmvksJqKtaEUAiQEaIGw/KFc8CJByJ8/YTg"
    "0F4gibYyV17Xlzx+cMnh0nUTCr+L1NYmZEjjtq5bI/J1SDYISYg6S7eaPjiheozjKqPdnCvq"
    "5cszHq/+h3E9mOLoB9lOUYovP8k8sV9w+k/Ar6+kTOEmaKkItr7Ts3rXrcH2BVmIAAm1IBQj"
    "TcWiLZZVuxmEM1F0txqec7Tlu8xeETd+/q8UV79dxi/+6bfQGb/zujUsyqGdc06KCPHNVKyT"
    "IECuH0oHSnpJax2ihfMHTtsC4rvQ9Vu3EVBFVRBXNPWFE2p3g0LbzPpmSS7Q0+6vtQ5a9Zn3"
    "0vVbt/OAaDNTIBAKLyWIJbSdbK1RCk7LfjjYJ9bw7FqLOk2feS9dv3U7CNh5qdrC4M9MYXoz"
    "XCW8mynLLXv3aIMvN1FisdZuvHt9t3kvXb9161FQWPNQ4V1Cc8JweQPmyYkyFhbmsHsoYmwS"
    "ByaJWDZouk06xgAStuKGKbTKGm40Y7Jv3IkukToiRpN20JKsI+LlRd8pnbZiFXbIe2sWy/ML"
    "+PjEHpgvnphAY+W/WONdcOy+Mg9x2KQDidFS2aBz1ti9RaH79VIKhSR3A0i16zXFO728ONJZ"
    "pTMduszVCrqoyha9hGq0homRCAsrq/Drt/D0sYeKNPXd81f8mbNXceijxzG0exibayws2qyq"
    "yhI3PWjrDbG3S5TmrtOGtSdrqp8xADHEy8/8yh3MXv47Xvz8YXz1s0eknSd/+Pu3/Y8oRKOy"
    "D8N798MlCQUw4erVO6RbFpGtd8fuq1yXACZbx9LCIk23jG9+4TF87akPdS6nrfHL16b8+dfn"
    "cHV+mfARUBc378YFaUtcBhIBycK7cC8ogjP0lMZ3enNfyBf2Il7LJsYSPHf0MJ7/9OE23/8B"
    "F8pelXv5qYcAAAAASUVORK5CYII="
)


"""Icon for the SQL Window page in a database tab."""
PageSQL = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAK3RFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjYgamFhbiAyMDEyIDIwOjA4OjAyICswMjAwNrNp7AAAAAd0SU1FB9wBGhIZOulC7wsA"
    "AAAJcEhZcwAACxIAAAsSAdLdfvwAAAAEZ0FNQQAAsY8L/GEFAAAGuElEQVR42o1XS48dxRX+"
    "TlV33zt3HrENxESykTLEMY8IEUSQeUjZRMpDJosowBK2zo7HAgErTACxsBRYwE9A4AVScKQs"
    "QER5yBJZJJYcJ5aVRASDbI9lecZ3Zm53V1W+U9V9b9+ZO9a0Zm7XeXTVqfP46pSg85w8txL+"
    "fO4yvri4imp9iEw8amsRMP0EMkLkSqSleetLusoyGRS9Aoe+tYDHvrcfRw/vl20qL548G/66"
    "OsBj9xZ44OAe9Cnp2YCKKjrnViN887EEiYNokHQmbAxtn7XS4Py/L+H02Wu46+A38NbRZRkb"
    "8PxH/wnnrwI/O7IfC/1NDIcGm5JBMo9e5Tm1TK/ORSXQM6aCNzUX0pVs5GGrbjRe4HKLBYpX"
    "ncGpP17FIwcdXv3pHSKfnL8aXvn9FXz/8B24fVBjbTAHaz1yV2OEHFZ8d6Y4vaefayPIQg3r"
    "de8WteS0yyD3bpsBGhdHX/Z8hmrkcHnd4cy/LuLE0QMwf2DMe/tuhxsUGM73IcZAOGnNj7xV"
    "Vxs4Tp4mqlFaExc0Iee8GY0R8rIYKInOocekJA/8DjTUxvAYytYzg3Kuj0V6dn5hPz7j2uaf"
    "VzyWBtxprjvjxC6gpBGBBqhLxasBLu3Q92Mu3EwvSEXdOWS14eLgwuuRp3r9uuYm6LmiwMKC"
    "4IvLDtn1MsNeT1eGIk4qcdsBb9y/0PGhvck43xX93N8dw2BSYmoouZmNCqy0INGltAEj7kg1"
    "QmXiR89+7qNyaLN7xjhwRybV5Y70bx5kEnJOHzgfUyTQW9ASJ5kFrwkiJGhIpeu7JpMZrw4t"
    "YmaOEy03pdUT1cjz18XyDVpBTErPtTPHrNUycswY79H5CKiqaXr2eHd0XUsMQUQqrxuQ6O1M"
    "f5xXhQbZghnPEQ3o0DuOd0HrXEEUK1wq5RgKj8zH3dMhdYLULnqp1S0tgpnj3dDJgKZQ+ac5"
    "4nTDMQdI1MwG/W+yZgy7VRXGtMjsccS5BAA70rqqzsXChNPSDGjWowGW/i/JrSrbHDDNN/zo"
    "/cdNF852Md6ZrrTEaZQnSDnM0QPku4o54NQDqoAG09u3weMf1nECpaU5On77hJ3ity5vZUc/"
    "qMd0K//4yYwVlQCrtD30cYOykmWZaw542hBiEmouiEx2UJYTOhkh2/hbZbqZLj3Jp6iIvFqF"
    "IQZoDvDApwHOjZPQuYRUguT6qu7QjHubWFN8zSQxY1lZYkyP5Q0/GEI6q8PLPGx1jfJNGkAw"
    "0Pqvan2biFKt7Z8+bTHrmeabadkz2RbtJK81+5h4lgdYWTgMTY69o5pVoKnXhoAKCSBSDB9+"
    "r46oFbHbuDg+fWyar5Aq0TtJduRdN6Zb+eljNoaN8Sbcx9RHybMnpDLUsmDC0MJa8TqCdYLP"
    "mpXR0kaaMePW5UdHixnLtN7HdOiiqkQEHGYDLBJ1i7AGPZ4IxQoIyYCqZJlYP3ZblzauHU/z"
    "FfGM78ioN6GTvOXbMIRUC5CcHuLRraGPSagZ40lVtY0HRNsB/u15/S068Sx24E/oM8/KFN0+"
    "CkRsX1DFCpAY6ngY+VgyzAECUqUhMJMG9Luv0TBTxn7Gsp3S8YWXelN8rW0NYSv7zq/9mG7l"
    "F1428Sxw6MV2zntJFRIaKNZBNEArwcgkBB06VouRbfwYAmbTRNalOyEgP9ONcux8wtyUhHS9"
    "qioYKVz6Tg/apU2YjKf0NInRkdXTNMKEr02rQ/JAPHhpQSax7WRZMWaatOyHUyny+fK4xax2"
    "bDY/0V8dn6bbR48mbUl028JSrEwKfbY0Z7C2WaLXm4ezG7EnzF2BA4xbXQwjndUFqmJj5jie"
    "cR29WTSaYtR2k3mOPu8So1HAUm8D5gfLPaysXMSeRcvapB/0Q9lg6DapPY+crbghCM0xiYwi"
    "JvvGm+nlMoTlKtpBS74Oy8uL8lTPMyH77JD3DRyuX1rBQ8t7YZ56dBmj1a9xg3fBA9/scRKP"
    "TSaQGD0qR0zOAbs3ixFb7SBFPEhqP4+SwLRpEk8vL556TvXMRK/yg6Rn59iiF5izN7B8m8XK"
    "6hrC+v/wkyN3Jph6/dQ/womTF/Dt+x7B4p4lbN7gZUObVd2yNNgeJvjQurR9t//pzmC23SM1"
    "z1iAWOTl59LqFfz37Od485eHcOzHhydn5tu/OxPeoRGj/i1Y2ncrfJ7TABOvXlsf6doiMn13"
    "7F7lOgaYah3XVq4ydNfxwpP34Fc/untyOW2fD/90Lpz6y0VcuHSd7nMRfCYNmqA1V/M5aA+v"
    "Umk1TOwpzfgwa84iSTLLa9nygRw/f+AQfvHDQ+N1/w/jDD9W5IqYSwAAAABJRU5ErkJggg=="
)


"""Icon for the Search page in a database tab."""
PageSearch = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAB3RJTUUH3AEaEgYCDBpZCwAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAAf0SURBVHjafVdLbFxXGf7O"
    "uXeeHjt2HMcuiV3i1k3SlFSNUkjToCoSUmmJWLRReQkJEBKPLUiwQKKwAMSiC7pCILGoukBk"
    "gRBBdEEp0FaljSIUSJO6CWkdJonjsR2PZ8Yzc+85h+8/d96e9kp35jz+85//+f3/Veh5zlwq"
    "udcu3cZSsYyoVkWoLOIggEP/47jg/Kryc9X6lz/VS6y6g3QmjYW7CjjxwDRO7Z9W20i+f+ai"
    "O1fO48ShNI7MjiPLnUzgEJFEeA4KYVuHlVN+4AVSPQxbgrafzabG4n+X8cbFdRyY3YGfn5pX"
    "HQG+8/trbnEVePLYNArZOqpVjboKoUKLTGTJWvXfzkuVo2V0BKtjXiQ3BX4Ng7ReeAWTClDg"
    "dtlonP3HKo7PGvz4iTml/rK46n7w0goe2j+HmXyMzXwOQWCRMjEaSCFQtpeTZ29p51grhC5G"
    "YEX3ALFKUS6NlDXbBBC/GNoyY0NEDYPbNYML7xTx3Km90H+jzzM7Z2DyaVRHslBaQ5FpzEM2"
    "EFNrGDJPGMVoBtpfqF2KfEMKo7gWekcpbxxaTDW5Bp4DBQ28ezT3aqFGM5fFKC07UpjGK7xb"
    "X16xGMtT05RoRsbGoUkhHAUQkyorAphEQ5v1sfBhdE5FpM0hjDUvBy+u+TWhy8YxlaDl0mkU"
    "CgpLtw3CjWaICUtTurRnqrzaDg0ySRvbmWs7fDxIN2zefgzFca2IVlRmKwIzzSlvUsqABjUS"
    "CheJZgoNn2+O8aCxJw/sKwCT2QDFGnC17LDcoNbo0g2ea887meOsSEFLapEAMaehs84ziS0F"
    "ieScaScYAgba4QmHT85ofHRUIRt046rcVFjccHj5psP7lf5zipf2zttPwJusT0+JC2YQ7w4N"
    "o1bSyDBirEXnkCaTR3crPD2viQfAegN4/ZZFsQocmFA4xPfolMLd9OULiw6X76iBC9U2AcQF"
    "HqmsuEJ5K4XyQ5chjlvI5rT/O+ovJ4Jx+sr/HP685LBS9RCAv3M+V3A4fa/CwZ0KX1xQeP5f"
    "GjdqvcmvsR0+BCtMksreFcwx67WnQWIRQvGfJqQ7HtuTmPzVIvDiJeBGmekm+5FCneZ/Z1Xh"
    "V/9mLNwB8QM4RjfJnrxtPu15520KD3DMO0VhKq7F/DGjQV7ZiBkHs2R4zw5grQ784V2HzS10"
    "9qKoOy5uAH+86nwwPTQFjOhkb5C2/VouRLHzr9wn2RAGtH/TiPRBq8AAC/RvLgReKzpcu+OS"
    "1GlXmYHx+VsONysaMyPKW2Kl1lMAemlbdcMSpAxytAAdYSLGgBFpKKn4pFU9pnJJEL23TrCJ"
    "nV9vZ9PgeMMmlpodA3ZkqEzUv9+PyAERMoMsKtxrMi1TEgOWMjgfhM2m82Z7dzU5cGgXmRh4"
    "Adt7g+NRWmqamguovL++fb/3FdOn6mXoRtPHAAs+QmtMJwiNSZDqPANv7QHgwWmNe8ctLtxK"
    "Cr3zxUj3jR9ZoABMxf8sO1wtKW8BFpQuLbrFzGlCOrPDqhEE0Tr364Kqzue/DwwjERzg2prG"
    "+RsOLBH4yoMau3O6sxfH3fHh3RpPHdSe+cfYYzx7kutmkLb7GmYBCP1NmrWqU0jTLMHcya8/"
    "G+cm6beUj1zjDwPLVYeje4D7djHXGeGVusXtiqQgsCtv8Zn7FL75MANvFDhHXCgRA47cpfDV"
    "I4rCMDveFl58BWlNMlZU0rCEG1a+GvmMN9egTvzony6ePAAdjtB3NI8H6wTBPjGr8L2TwN3j"
    "ygfpEqFXUm8fwWfvjsSsb153+OlfgesbyblfntY4+pEkCs/Rit840w3EFIWphgqjbLeaW2uY"
    "2boG9cgP33Bu8n6ooIDaFtMkoGjiO0asJiDtm4pw6mCA43MasxMJuMW8a3HV4uUrwEuLCmub"
    "28/9+guuT5Cv/RbIugo7rQJG8w71qITpylIShBIxloEQ0W8SEz6Buaap0eVly5fmIiZPjTVZ"
    "I+jPOESxEpFJ60LJ74FzX34xEeiFL4VeCAEiti+IWAUFA1zrTGh9XksAChJqqRMd/NAiWGu+"
    "2pBa4DxYSRv2QXSD88//pusCg4xv56xVSYYIErqkx04EiOWgaplSk5HtzH22DBkP0g2bt59Q"
    "FEUSmN5eST9gvV8FjCLjWiU5saZ0f+2512rIeJBu2LxbDaWBSSzgCy8lYExK28nWGmkf/OyH"
    "vX9CMXbPXCkzdDxIN2ze6YjYkojaSlNZnbhejxFkLJM7Q1FMsMXFOtIs1tKN984/aDxIN2ze"
    "fsUorHnI8lvCMqbGMlvQD89nUCoVMT4aIE3TptlMhmqLrmOFiUeQYiuuCaE5gpQWxGTf+GF0"
    "KVVFwGySDlqlamzrIr8mdJYZk2WHvDNvsLFcwsfnJ6A/9+g8GuWbqPBbcO/uDJlY1BlASkup"
    "bDA48+zeAt/dOsXO2TeTBC12N3WdrMnHiyWdETrdpYtsPqELcmzR08gFFcxPBSiVN+Fq1/Hp"
    "Y/ckTdNPzr7tnjtzBfsOH8fo+BjqhNymQKiorMJ2be2LqGGdX/LNoLd9R0qcMQEJQGksl1fw"
    "3sW38LPTC/jW4/u7PfMv/nTBPU8hGtlJjO3cBZtKUQDtc37wUb2yKNX/7dj7KdcjgI5qWC+t"
    "0nUb+O4z9+PbnzrY/ThtP7979ZI7+3oRV5Y3aD4a1Iatb+OEtNNokKGTHl52VZtC+8907bvd"
    "bgYmZ6gIP8vm96bw2SMLeOqxhc69/wd0rhV4d9yO5gAAAABJRU5ErkJggg=="
)


"""Icon for the Data tables page in a database tab."""
PageTables = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAK3RFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjYgamFhbiAyMDEyIDIwOjA4OjAyICswMjAwNrNp7AAAAAd0SU1FB9wBGhQHKMo33S4A"
    "AAAJcEhZcwAACxIAAAsSAdLdfvwAAAAEZ0FNQQAAsY8L/GEFAAAH4ElEQVR42pVXW29dRxX+"
    "1uy9z82XxEmI09QJqotJgQJqEG0aKoqqSIEoKgiqln+A4AkFHhBqeUAVqgBFopV4Kg88VELC"
    "SAXJkYAgtRQKJJUS3IYQt7Spg3Nx7Bof+xyfc/aeGb41+1wdt9BRJmfPeM2ab93XCPrG9MUl"
    "/+eLi5hfqCKt1xCLQxZF8Bgcnhs+7EpYS/tXf6SfWHofhWIBU7cN44G7x3H8wLjcQvKd6Qv+"
    "lWoFD3ysgIP7tqPEvxQjj5QkynMzCNc+LF7CRwAkfQzbQDtjrWUw9+YN/PXCCu7atw0/PD4p"
    "XQDfev4tP7cMHDs0juFSA7WaQUNiSOxQTB1Zy+DtvFQ8NWNSOJPxIr0pCnvYTBvAC2wSYZh/"
    "rlqDmZeWcXifxfe/sF/kD3PL/vHf3sQ9B/ZjTyXDWqWMKHJIbIYmEkTi+jkF9o56zowg9hki"
    "p7JHyCQhLoPE2VsAqF0sdVl0MdKmxWLdYvbSAk4en4B5kTYv7tgDWymgNlSCGAMh04yHXKSq"
    "NrBknjPK0IpMuND4hHxjghHuxcFQEpRDjUmLe+A5EGgUzGP4t3ps0CqXMELNDg2P4wXebf55"
    "02G0QkkTlYyMrUeLIDwBqErFKQCbS+hKwRfei85LStoy4szwcvDiethTulKWUQhqrlDA8LBg"
    "ftEiXm3FGHNUpS8EphLE9miSScG67tq4rb8302217gxLOL7t0UJhNlIw0rwElRIDmpRIKXyq"
    "kgmaId7ytW279ebvzXRbrbuR452ioCaNIkDGZeydOohwQSCpnrOdAKO2pLsWOljRWHxw2GDv"
    "kMHtFWqizbtFY1+pOVytAdc3KLWTAT6dEfEmF7CrXzCCeHds6bUaRpZMnEPfoc5BJhHe9Mmd"
    "gocmYgKgDc1ACumyZ8Ti7TXg9ILH+SXy84MA1AQhUzk1hQQtxfofTYYsa2c2bwZ4x5T8Sx/W"
    "y6Ur8buNhEc/tA3YT5A/yyKcWRxMX140V9g8lIMpHGIXpKdCsjyl9mcvNd99e4EHbxfI/7i8"
    "fxTof49MCS6vAgvrgwlJ/xm9UwUOPsBFRm/QiaAWaaddHy6d2maCZO937KLz7+G8vNKTiIEJ"
    "q6Hp0b6PACLqv8XdNI3aBSYPk05VeXHe497bBCOF9wfg0jvAqzRBansANOIck5RFmRqgdDbV"
    "NKcaAAk5Mx9mi+haaf59/rrDz2cdVpv//+XXax7PnnNYXM/59GaEui1C7DpN3aKTJuoDjhh8"
    "cEL1BekztppH1zNzDrMEcmwqwmE647bSrTGgvnNt3eNvVz1Ov+lwpbqF05AoSat0Zhd8gAWf"
    "AKztOqG1eaaSkLlpI0bAifsNKkPAk6cdnl5yeK5MAGW06dr2Eg05YLmmZdeF9eNHDF56nSa8"
    "3CtOnnkkY5Q5GUKUrvB8gwB4UuNf1e00Pjm7hZrzd68LnjwCfPchg1+96rDe4EXrwl9mvNAU"
    "sGQT6HDJY5Rz73bBVz5usDsWnFugs9k+D6ZpIxawVsGiZhKMNTNGgbpexwQkyBNE7oTCeXZe"
    "8NO/AF+7T3DkWIRFSvkv2rhFrWn207FvyIfQu5MZcje1tVD1+N5pj+urGGiRYq0NJrg+Wqw9"
    "Pg9DDQuGHpFmzN0uJOtcfCP5enpWpWFCutvjc5MGB3fkeeF+RF3maoKFVY9TF4FTlwjynR6f"
    "HpFBLa5ghFm34NfI2Woq1oSQA0hbtGXkQsRqRjTcf+KowdiY4MQvBE/dTPHsGYOdRfYB7Ia6"
    "dLwoY8ldpGmqjSicO/nVCDPnHX4/11cLfA2SDkMSap2lW00fnFA9ynGVMn2qTwQbcE8Z/+Y1"
    "i598McGPH/X0gSjYfr3hORP8ezkNUk7sEvpAjDvGPX9d8IEx8jjzNnmmPQCEjTREgARTh2Lk"
    "fG7rzCqxUS1185CmzJcZUj96weLEZ6MtfMAEsJt94MoK8M1fZ7j6n8Eu2aIY2jnnJI80307F"
    "+hEAaCSEitNRrQvr584Cr7zl8NinPI7eFeGesdwHDn1g0AfmmXanzwme/7vDpeW8UQ3FraMB"
    "FVSBuDzn5k6o3Y1u+jxtuk4PqhpAb33hmscTpyye+SPzfEVyZw1OSKC6YpW7tmawWo/yc9Lj"
    "080Dos1MroFQeIkgltB2srVGITgt++Fgn1jDs28tdBrL1HmVGW5xLfhvly7veuMtz8V9CFzQ"
    "CAVmKKYmN70ZLRN/o4Uiodhog5sNFFistRvvX7/b92a6rdadqcpkOUCJbwnXZOIqbsB8erKI"
    "paUFbB+JGJvUA5vJWDZoOqa8bAgJW3HDFFqmwxvNmOwb34sukRoiRpN20JLUETFcdU/pHFux"
    "EsN1R8Vi9cYS7p0cg3nsM5NoVq9hnW/Bid1FMnFo0PGEB2M+TTJbYfcWhe7WSyEUkswNocXu"
    "pmHyPX28ONJZpTM9utRVcrqozBa9gHK0jkk67lJ1Db5+BZ8/dGeepn4w8w9/cvoN3PGJwxjZ"
    "PooGE0pLm1UVmU+0brnrs6f0/XZm/mYwt7wj1c8YgBjh4+dG9SYuXziLpx6ZwtePHujV3qdP"
    "zfpnCKJZ2onRHbvgkoQATHh6bR7Sj0Vk8O3Y/5TrA2DSOlaWlmm6VXz70Y/iG0c+0nucdsYv"
    "/3TRz7y8gDdurFJ9VKiLIb2bun0hA4kKScOelw6FCT2l6RSzNsb8DAVhKz05keDhg1P48oNT"
    "3Xv/CzAtpFE0X7e2AAAAAElFTkSuQmCC"
)


"""Toolbar icon for commit button in database table grids."""
ToolbarCommit = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAB3RJTUUH3AEaFDUPgk08NAAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAABHSURBVHjaY2CgKajn+w/G"
    "eAATpXaMGoDNAFwhj0OcEatCfKDxEyN+A/AZgqYZtwHYDMGimTAgIiVSDBDOItUmqJcoTgcU"
    "AwAXqRhzWeRKpQAAAABJRU5ErkJggg=="
)


"""Toolbar icon for contacts button on search page."""
ToolbarContact = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAALHRFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjEgbeRydHMgMjAxMyAyMjo0NjoyMSArMDIwMGnY+V8AAAAHdElNRQfdAxUVAARfk92u"
    "AAAACXBIWXMAAAsSAAALEgHS3X78AAAABGdBTUEAALGPC/xhBQAAATFJREFUeNqlkrGKg0AQ"
    "hjeXE1QIFlaW2oiVjYUWGsFK8Bks7H0GzRNY2ecNAja2e21II9gJipJXSKGiOdci5LzsGbi/"
    "2eXf2W9nZgeAf2qDO0jTdF9VlYn2PM8D27YPbwGyLNsnSRI2TWMOwzB72+0WSJIEDMMwFUX5"
    "eo7/XAJOp9N8+dlDoDzPQdu2cPnoxyLt4Hq9mgCjoijA8XgMsICpZjCO459NO5/PAAsgCGK1"
    "67vdDg9gGGYVwLIsHgAhXAWUZYkHqKq6ChBFEQ9wXfcwlYFNg6ZpqOs6xAKQLMvCAjRNg8tB"
    "ejnKvu/fb7fbD4+iKBDH8a/4h3G5XNAIm9NXhnVdv5wHjuNQGaHjOFCW5TmTTRRFQdd1Iepu"
    "3/erTUQiSRIIgoDWcON53v2tWxh9A8ITakM7BOFPAAAAAElFTkSuQmCC"
)


"""Toolbar icon for delete button in database table grids."""
ToolbarDelete = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAB3RJTUUH3AEbFAgjYsF/DAAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAAAkSURBVHjaY2AYBcMAMCJz"
    "TvDx/SdGk8WnT3B9TAPtg1EwKAAAoFkEBI8Fc8MAAAAASUVORK5CYII="
)


"""Toolbar icon for filter chat button in chat page."""
ToolbarFilter = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAYVJREFUeNqUUsFKAlEUPa95OqOptHBTiUZtwjCCCYJA0I27"
    "6BPyD4Q+wIVf4ipctGnrxr3Q1hZCBJWrChHR0nFe9zqpM+TieeHO4717znnnvrl4FQJvlPeU"
    "FBeUN7zy/v0v19WYw1wx/wA1KmQflDoodjp2K5t9vBbiBb5YU+sooConXv3ssFa7KjSb6Ha7"
    "KOTz9n6xaPsFCq3WqlYq2c/VapjP5bdXN93hEB+DAZ7KZeyk03D6fT8/UGMs8cy5wEgpXiPu"
    "aITLRgMn7TZCponPVArxen0pcJrL4SiTwXYiAcYSLxJwMKPDXqWCSDSKKR2MaW8lk0sBfsZY"
    "LAa4Lhi7cIBjL2/VhsEc5krlXWC41IqrFHRii/4cIQ34BRwWmM30BAxjJYCFAPXmaApIb7AC"
    "DiQ7mDiOlgA8BzLYAjmYagpshUL/3kCywESzBYOwSwdj6sfiOeAWNAXChFVCmDxDsqfUXkaI"
    "uCHlfIi0HBCWroozl23sjoGvc8u6wwbxQxzm/gowAL873XjUIXE3AAAAAElFTkSuQmCC"
)


"""Toolbar icon for insert button in database table grids."""
ToolbarInsert = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAB3RJTUUH3AEbFAgw5n8+0gAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAAAvSURBVHjaY2AY3oCP78R/"
    "EManholSSwbeAEZ0PxOj6dMnC7g+il2AF4yQWBgGAADm3wucN+5mGAAAAABJRU5ErkJggg=="
)


"""Toolbar icon for maximize chat button in chat page."""
ToolbarMaximize = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAhZJREFUeNqMUr1rU1EU/7373kteW5QOrVJBEVQ0k8MtONSK"
    "LhWpiotQNUJx6ZrRP8DBUdBBcRGMi3+AU6HWRRQz2KUhwcmEQEIeeUljk/u+POf50rxnOvTA"
    "vefe8/E7n1oYhrjxrvUGgMTRqbS1Pr/BD4Mv3/Pk2sqCdN1JS8JPkUEeHzcb4z9fge+XPnz6"
    "feTwmqaVUgBUxkaYCEUZsTBPlgXiL4gXdcM4FEwkP4N+H33HgadU3vO8wqPVs5I5/1nO+kMB"
    "eraNVq0WGVAmec91C/k752W53MPD1XOS/yxnPduxfQqgGwsiZ6UKj+/lyLkL3w9QqfTw4PZF"
    "yXLWJ+1TJYwir9+/LKvVHlwaiUNpNxoNbG/v4ta10weZTPYgjvxkbVE2m/vUswE6nRba7TaE"
    "CDA7a6Fe72Dl6pkoEyRAotZSo5Z5Cq/ebh2MZ2npglT7Po7PWPjy9VfpvzEuEyumxxgEo1J4"
    "jD8ckUU3E8Ik7g6HoDEukmOct0gv0ohooTAcDKK3bU7BnjIBU4erVHSylgWh6+nNjMb4vQhV"
    "24GeuwtxIhcDTKNjUVYZwXsRyVTtJ/5sPsOxK9yCmwmAb8V/cPUdZK8/hTZ/CY6RxV6GsqWA"
    "1H2ErTKGn59jbP9+YhMZbI6MTvnkwPVOmxo0oYH/LCf9STqZiRJi4uLm6Mz4zd2K/fL1WGNX"
    "K3QvcBV09mIe0V8BBgDmbCZA3KkdNgAAAABJRU5ErkJggg=="
)


"""Toolbar icon for message toggle button on search page."""
ToolbarMessage = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAALHRFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjEgbeRydHMgMjAxMyAyMjoyNzo0OSArMDIwMJsvH8IAAAAHdElNRQfdAxUVATP+NUng"
    "AAAACXBIWXMAAAsSAAALEgHS3X78AAAABGdBTUEAALGPC/xhBQAAADlJREFUeNpjYKAQMIKI"
    "pKSk/+RonjdvHiMTpS4YNWDUABBgxCWBnjpBqY4sG0AGkZvUqW8IRQbgAgCLFhLOKKE9wQAA"
    "AABJRU5ErkJggg=="
)


"""Toolbar icon for rollback button in database table grids."""
ToolbarRollback = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAB3RJTUUH3AEaFDUVfy/FTgAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAABHSURBVHjaY2CgEDDCWcL1"
    "/0nS+bYRrJeJUhfQGIC8RcBrjHg1Y/EzcQbgshWLIYxEa8ZhCBNRCnE4nzAgIhApTgejBlAB"
    "AACTkRU1bx46WQAAAABJRU5ErkJggg=="
)


"""Toolbar icon for stats button in chat page."""
ToolbarStats = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAB3RJTUUH3QMZFCYg61pv1AAA"
    "AAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAAHpSURBVHjapZO/ahtBEMa/"
    "XUuyJXAs2Q7YhDSCFEkj8L5ButRpUghjgyEuz4bkIVKJ4DIQCCr8BiZgUqdRQIXShKRQI4Nk"
    "+4Tks+7frmf3VneSkKsMzN3e3Hy/3Z2dZUop/I/l9IMd8WSkfQV1ejqP5DcQo4mIRuTqi0wA"
    "M1anBOfFiRCBO0kiMnkVKmv402hNwc25FaTiiMQfhHA7AxPwe/fmvbpbhNcbo3pcE//O2nMQ"
    "Pic+3RPXP3uIRyG8vyMM318Z12Mdc9t9PD94KXSu0aSAEE7VqYmryy7CYWBcTuJ0aXo8jbu/"
    "+th5WxVak20hBIada8R3sZnJiDwJvp8HAgVWymHij4E8A6/k4Xc9o5kDxDRL5IbprGyVQ35L"
    "vvkhgcrJYmUQQUm2ANBHEknzc6kVCbiRANQK5Uy40WSAgCru3SNeI+ytFQUzgMgKramiTP9n"
    "KyhTUB/9E5s1mgHskK8vrMryuP1o+F/HLf6MeGUkXppJLiGLk6uLuKU1plb6LrDXxAlVnbrO"
    "4acFoeth7LcFvMpY6nPUomkbdCJN9UNaAGP6DgBPqTm24bBPObGsluojiQck7lMXUpsYrQXo"
    "WmyRb9Ne32AT75aexg3OqTbfadQnH5A2mgK43Wlp4X4sM92id+QeaeUD+ILiaqdNXpsAAAAA"
    "SUVORK5CYII="
)


"""Toolbar icon for stop button on search page."""
ToolbarStop = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAIAAACQkWg2AAAALHRFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjEgbeRydHMgMjAxMyAyMTo1ODowNSArMDIwML0/WYwAAAAHdElNRQfdAxUTOiUy3m87"
    "AAAACXBIWXMAAAsSAAALEgHS3X78AAAABGdBTUEAALGPC/xhBQAAAAZ0Uk5TANQA0ADICuWh"
    "ugAAANVJREFUeNpjvHLhBAMpgAWZ83vmwgfTp6OpUMjMZE2Px6IBoto/ORlNw8bp0xUYGOB6"
    "mPCrBgKgIFAKqAChAY9qTD2MQE/fNrDAoxrhtrlzVS+cgPrhz+/fEAbrQpAxv+Pj0djonv75"
    "4weUER7Os3IlRCkQfAkPR7OHCa4Bjt76+0MEgQxkcVQbvn+HmyG1axeEIbxx4zM3NwI2QFTf"
    "t7MDIohmNBtAoQQJVm0LCzxBdPXECUiUg2wAUkAOUIigaoST8OhBVs2AnJZAeoDShBIfI6nJ"
    "GwC7TZxZQGalGAAAAABJRU5ErkJggg=="
)


"""Toolbar icon for inactive stop button on search page."""
ToolbarStopped = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAQAAAC1+jfqAAAALHRFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjEgbeRydHMgMjAxMyAyMTo1ODowNSArMDIwML0/WYwAAAAHdElNRQfdAxYXCxC3jKuY"
    "AAAACXBIWXMAAAsSAAALEgHS3X78AAAABGdBTUEAALGPC/xhBQAAAIZJREFUeNpjYCAAGGGM"
    "9v9rkYSDGSoZURSApGORFCyGK2HEJo2shBG7NEIJUIHJf2zSECVnGFlAjN9gbjEQ9yLREABW"
    "8BPMbGOoAkuCWAjABCJ+QGEdWKgOzsdQ0AQWasKloAvIygNCBiALoQDqTRMsfjgD8yZ2JWcQ"
    "AYVNyRnUoMYXWQQBALXDQEUeuvmAAAAAAElFTkSuQmCC"
)


"""Toolbar icon for tables button on search page."""
ToolbarTables = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAAcZJ"
    "REFUOI2dk81qIkEQx38zjh8T3KsYDyLxc1T0CXLMcV9Ddm8+iA+wz5B3kRCJDoiCeNnTgLDg"
    "qNNdtYfsiMbLJgUF1d3Uv3/9L9oZj8ffM5nMPV8Ia+1vT1Urk8nk11cExuPxD09VHYDn52dU"
    "9Zwics7LdXo+Go1QVcdNksQVEQCCIACg0+lc1e12+6YWEZIkcT1rraOqAIRhSLFYZLfbISKU"
    "y2V2ux2qelOrKtZaxzPGuKlAEAREUcTj4yMAi8WCbrcLwOvrK8PhEIDpdIqqYox5J0ifEIYh"
    "pVKJ5XKJiOD7Pm9vb6gqd3d3vLy8kF4mIlhrHdcYc+NBvV6n2WwSxzFBENDtdtnv9wwGgzOF"
    "iLwTXAqkBOv1GhGhUCgQhiGqiu/7zGazKwJjjOsmSeJ8JKjVajw8PHA4HGi1WrTbbeI4ptfr"
    "0e/3L6dwbWJKsNlsEBFyuRzL5RJVJZ/PM5/PzwSpia619moKANVqlVqtxul0otFo0Gw2OR6P"
    "Zz9SAWut68VxnPnowXa7Jd1brVZcEl56EMdxxouiKJ/NZnl6evrvPwCQzWaJoijvVCqVn7lc"
    "rvqp7n9xOp22DuAD3wDvk/0G+PMX80pIbz0EU9oAAAAASUVORK5CYII="
)


"""Toolbar icon for tabs toggle button on search page."""
ToolbarTabs = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAARnQU1B"
    "AACxjwv8YQUAAAAJcEhZcwAADsMAAA7DAcdvqGQAAAAadEVYdFNvZnR3YXJlAFBhaW50Lk5F"
    "VCB2My41LjEwMPRyoQAAASNJREFUOE+d0b1qg1AYxvF3Chkyd+gUehfF1dHF0VJwMm3sh0IK"
    "RRA0d+TuBXgBvYIgKH4hRiIxGk99pEuhkJwe+OPrCz+OIDmOc+e6rjI+Vwjzdru9v2YHS7Zt"
    "PyRJ8sV+DmbTNFfX7GDJsqynYRjY+Xyewuz7/vS8tIOlzWbz3Pc967puCrPneSwIgl/9tYMl"
    "wzDWp9OJHY/HqbZtGd4vhctgSdf1NeDhcOAKBpY0TdObpmH7/Z4rGFhSVfWlrmtWliVXMLCk"
    "KMprVVUsz3OuYGBJluU33D7+d65gYEmSpPeiKFgURVzBwJIoikaapiwMQ65gYEkQBBOfv9vt"
    "uIKBpeVy+ZFlGYvjmCsYWJrP54+LxeLzP8HSeGZjN2O3nMHMvgFTmvUJKTInOAAAAABJRU5E"
    "rkJggg=="
)


"""Toolbar icon for title toggle button on search page."""
ToolbarTitle = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAALHRFWHRDcmVhdGlvbiBUaW1l"
    "AE4gMjEgbeRydHMgMjAxMyAyMjowNjo0MSArMDIwMB9ug1oAAAAHdElNRQfdAxUVAhfpG/7y"
    "AAAACXBIWXMAAAsSAAALEgHS3X78AAAABGdBTUEAALGPC/xhBQAAAEJJREFUeNpjYBhowAhj"
    "JCUl/SdF47x588B6WXBJIBuKTQwGmHBpJmQzUQBkGyGvMRFr2KgBw9oArKkKV+IhKRUSCwAF"
    "qxgZvjJU7QAAAABJRU5ErkJggg=="
)
